if exists (select 1 from sysobjects where name = 'chk_abteilungverplant' and type = 'P')
begin
   drop procedure chk_abteilungverplant
   print 'Procedure: chk_abteilungverplant deleted ...'
end
go
create procedure chk_abteilungverplant(
   @monatID        int  = 1,
   @schemaID       int  = 1,
   @jahrID         int  = 1
)
as
begin
  set nocount on
   
  select amount = sum(isnull(ps.AbteilungID,0)) from ArbeitsPlan ap 
    inner join PlanSchema ps
      on ps.SchemaID = ap.SchemaID
     and ps.AbteilungID in (select distinct 
                                   abteilungid = ps.AbteilungID
                              from PlanSchema ps 
                             where ps.SchemaID = @schemaID)
   where ap.ATID = @monatID
     and ap.JahrID = @jahrID

end
go
print 'Procedure: chk_abteilungverplant done ...'
go
grant exec on chk_abteilungverplant to prsadmins with grant option
go
grant exec on chk_abteilungverplant to prsusers
go


if exists (select 1 from sysobjects where name = 'chk_dienstplan' and type = 'P')
begin
   drop procedure chk_dienstplan
   print 'Procedure: chk_dienstplan deleted ...'
end
go
create procedure chk_dienstplan(
   @prsID          int  = 1,
   @datum          char(8) = '20070501',
   @planID         int  = 1
)
as
begin
  set nocount on
   declare @exists int
   declare @dienstid int

   select @exists = 0
   
   if exists (select 1 from Dienstplan where PrsID = @prsID and Datum = @datum and PlanID not in (@planID))
    begin
       select @exists = 1 -- bereits belegt in einem anderen Plan
    end

--   if exists (select 1 from dienstplan where prsid = @prsid and datum = @datum and dienstid = 2)
--    begin
--       select @exists = 2 -- Ferien
--    end

   select 'exists' = @exists

end
go
print 'Procedure: chk_dienstplan done ...'
go
grant exec on chk_dienstplan to prsadmins with grant option
go
grant exec on chk_dienstplan to prsusers
go


-- TODO: Check if needed
if exists (select 1 from sysobjects where name = 'chk_planabteilung' and type = 'P')
begin
   drop procedure chk_planabteilung
   print 'View: chk_planabteilung deleted ...'
end
go
create procedure chk_planabteilung(
  @atid       int = 1,
  @planid     int = 1
)
as
begin
  set nocount on

   select distinct a.AbteilungID from ArbeitsPlan ap 
    inner join Schemas s
       on s.SchemaID = ap.SchemaID 
    inner join PlanSchema ps 
       on ps.SchemaID = s.SchemaID
    inner join Abteilung a 
       on a.AbteilungID = ps.AbteilungID
    where ap.ATID = @atid
      and ap.PlanID = @planid

end
go
print 'Procedure: chk_planabteilung done ...'
go

grant exec on chk_planabteilung to prsadmins with grant option
go
grant exec on chk_planabteilung to prsusers
go


if exists (select 1 from sysobjects where name = 'del_abteilung' and type = 'P')
begin
   drop procedure del_abteilung
   print 'Procedure: del_abteilung deleted ...'
end
go

create procedure del_abteilung(
  @abteilungid          int   = 1
)
as
begin

  set nocount on

  declare @beschrid int 
   select @beschrid = BeschrID
     from Abteilung
    where AbteilungID = @abteilungid

  delete from Abteilung where AbteilungID = @abteilungid

  exec del_beschreibung @beschrid

end
go

print 'Procedure: del_abteilung done ...'
go

grant exec on del_abteilung to prsadmins with grant option
go
grant exec on del_abteilung to prsusers
go


if exists (select 1 from sysobjects where name = 'del_anstellung' and type = 'P')
begin
   drop procedure del_anstellung
   print 'Procedure: del_anstellung deleted ...'
end
go
create procedure del_anstellung(
  @anstellungid          int   = 1
)
as
begin
  set nocount on
  declare @beschrid int
   select @beschrid = BeschrID
     from Anstellung
    where AnstellungID = @anstellungid
   delete from Anstellung where BeschrID = @beschrid
     exec del_beschreibung @beschrid

end
go
print 'Procedure: del_anstellung done ...'
go
grant exec on del_anstellung to prsadmins with grant option
go
grant exec on del_anstellung to prsusers
go


if exists (select 1 from sysobjects where name = 'del_beschreibung' and type = 'P')
begin
   drop procedure del_beschreibung
   print 'Procedure: del_beschreibung deleted ...'
end
go
create procedure del_beschreibung(
  @beschrid          int           = 6
)
as
begin
  set nocount on
  delete from Beschreibung where BeschrID = @beschrid
end
go
print 'Procedure: del_beschreibung done ...'
go
grant exec on del_beschreibung to prsadmins with grant option
go
grant exec on del_beschreibung to prsusers
go


if exists (select 1 from sysobjects where name = 'del_dienst' and type = 'P')
begin
   drop procedure del_dienst
   print 'Procedure: del_dienst deleted ...'
end
go
create procedure del_dienst(
  @diesntid          int   = 1
)
as
begin
  set nocount on
  
  if exists (select * from Dienstplan where DienstID = @diesntid)
  begin
	  update Dienst
		 set Status = 1  -- 0:Aktiv 1:Nicht Aktiv
	   where DienstID = @diesntid
   end
   else
   begin
	   delete from Dienst where DienstID = @diesntid
   end

end
go
print 'Procedure: del_dienst done ...'
go
grant exec on del_dienst to prsadmins with grant option
go
grant exec on del_dienst to prsusers
go


if exists (select 1 from sysobjects where name = 'del_pensum' and type = 'P')
begin
   drop procedure del_pensum
   print 'Procedure: del_pensum deleted ...'
end
go
create procedure del_pensum(
  @prsid          int   = 1,
  @von            char(8),
  @bis            char(8),
  @pensum         int
)
as
begin
  set nocount on
  
  delete from Arbeitspensum 
   where PrsID = @prsid 
     and Von = @von
     and Bis = @bis
     and Prozent = @pensum

end
go
print 'Procedure: del_pensum done ...'
go
grant exec on del_pensum to prsadmins with grant option
go
grant exec on del_pensum to prsusers
go


if exists (select 1 from sysobjects where name = 'del_Person' and type = 'P')
begin
   drop procedure del_Person
   print 'Procedure: del_Person deleted ...'
end
go
create procedure del_Person(
  @prsid          int   = 1
)
as
begin
  set nocount on
  delete from PersonalAbteilung where PersonalAbteilung.PrsId = @prsid
  delete from Arbeitspensum where Arbeitspensum.PrsID = @prsid
  delete from PersonAnstellung where PersonAnstellung.PrsID = @prsid
  delete from Arbeitszeit where Arbeitszeit.PrsID = @prsid
  delete from PersonZeiten where PersonZeiten.PrsID = @prsid
  delete from Ferien where Ferien.PrsId = @prsid
  delete from Dienstplan where Dienstplan.PrsID = @prsid
  delete from Colorplan where Colorplan.PrsID = @prsid
  delete from Saldoanpassung where PrsID = @prsid
  
  delete from Person where Person.PrsId = @prsid
  
end
go
print 'Procedure: del_Person done ...'
go
grant exec on del_Person to prsadmins with grant option
go
grant exec on del_Person to prsusers
go



if exists (select 1 from sysobjects where name = 'del_plan' and type = 'P')
begin
   drop procedure del_plan
   print 'Procedure: del_plan deleted ...'
end
go
create procedure del_plan(
  @planid             int           = 1
)
as
begin
  set nocount on

  
  delete from Dienstplan where PlanID = @planid
  delete from Colorplan where PlanID = @planid
  delete from Arbeitszeit where PlanID = @planid
  delete from Terminplan where PlanID = @planid
  delete from Saldoanpassung where PlanID = @planid
  delete from ArbeitsPlanzprs where planID = @planid
  delete from ArbeitsPlan where PlanID = @planid

end
go
print 'Procedure: del_plan done ...'
go
grant exec on del_plan to prsadmins with grant option
go
grant exec on del_plan to prsusers
go



if exists (select 1 from sysobjects where name = 'del_prsabteilung' and type = 'P')
begin
   drop procedure del_prsabteilung
   print 'Procedure: del_prsabteilung deleted ...'
end
go
create procedure del_prsabteilung(
  @abteilungid      int   = 1,
  @prsid            int   = 1,
  @von             char(8) = '20070501',
  @bis             char(8) = '20090501'
)
as
begin
  set nocount on

   delete from PersonalAbteilung 
    where PrsId = @prsid
      and AbteilungID = @abteilungid
      and Von = @von
      and Bis = @bis

end
go
print 'Procedure: del_prsabteilung done ...'
go
grant exec on del_prsabteilung to prsadmins with grant option
go
grant exec on del_prsabteilung to prsusers
go



if exists (select 1 from sysobjects where name = 'del_schema' and type = 'P')
begin
   drop procedure del_schema
   print 'Procedure: del_schema deleted ...'
end
go
create procedure del_schema(
  @schemaid           int           = 1
)
as
begin
  set nocount on

  delete from Schemas where SchemaID = @schemaid
  delete from PlanSchema where SchemaID = @schemaid
--select * from schemas
--select * from PlanSchema

end
go
print 'Procedure: del_schema done ...'
go
grant exec on del_schema to prsadmins with grant option
go
grant exec on del_schema to prsusers
go




if exists (select 1 from sysobjects where name = 'get_abteilungen' and type = 'P')
begin
   drop procedure get_abteilungen
   print 'Procedure: get_abteilungen deleted ...'
end
go
create procedure get_abteilungen(
  @mandid          char(2)           = 'li',
  @abteilungid     int               = 0
)
as
begin
  set nocount on
         
  select abteilungid = AbteilungID,
         name = a.Name,
         beschreibung = b.Langtext,
         einsatzprotag = stundenprotag
    from Abteilung a
   inner join Beschreibung b 
      on b.BeschrID = a.BeschrID
   where (mandid = @mandid or @mandid = '')
     and (AbteilungID = @abteilungid or @abteilungid = 0)
   order by AbteilungID

end
go
print 'Procedure: get_abteilungen done ...'
go
grant exec on get_abteilungen to prsadmins with grant option
go
grant exec on get_abteilungen to prsusers
go



if exists (select 1 from sysobjects where name = 'get_abteilungeninschema' and type = 'P')
begin
   drop procedure get_abteilungeninschema
   print 'Procedure: get_abteilungeninschema deleted ...'
end
go
create procedure get_abteilungeninschema(
  @schemaid          int           = 25
)
as
begin
  set nocount on   
   
  select abteilungid = a.AbteilungID,
         name = a.Name
    from Schemas s
   inner join PlanSchema ps
      on ps.SchemaID = s.SchemaID
   inner join Abteilung a
      on a.AbteilungID = ps.AbteilungID
   where s.SchemaID = @schemaid
     and ps.PrsID = 0

end
go
print 'Procedure: get_abteilungeninschema done ...'
go
grant exec on get_abteilungeninschema to prsadmins with grant option
go
grant exec on get_abteilungeninschema to prsusers
go



if exists (select 1 from sysobjects where name = 'get_activemonths' and type = 'P')
begin
   drop procedure get_activemonths
   print 'Procedure: get_activemonths deleted ...'
end
go
create procedure get_activemonths(
  @mandid             char(2)           = 'li'
)
as
begin
  set nocount on

  select jahrid = pj.JahrID,
         jahr = Jahr,
         name = Name,
         von = at.Von,
         bis = at.Bis,
         atid = ATID,
         tage = Tage,
         typwert = typwert
    from PlanungsJahr pj
   inner join ArbeitsTage at
      on at.JahrID = pj.JahrID
   where pj.mandid = @mandid
     and pj.Status = 1
     and at.typ = 1

end
go
print 'Procedure: get_activemonths done ...'
go
grant exec on get_activemonths to prsadmins with grant option
go
grant exec on get_activemonths to prsusers
go



if exists (select 1 from sysobjects where name = 'get_anstellung' and type = 'P')
begin
   drop procedure get_anstellung
   print 'Procedure: get_anstellung deleted ...'
end
go
create procedure get_anstellung(
  @mandid          char(2)           = 'li'
)
as
begin
  set nocount on
         
  select anstellungid = AnstellungID, 
         name = Name, 
         prioritaet = Prioritaet, 
         einsatzprotag = EinsatzProTag, 
         gerechnet = Gerechnet,
         ausgelernt = ausgelernt,
         beschreibung = b.Langtext
    from Anstellung a
    inner join Beschreibung b 
      on b.BeschrID = a.BeschrID
   where (mandid = @mandid or @mandid = '')
   order by Gerechnet

end
go
print 'Procedure: get_anstellung done ...'
go
grant exec on get_anstellung to prsadmins with grant option
go
grant exec on get_anstellung to prsusers
go



if exists (select 1 from sysobjects where name = 'get_arbeitspensum' and type = 'P')
begin
   drop procedure get_arbeitspensum
   print 'Procedure: get_arbeitspensum deleted ...'
end
go
create procedure get_arbeitspensum(
  @prsid              int           = 7,
  @abteilungid        int           = 0
)
as
begin
  set nocount on
  
  select abteilungid = PersonalAbteilung.AbteilungID,
         pensum = Prozent,
         von    = Von,
         bis    = Bis,
         a.Name    
    from PersonalAbteilung 
   inner join Abteilung a
      on a.AbteilungID = PersonalAbteilung.AbteilungID
   where PrsId = @prsid 
     and (PersonalAbteilung.AbteilungID = @abteilungid or @abteilungid = 0)
     and getdate() >= Von 
     and getdate() <= Bis

  

end
go
print 'Procedure: get_arbeitspensum done ...'
go
grant exec on get_arbeitspensum to prsadmins with grant option
go
grant exec on get_arbeitspensum to prsusers
go


if exists (select 1 from sysobjects where name = 'get_arbeitsplandata' and type = 'P')
begin
   drop procedure get_arbeitsplandata
   print 'Procedure: get_arbeitsplandata deleted ...'
end
go
create procedure get_arbeitsplandata(
  @planid     int = 1
)
as
begin
  set nocount on

  create table #plan
  (
         prsid             int,
         prsschemaid       int,
         nachname          varchar(32),
         vorname           varchar(32),
         prozent           int,
         abteilungid       int,
         rgb               int,
         colorname         varchar(32),
         abteilungname     varchar(32),
         abteilungbeschrid int,
         persontyp         int
  )

create table #abtschema
(
     abteilungid int,
     prsschemaid int
)

  insert #plan
  select prsid             = isnull(ps.PrsID,0),--isnull(p.prsid,0),
         prsschemaid       = PrsSchemaID,
         nachname          = isnull(p.Nachname,''),
         vorname           = isnull(p.Vorname, ''),
         prozent           = isnull(nullif(pa.Prozent,0) ,isnull(a.Prozent,0)),--isnull(a.prozent,0),--
         abteilungid       = ps.AbteilungID,
         rgb               = ps.RGB,
         colorname         = ps.ColorName,
         abteilungname     = abt.Name,
         abteilungbeschrid = abt.BeschrID,
         persontyp         = isnull(p.PersonTyp,1) --1 --Stammperson

    from ArbeitsPlan ap
   inner join Schemas s
      on ap.SchemaID = s.SchemaID
   inner join PlanSchema ps
      on ps.SchemaID = s.SchemaID
    left outer join Person p
      on p.PrsId = ps.PrsID
     and p.PersonTyp = 1
    left outer join Arbeitspensum a 
      on a.PrsID = p.PrsId 
--     and a.status = 1
     and (a.Von <= ap.Von and a.Bis >= ap.Bis)
--(select max(ab.bis) from arbeitspensum ab where ab.prsid = p.prsid)
    left outer join PersonalAbteilung pa
      on pa.PrsId = p.PrsId
     and pa.AbteilungID = ps.AbteilungID
     and pa.Bis = (select max(pa2.Bis) from PersonalAbteilung pa2 where pa2.PrsId = p.PrsId and pa.AbteilungID = ps.AbteilungID)
     and (pa.Von <= ap.Von and pa.Bis >= ap.Bis)
    left outer join Abteilung abt
      on abt.AbteilungID = ps.AbteilungID
   where ap.PlanID = @planid
   order by prsschemaid

  insert #plan
  select prsid             = isnull(p.PrsId,0),
         prsschemaid       = 0,--noch machen...
         nachname          = isnull(p.Nachname,''),
         vorname           = isnull(p.Vorname, ''),
         prozent           = 0,
         abteilungid       = apz.abteilungID,
         rgb               = -1,
         colorname         = 'White',
         abteilungname     = abt.Name,
         abteilungbeschrid = abt.BeschrID,
         persontyp         = p.PersonTyp--2 --Zusatzperson
    from ArbeitsPlan ap
   inner join ArbeitsPlanzprs apz
      on apz.planID = ap.PlanID
--    left outer join zusatzperson zp
--      on zp.zprsid = apz.zprsid
    left outer join Person p
      on p.PrsId = apz.zprsID
     and p.PersonTyp = 2
    left outer join Abteilung abt
      on abt.AbteilungID = apz.abteilungID
   where ap.PlanID = @planid

 declare @prsschemaid int
  select @prsschemaid = max(prsschemaid) from #plan

  update #plan 
     set prsschemaid = @prsschemaid +1 
   where persontyp = 2

  select * from #plan
   order by abteilungid, prsschemaid, prsid

end
go
print 'Procedure: get_arbeitsplandata done ...'
go

grant exec on get_arbeitsplandata to prsadmins with grant option
go
grant exec on get_arbeitsplandata to prsusers
go



if exists (select 1 from sysobjects where name = 'get_dienstlisting')
   begin
     drop Procedure get_dienstlisting
     print 'Procedure: get_dienstlisting deleted ...'
   end
go

create procedure get_dienstlisting 
(
		@mandid     char(2) = 'li',
		@dienstid   int     = 0
)
as
begin

	select Anzeigename = d.Name + ' - ' + b.Kurztext,
	       DienstID    = d.DienstID,
	       Name        = d.Name,
	       Kuzrtext    = b.Kurztext,
	       RGB         = RGB
	  from Dienst d
	 left outer join Dienstplan dp
	    on dp.DienstID = d.DienstID
	 inner join Beschreibung b
	    on b.BeschrID = d.BeschrID
	 where d.mandid = @mandid
	   and (d.DienstID = @dienstid or @dienstid = 0)
	   and d.Status = 0
	 group by d.DienstID, d.Name, b.Kurztext, RGB
	 order by count(dp.DienstID) desc


end
go

print 'Procedure: get_dienstlisting done ...'
go
grant exec on get_dienstlisting to prsadmins with grant option
go
grant exec on get_dienstlisting to prsusers
go


if exists (select 1 from sysobjects where name = 'get_dienstnotinschema' and type = 'P')
begin
   drop procedure get_dienstnotinschema
   print 'Procedure: get_dienstnotinschema deleted ...'
end
go
create procedure get_dienstnotinschema(
  @schemaid     int = 15,
  @abteilungid  int = 1
)
as
begin
  set nocount on

  if @abteilungid = 0 select @abteilungid = 1

  select dienstid = d.DienstID, 
         gerechnet = 0, 
         einsatzprotag = 0, 
         dienstname = d.Name, 
         kurztext = b.Kurztext,
         langtext = b.Langtext,
         abteilungid = 0,
         abteilungname = ''
    from Dienst d
   inner join Beschreibung b
      on d.BeschrID = b.BeschrID
   where d.DienstID not in (select ds.DienstID
                              from DienstSchema ds
                             where ds.SchemaID = @schemaid 
                               and (ds.AbteilungID = @abteilungid))
    and d.Status = 0
                              

end
go
print 'Procedure: get_dienstnotinschema done ...'
go

grant exec on get_dienstnotinschema to prsadmins with grant option
go
grant exec on get_dienstnotinschema to prsusers
go


if exists (select 1 from sysobjects where name = 'get_dienstschema' and type = 'P')
begin
   drop procedure get_dienstschema
   print 'Procedure: get_dienstschema deleted ...'
end
go
create procedure get_dienstschema(
  @schemaid     int = 1,
  @abteilungid  int = 1
)
as
begin
  set nocount on

  if @abteilungid = 0 select @abteilungid = null

  select dienstid = ds.DienstID, 
         gerechnet = ds.Gerechnet, 
         einsatzprotag = ds.EinsatzProTag, 
         dienstname = d.Name, 
         kurztext = b.Kurztext,
         langtext = b.Langtext,
         abteilungid = a.AbteilungID,
         abteilungname = a.Name
  from DienstSchema ds 
  left outer join Dienst d
    on ds.DienstID = d.DienstID
  left outer join Beschreibung b
    on b.BeschrID = d.BeschrID
  left outer join Abteilung a
    on a.AbteilungID = ds.AbteilungID
 where ds.SchemaID = @schemaid 
   and (ds.AbteilungID = @abteilungid or @abteilungid is null)

end
go
print 'Procedure: get_dienstschema done ...'
go

grant exec on get_dienstschema to prsadmins with grant option
go
grant exec on get_dienstschema to prsusers
go



if exists (select 1 from sysobjects where name = 'get_dienstzeiten' and type = 'P')
begin
   drop procedure get_dienstzeiten
   print 'Procedure: get_dienstzeiten deleted ...'
end
go
create procedure get_dienstzeiten(
  @dienstid             int               = 2
)
as
begin
  set nocount on

  select dienstzeitid = DienstZeitID,
         dienstid = DienstID,
         von = CONVERT(CHAR(5), Von, 14),
         bis = CONVERT(CHAR(5), Bis, 14),
         typ = case when Zeiteinheit = 'P'
                    then 'Pause'
                    else 'Arbeit'
               end,
         datumvon = datvon,
         datumbis = datbis,
         abteilungszeit = abteilungzeit
    from DienstZeit 
   where DienstID = @dienstid 

end
go
print 'Procedure: get_dienstzeiten done ...'
go
grant exec on get_dienstzeiten to prsadmins with grant option
go
grant exec on get_dienstzeiten to prsusers
go



if exists (select 1 from sysobjects where name = 'get_personanstellung' and type = 'P')
begin
   drop procedure get_personanstellung
   print 'Procedure: get_personanstellung deleted ...'
end
go
create procedure get_personanstellung(
  @prsid          int           = 3
)
as
begin
  set nocount on
         
  select anstellungid = a.AnstellungID, 
         name = Name, 
         prioritaet = Prioritaet, 
         einsatzprotag = EinsatzProTag, 
         gerechnet = Gerechnet,
         ausgelernt = ausgelernt
    from Anstellung a
   inner join PersonAnstellung pa 
      on pa.AnstellungID = a.AnstellungID
   where Von <= GETDATE() 
     and Bis >= GETDATE() 
     and PrsID = @prsid
   order by Gerechnet

end
go
print 'Procedure: get_personanstellung done ...'
go
grant exec on get_personanstellung to prsadmins with grant option
go
grant exec on get_personanstellung to prsusers
go



if exists (select 1 from sysobjects where name = 'get_personcanceldinschema' and type = 'P')
begin
   drop procedure get_personcanceldinschema
   print 'Procedure: get_personcanceldinschema deleted ...'
end
go
create procedure get_personcanceldinschema(
  @schemaid int = 1
)
as
begin
  set nocount on

  select p.PrsId,
         a.AbteilungID,
         p.Vorname,
         p.Nachname,
         a.Name
    from Abteilung a
    left outer join PersonalAbteilung pa
      on a.AbteilungID = pa.AbteilungID
    left outer join Person p
     on p.PrsId = pa.PrsId
    and p.PersonTyp = 1
   left outer join PlanSchema ps
     on ps.AbteilungID = a.AbteilungID
    and ps.PrsID = p.PrsId
  inner join PersonAnstellung pan on pan.PrsID = ps.PrsID 
  where pan.Bis <= getdate()
    and ps.SchemaID = @schemaid
and pa.Bis = (select max(pa2.Bis) from PersonalAbteilung pa2 where pa2.PrsId = p.PrsId and a.AbteilungID = pa2.AbteilungID)

end
go
print 'Procedure: get_personcanceldinschema done ...'
go

grant exec on get_personcanceldinschema to prsadmins with grant option
go
grant exec on get_personcanceldinschema to prsusers
go


if exists (select 1 from sysobjects where name = 'get_persondiversrow' and type = 'P')
begin
   drop procedure get_persondiversrow
   print 'Procedure: get_persondiversrow deleted ...'
end
go
create procedure get_persondiversrow(
  @prsid      int = 1,
  @von        varchar(8) ='00000000',
  @bis        varchar(8) ='00000000',
  @jahr       int = 2008
)
as
begin
  set nocount on

  declare @dtVon datetime
  declare @dtBis datetime
  select @dtVon = convert(datetime, @von, 112)
  select @dtBis = convert(datetime, @bis, 112)

  select p.Vorname,
         p.Nachname,
         ap.Name,
         sa.Datum,
         sa.Stunden,
         sa.Grund
    from Saldoanpassung sa
    left outer join ArbeitsPlan ap
      on ap.PlanID = sa.PlanID
    left outer join Person p
      on p.PrsId = sa.PrsID
   where sa.PrsID = @prsid
     and sa.Datum between @dtVon and @dtBis

end
go
print 'Procedure: get_persondiversrow done ...'
go

grant exec on get_persondiversrow to prsadmins with grant option
go
grant exec on get_persondiversrow to prsusers
go


if exists (select 1 from sysobjects where name = 'get_personen' and type = 'P')
begin
   drop procedure get_personen
   print 'Procedure: get_personen deleted ...'
end
go
create procedure get_personen(
  @mandid            char(2) = 'li',
  @prsid             int      = 0,
  @typ               int      = 1 --0: alle, 1:Stammpersonen, 2:Zusatzpersonen
)
as
begin
  set nocount on
  
  create table #prs
  (
       prsid     int,
       persontyp int
  )
  
  if @typ = 0
  begin
  
  insert #prs
  select prsid = PrsId,
         persontyp = PersonTyp
    from Person
   where PrsId = @prsid or @prsid = 0
  
  -- define nicht angestellte
  update #prs
     set persontyp = 3
    from #prs p
   inner join Arbeitspensum a
      on a.PrsID = p.prsid
     and a.Bis = (select max(a2.Bis) from Arbeitspensum a2 where a.PrsID = a2.PrsID)
   where (a.Bis <= GETDATE() or a.Von > GETDATE())
      
  
  
  end
  else if @typ = 1
  begin
  
  insert #prs
  select distinct
         prsid = Arbeitspensum.PrsID,
         persontyp = PersonTyp
    from Arbeitspensum, Person
   where (Arbeitspensum.PrsID = @prsid or @prsid = 0)
     --and Von <= getdate() 
     and Bis >= GETDATE()
     and Person.PrsId = Arbeitspensum.PrsID
     and Person.PersonTyp = 1
  
  end
  else if @typ = 2
  begin
  
  insert #prs
  select prsid = PrsId,
         persontyp = PersonTyp
    from Person
   where PersonTyp = 2
   
  end
  
  select prsid = p.PrsId,
         vorname = p.Vorname,
         nachname = p.Nachname,
         strasse = isnull(p.Strasse,''),
         stnr = isnull(p.Strassennr,0),
         postfach = isnull(p.Postfach,''),
         plz = isnull(p.PLZ,''),
         ort = isnull(p.Ort,''),
         telp = isnull(p.TelnrP,''),
         teln = isnull(p.TelnrN,''),
         email = isnull(p.Email,''),
         typ = #prs.persontyp,
         anzeigename = p.Vorname + ' ' + p.Nachname
    from Person p
   inner join #prs
      on p.PrsId = #prs.prsid
   where (p.mandid = @mandid or @mandid = '')
     and (p.PrsId = @prsid or @prsid = 0)
     and (p.PersonTyp = @typ or @typ = 0)
     
     
     --select * from arbeitspensum,person where person.prsid = arbeitspensum.prsid
     --select * from person

end
go
print 'Procedure: get_personen done ...'
go

grant exec on get_personen to prsadmins with grant option
go
grant exec on get_personen to prsusers
go


if exists (select 1 from sysobjects where name = 'get_personeninabteilung' and type = 'P')
begin
   drop procedure get_personeninabteilung
   print 'Procedure: get_personeninabteilung deleted ...'
end
go
create procedure get_personeninabteilung(
  @abteilungid            int = 1
)
as
begin
  set nocount on

   create table #prs(
       schemaname    varchar(32),
       schemaid      int,
       prsschemaid   int,
       prsid         int,
       vorname       varchar(32),
       nachname      varchar(32),
       abteilungid   int,
       abteilungname varchar(32),
       rgb           int
   )
      
   insert #prs
   select 
          schmaname     = '',
          schemaid      = 0,
          prsschemaid   = 0,
          prsid         = 0,
          vorname       = Name,
          nachname      = '',
          abteilungid   = AbteilungID,
          abteilungname = Name,
          rgb           = -1
     from Abteilung
    where Abteilung.AbteilungID = @abteilungid
    
   insert #prs
   select schemaname    = '',
          schemaid      = 0,
          prsschemaid   = 1,
          prsid         = isnull(p.PrsId,0),
          vorname       = isnull(Vorname,''),
          nachname      = isnull(Nachname,''),
          abteilungid   = a.AbteilungID,
          abteilungname = a.Name,
          rgb           = -1
      from Abteilung a
     inner join PersonalAbteilung pa
        on pa.AbteilungID = a.AbteilungID
     inner join Person p
        on p.PrsId = pa.PrsId
     
     where a.AbteilungID = @abteilungid
       and pa.Von <= GETDATE()
       and pa.Bis >= GETDATE()
       and p.PersonTyp = 1

   select * 
     from #prs
    order by abteilungid, prsschemaid, prsid

end
go
print 'Procedure: get_personeninabteilung done ...'
go

grant exec on get_personeninabteilung to prsadmins with grant option
go
grant exec on get_personeninabteilung to prsusers
go



if exists (select 1 from sysobjects where name = 'get_personeninabteilunginschema' and type = 'P')
begin
   drop procedure get_personeninabteilunginschema
   print 'Procedure: get_personeninabteilunginschema deleted ...'
end
go
create procedure get_personeninabteilunginschema(
  @abteilungid       int           = 1,
  @schemaid          int           = 25
)
as
begin
  set nocount on   
   
  select prsid = p.PrsId,
         anzeigename = p.Vorname + ' ' + p.Nachname,
         vorname = p.Vorname,
         nachname = p.Nachname
    from Schemas s
   inner join PlanSchema ps
      on ps.SchemaID = s.SchemaID
   inner join Abteilung a
      on a.AbteilungID = ps.AbteilungID
   inner join Person p
      on p.PrsId = ps.PrsID
   where s.SchemaID = @schemaid
     and ps.AbteilungID = @abteilungid
     and ps.PrsID <> 0
   order by PrsSchemaID

end
go
print 'Procedure: get_personeninabteilunginschema done ...'
go
grant exec on get_personeninabteilunginschema to prsadmins with grant option
go
grant exec on get_personeninabteilunginschema to prsusers
go


if exists (select 1 from sysobjects where name = 'get_personeninplan' and type = 'P')
begin
   drop procedure get_personeninplan
   print 'Procedure: get_personeninplan deleted ...'
end
go
create procedure get_personeninplan(
  @planid            int = 55
)
as
begin
  set nocount on

   create table #prs(
       prsID         int,
       name          varchar(32),
       nachname      varchar(32),
       abteilungid   int,
       abteilungname varchar(32),
       prsschemaid   int,
       von           datetime,
       bis           datetime,
       rgb           int,
       pensum        int,
       colorname     varchar(32)
   )

   insert #prs
   select prsID         = isnull(ps.PrsID,0),
          name          = isnull(Vorname,''),
          nachname      = isnull(Nachname,''),
          abteilungid   = a.AbteilungID,
          abteilungname = a.Name,
          prsschemaid   = PrsSchemaID,
          von           = ap.Von,
          bis           = ap.Bis,
          rgb           = ps.RGB,
          pensum        = isnull(Prozent,0),
          colorname     = ColorName
     from ArbeitsPlan ap
    inner join PlanSchema ps
       on ps.SchemaID = ap.SchemaID
     left outer join Person p
       on p.PrsId = ps.PrsID
    inner join Abteilung a
       on a.AbteilungID = ps.AbteilungID
     left outer join PersonalAbteilung pab
       on pab.PrsId = p.PrsId
      and pab.AbteilungID = a.AbteilungID
      --and (pab.Status = 1 or pab.Status is null)
      and ap.Von between pab.Von and pab.Bis
      and ap.Bis between pab.Von and pab.Bis
    where ap.PlanID = @planid
    order by prsschemaid


   insert #prs
   select prsID         = p.PrsId,
          name          = Vorname,
          nachname      = Nachname,
          abteilungid   = a.AbteilungID,
          abteilungname = a.Name,
          prsschemaid   = 100,
          von           = ap2.Von,
          bis           = ap2.Bis,
          rgb           = 0,
          pensum        = 100,
          colorname     = ''
     from ArbeitsPlanzprs ap
     left outer join Person p
       on p.PrsId = ap.zprsID
    inner join Abteilung a
       on a.AbteilungID = ap.abteilungID
    inner join ArbeitsPlan ap2
       on ap2.PlanID = ap.planID
    where ap.planID = @planid

   update #prs
      set name = a.Name
     from #prs
    inner join Abteilung a
       on a.AbteilungID = #prs.abteilungid
    where #prs.prsID = 0

   select * 
     from #prs
    order by abteilungid, prsschemaid, prsID

end
go
print 'Procedure: get_personeninplan done ...'
go

grant exec on get_personeninplan to prsadmins with grant option
go
grant exec on get_personeninplan to prsusers
go


if exists (select 1 from sysobjects where name = 'get_personeninschema' and type = 'P')
begin
   drop procedure get_personeninschema
   print 'Procedure: get_personeninschema deleted ...'
end
go
create procedure get_personeninschema(
  @schemaid            int = 9
)
as
begin
  set nocount on

   create table #prs(
       prsid         int,
       name          varchar(32),
       nachname      varchar(32),
       abteilungid   int,
       abteilungname varchar(32),
       prsschemaid   int,
       --von           datetime,
       --bis           datetime,
       rgb           int,
       --pensum        int
       vorname       varchar(32)
   )
    
   insert #prs
   select prsid         = isnull(ps.PrsID,0),
          name          = isnull(Vorname,''),
          nachname      = isnull(Nachname,''),
          abteilungid   = a.AbteilungID,
          abteilungname = a.Name,
          prsschemaid   = PrsSchemaID,
          --von           = ap.Von,
          --bis           = ap.Bis,
          rgb           = ps.RGB,
          --pensum        = isnull(Prozent,0)
          vorname       = isnull(Vorname,'')
      from PlanSchema ps
     inner join Person p
        on p.PrsId = ps.PrsID
     inner join Abteilung a
        on a.AbteilungID = ps.AbteilungID
     where ps.SchemaID = @schemaid

   select * 
     from #prs
    order by abteilungid, prsschemaid, prsID

end
go
print 'Procedure: get_personeninschema done ...'
go

grant exec on get_personeninschema to prsadmins with grant option
go
grant exec on get_personeninschema to prsusers
go


if exists (select 1 from sysobjects where name = 'get_personenwithoutdienst' and type = 'P')
begin
   drop procedure get_personenwithoutdienst
   print 'Procedure: get_personenwithoutdienst deleted ...'
end
go
create procedure get_personenwithoutdienst(
  @planid            int      = 74,
  @abteilungid       int      = 1,
  @date              char(8)  = '20181001'
)
as
begin
  set nocount on
  
  --declare @prs table (prsid int)
  
 --insert @prs
 select prsid = ps.PrsID
   from PlanSchema ps
  inner join ArbeitsPlan ap
     on ap.SchemaID = ps.SchemaID
  inner join PersonalAbteilung pa
     on pa.PrsId = ps.PrsID
    and pa.AbteilungID = ps.AbteilungID
    and pa.Von < CONVERT(datetime,@date,112)
    and pa.Bis > CONVERT(datetime,@date,112)
    and isnull(pa.Status,1) < 2
   left outer join Arbeitszeit az
     on az.PrsID = ps.PrsID
    and az.AbteilungID = ps.AbteilungID
    and az.PlanID = ap.PlanID
    --and az.Haben < az.Soll
  inner join PersonAnstellung pan
     on pan.PrsID = ps.PrsID
    and pan.Von < CONVERT(datetime,@date,112)
    and pan.Bis > CONVERT(datetime,@date,112)
  inner join Anstellung ans
     on ans.AnstellungID = pan.AnstellungID
  where ap.PlanID = @planid
    and ps.PrsID not in (select PrsID from Dienstplan 
                                  where PlanID = @planid 
                                    and AbteilungID = @abteilungid 
                                    and Datum = CONVERT(datetime,@date,112))
    and ps.AbteilungID = @abteilungid
    and ((az.Haben < az.Soll) or az.Haben is null or az.Soll is null)
    and ans.Gerechnet = 1
    ----and ans.AnstellungID not in (3)
    --and ans.ausgelernt = 1
  order by ps.PrsID
 
 --select prsid from @prs
 
 
end
go
print 'Procedure: get_personenwithoutdienst done ...'
go

grant exec on get_personenwithoutdienst to prsadmins with grant option
go
grant exec on get_personenwithoutdienst to prsusers
go


if exists (select 1 from sysobjects where name = 'get_personferiencolumndienst' and type = 'P')
begin
   drop procedure get_personferiencolumndienst
   print 'Procedure: get_personferiencolumndienst deleted ...'
end
go
create procedure get_personferiencolumndienst(
  @prsid             int = 1,
  @atid              int = 1,
  @jahrid            int = 1
)
as
begin
  set nocount on

   select datum = dp.Datum,
          dienstid = d.DienstID,
          name = d.Name,
          typwert = at.typwert
     from ArbeitsTage at
     left outer join ArbeitsPlan ap
       on ap.ATID = at.ATID
    inner join Dienstplan dp
       on dp.PlanID = ap.PlanID
      and dp.PrsID = @prsid
     left outer join Dienst d
       on dp.DienstID = d.DienstID
    where at.JahrID = @jahrid
      and at.ATID = @atid
      and d.Typ = 2 -- ferien

end
go
print 'Procedure: get_personferiencolumndienst done ...'
go

grant exec on get_personferiencolumndienst to prsadmins with grant option
go
grant exec on get_personferiencolumndienst to prsusers
go


if exists (select 1 from sysobjects where name = 'get_personferienrow' and type = 'P')
begin
   drop procedure get_personferienrow
   print 'Procedure: get_personferienrow deleted ...'
end
go
create procedure get_personferienrow(
  @prsid      int = 1,
  @jahr       int = 2008
)
as
begin
  set nocount on

  select ps.AbteilungID, 
         a.Name, 
         ap.PlanID, 
         ap.Name 
    from PlanSchema ps
    left outer join ArbeitsPlan ap
      on ap.SchemaID = ps.SchemaID
    left outer join PlanungsJahr pj
      on pj.JahrID = ap.JahrID
    left outer join Abteilung a
      on a.AbteilungID = ps.AbteilungID
   where ps.PrsID = @prsid
     and ap.PlanID is not null
     and pj.Jahr = @jahr
   order by ps.AbteilungID, ap.PlanID

end
go
print 'Procedure: get_personferienrow done ...'
go

grant exec on get_personferienrow to prsadmins with grant option
go
grant exec on get_personferienrow to prsusers
go


if exists (select 1 from sysobjects where name = 'get_personjpcolumn' and type = 'P')
begin
   drop procedure get_personjpcolumn
   print 'Procedure: get_personjpcolumn deleted ...'
end
go
create procedure get_personjpcolumn(
  @prsid             int = 3,
  @abteilungid       int = 1,
  @planid            int = 27,
  @dienstids         varchar(500) = '',
  @datumer           varchar(4084) = '',--20081101
  @selectall         bit = 1
)
as
begin
  set nocount on

   declare @counter int
   declare @lendienstids int
   declare @lendatum int
   
   create table #datum
   (
      datum           datetime
   )
   
   create table #dienste
   (
      dienstid       int
   )
   
   create table #plan
   (
      datum          datetime,
      dienstname     varchar(32),
      dienstid       int,
      colorname      varchar(32),
      rgb            int,
      apname         varchar(32),
      monatname      varchar(32),
      soll           decimal(5,2),
      haben          decimal(5,2),
      ferien         decimal(5,2),
      anpassung      decimal(5,2)
   )
   
   select @lendienstids = LEN(@dienstids)
   select @lendatum = LEN(@datumer)
   
   select @counter = 1
   while @counter < @lendienstids
   begin
      insert into #dienste (dienstid) values (CONVERT(int,SUBSTRING(@dienstids,@counter,2)))
      select @counter = @counter + 2
   end
   
   
   select @counter = 1
   while @counter < @lendatum
   begin
      insert into #datum (datum) values (convert(datetime,SUBSTRING(@datumer,@counter,8),112))
      select @counter = @counter + 8
   end

   
   insert #plan
   select datum        = dp.Datum,
          dienstname   = d.Name,
          dienstid     = d.DienstID,
          colorname    = d.ColorName,
          rgb          = d.RGB,
          apname       = ap.Name,
          monatname    = at.Name,
          soll         = isnull(az.Soll,0),
          haben        = isnull(az.Haben,0),
          ferien       = isnull(az.Ferien,0),
          anpassung    = 0
     from PlanSchema ps
     left outer join ArbeitsPlan ap
       on ap.SchemaID = ps.SchemaID
    inner join Dienstplan dp
       on dp.PlanID = ap.PlanID
      and dp.PrsID = ps.PrsID
      and dp.AbteilungID = ps.AbteilungID
     left outer join Dienst d
       on dp.DienstID = d.DienstID
     left outer join ArbeitsTage at
       on at.ATID = ap.ATID
     left outer join Arbeitszeit az
       on az.PlanID = ap.PlanID
      and az.PrsID = ps.PrsID
      and az.AbteilungID = ps.AbteilungID
    where dp.AbteilungID = @abteilungid
      and ap.PlanID = @planid
      and ps.PrsID = @prsid
      and (@selectall = 1
           or(dp.Datum in (select datum from #datum) 
            and @selectall = 0))
    order by dp.Datum

if @lendienstids > 0
  begin
   insert #plan
   select datum        = dp.Datum,
          dienstname   = d.Name,
          dienstid     = d.DienstID,
          colorname    = d.ColorName,
          rgb          = d.RGB,
          apname       = ap.Name,
          monatname    = at.Name,
          soll         = isnull(az.Soll,0),
          haben        = isnull(az.Haben,0),
          ferien       = isnull(az.Ferien,0),
          anpassung    = 0
     from PlanSchema ps
     left outer join ArbeitsPlan ap
       on ap.SchemaID = ps.SchemaID
    inner join Dienstplan dp
       on dp.PlanID = ap.PlanID
      and dp.PrsID = ps.PrsID
      and dp.AbteilungID = ps.AbteilungID
     left outer join Dienst d
       on dp.DienstID = d.DienstID
     left outer join ArbeitsTage at
       on at.ATID = ap.ATID
     left outer join Arbeitszeit az
       on az.PlanID = ap.PlanID
      and az.PrsID = ps.PrsID
      and az.AbteilungID = ps.AbteilungID
    where dp.AbteilungID = @abteilungid
      and ap.PlanID = @planid
      and ps.PrsID = @prsid
      and d.DienstID in (select dienstid from #dienste)
    order by dp.Datum
  end
    
   update #plan
      set colorname = cp.Colorname,
          rgb       = cp.ARGB
     from Colorplan cp
    where cp.Datum = #plan.datum
      and cp.AbteilungID = @abteilungid
      and cp.PlanID = @planid
      and cp.PrsID = @prsid

   insert #plan
   select datum        = cp.Datum,
          dienstname   = '',
          dienstid     = 0,
          colorname    = cp.Colorname,
          rgb          = cp.ARGB,
          apname       = ap.Name,
          monatname    = at.Name,
          soll         = isnull(az.Soll,0),
          haben        = isnull(az.Haben,0),
          ferien       = isnull(az.Ferien,0),
          anpassung    = 0
     from PlanSchema ps
     left outer join ArbeitsPlan ap
       on ap.SchemaID = ps.SchemaID
    inner join Colorplan cp
       on cp.PlanID = ap.PlanID
      and cp.PrsID = ps.PrsID
      and cp.AbteilungID = ps.AbteilungID
     left outer join ArbeitsTage at
       on at.ATID = ap.ATID
     left outer join Arbeitszeit az
       on az.PlanID = ap.PlanID
      and az.PrsID = ps.PrsID
      and az.AbteilungID = ps.AbteilungID
    where ps.AbteilungID = @abteilungid
      and ap.PlanID = @planid
      and ps.PrsID = @prsid
      and cp.Datum not in (select datum from #plan)

 declare @anpassung decimal(5,2)
  select @anpassung = isnull(sum(sa.Stunden),0)
    from Saldoanpassung sa
    left outer join ArbeitsPlan ap
      on ap.PlanID = sa.PlanID
   where sa.PrsID = @prsid
     and ap.PlanID = @planid
     and sa.AbteilungID = @abteilungid

  update #plan
     set anpassung = @anpassung

   select datum,
          dienstname,
          dienstid,
          colorname,
          rgb,
          apname,
          monatname,
          soll,
          haben,
          ferien,
          anpassung
     from #plan

end
go
print 'Procedure: get_personjpcolumn done ...'
go

grant exec on get_personjpcolumn to prsadmins with grant option
go
grant exec on get_personjpcolumn to prsusers
go


if exists (select 1 from sysobjects where name = 'get_personjprow' and type = 'P')
begin
   drop procedure get_personjprow
   print 'Procedure: get_personjprow deleted ...'
end
go
create procedure get_personjprow(
  @prsid      int = 1,
  @jahrid     int = 1
)
as
begin
  set nocount on

  select ps.AbteilungID, 
         a.Name, 
         ap.PlanID, 
         ap.Name,
         at.Name

    from PlanSchema ps
    left outer join ArbeitsPlan ap
      on ap.SchemaID = ps.SchemaID
    left outer join PlanungsJahr pj
      on pj.JahrID = ap.JahrID
    left outer join Abteilung a
      on a.AbteilungID = ps.AbteilungID
    left outer join ArbeitsTage at
      on at.ATID = ap.ATID
   where ps.PrsID = @prsid
     and ap.PlanID is not null
     and pj.JahrID = @jahrid
   order by ps.AbteilungID, ap.PlanID

end
go
print 'Procedure: get_personjprow done ...'
go

grant exec on get_personjprow to prsadmins with grant option
go
grant exec on get_personjprow to prsusers
go



if exists (select 1 from sysobjects where name = 'get_personnotinschema' and type = 'P')
begin
   drop procedure get_personnotinschema
   print 'Procedure: get_personnotinschema deleted ...'
end
go
create procedure get_personnotinschema(
  @schemaid int = 1
)
as
begin
  set nocount on

  select p.prsid,
         a.abteilungid,
         p.vorname,
         p.nachname,
         a.name

    from abteilung a
    left outer join personalabteilung pa
      on a.abteilungid = pa.abteilungid
    left outer join Person p
     on p.prsid = pa.prsid
    and p.persontyp = 1
  where a.abteilungid in (select ps.abteilungid from planschema ps where ps.schemaid = @schemaid)
    and p.prsid not in (select ps.prsid from planschema ps where ps.schemaid = @schemaid)
    and pa.bis = (select max(pa2.bis) from personalabteilung pa2 where pa2.prsid = p.prsid and a.abteilungid = pa2.abteilungid)

end
go
print 'Procedure: get_personnotinschema done ...'
go

grant exec on get_personnotinschema to prsadmins with grant option
go
grant exec on get_personnotinschema to prsusers
go



if exists (select 1 from sysobjects where name = 'get_personnotintermin' and type = 'P')
begin
   drop procedure get_personnotintermin
   print 'Procedure: get_personnotintermin deleted ...'
end
go
create procedure get_personnotintermin(
  @planid                int           = 28,
  @planterminid          int           = 1
)
as
begin
  set nocount on

  declare @datevon datetime
  
  select @datevon = Von from ArbeitsPlan where PlanID = @planid
  
  select p.PrsId,
         Vorname,
         Nachname, 
         a.AbteilungID, 
         Anzeigename=Nachname + ' '+ Vorname + ' - ' + a.Name
    from Person p 
   inner join PersonalAbteilung pa 
      on pa.PrsId = p.PrsId 
   inner join Abteilung a 
      on a.AbteilungID = pa.AbteilungID 
   where p.PrsId not in (select tp.PrsID 
                           from Terminplan tp 
                          where tp.PlanTerminID = @planterminid 
                            and tp.PlanID = @planid) 
     and PersonTyp = 1
     and (@datevon >= Von and @datevon<= Bis)
   order by a.AbteilungID,p.PrsId
   

end
go
print 'Procedure: get_personnotintermin done ...'
go
grant exec on get_personnotintermin to prsadmins with grant option
go
grant exec on get_personnotintermin to prsusers
go



if exists (select 1 from sysobjects where name = 'get_personpensum' and type = 'P')
begin
   drop procedure get_personpensum
   print 'Procedure: get_personpensum deleted ...'
end
go
create procedure get_personpensum(
  @prsid              int           = 3,
  @stichdatum         char(8)       = '20090831'
)
as
begin
  set nocount on
 if @stichdatum is null select @stichdatum = GETDATE()
 if @stichdatum = '00000000' select @stichdatum = GETDATE()

 select prsid = a.PrsID,
        von = a.Von ,
        bis = a.Bis,
        pensum = a.Prozent
   from Arbeitspensum a
  where a.PrsID = @prsid
    and @stichdatum >= a.Von
    and @stichdatum <= a.Bis
    and a.Bis = (select MAX(ap.Bis) from Arbeitspensum ap 
                                 where ap.PrsID = a.PrsID
                                   and @stichdatum >= ap.Von
                                   and @stichdatum <= ap.Bis)

end
go
print 'Procedure: get_personpensum done ...'
go
grant exec on get_personpensum to prsadmins with grant option
go
grant exec on get_personpensum to prsusers
go


if exists (select 1 from sysobjects where name = 'get_personplanovervieweaders' and type = 'P')
begin
   drop procedure get_personplanovervieweaders
   print 'Procedure: get_personplanovervieweaders deleted ...'
end
go
create procedure get_personplanovervieweaders(
  @prsid      int = 1,
  @jahrid     int = 1
)
as
begin
  set nocount on

  select abteilungid = ps.AbteilungID, 
         abteilungname = a.Name, 
         planid = ap.PlanID, 
         planname = ap.Name,
         monat = at.Name,
         soll = isnull(az.Soll,0),
         haben = isnull(az.Haben,0),
         ferien = isnull(az.Ferien,0),
         anpassung = (select isnull(sum(sa.Stunden),0)
						from Saldoanpassung sa
					   where sa.PrsID = @prsid
						 and sa.PlanID = ap.PlanID
						 and sa.AbteilungID = ps.AbteilungID),
		 prsschemaid = ps.PrsSchemaID,
		 von = ap.Von,
		 bis = ap.Bis
    from PlanSchema ps
    left outer join ArbeitsPlan ap
      on ap.SchemaID = ps.SchemaID
    left outer join PlanungsJahr pj
      on pj.JahrID = ap.JahrID
    left outer join Abteilung a
      on a.AbteilungID = ps.AbteilungID
    left outer join ArbeitsTage at
      on at.ATID = ap.ATID
    left outer join Arbeitszeit az
      on az.PlanID = ap.PlanID
     and az.PrsID = ps.PrsID
     and az.AbteilungID = ps.AbteilungID
   where ps.PrsID = @prsid
     and ap.PlanID is not null
     and pj.JahrID = @jahrid
   order by ps.AbteilungID, ap.PlanID

end
go
print 'Procedure: get_personplanovervieweaders done ...'
go

grant exec on get_personplanovervieweaders to prsadmins with grant option
go
grant exec on get_personplanovervieweaders to prsusers
go


if exists (select 1 from sysobjects where name = 'get_plandienstserie' and type = 'P')
begin
   drop procedure get_plandienstserie
   print 'Procedure: get_plandienstserie deleted ...'
end
go
create procedure get_plandienstserie(
    @planid     int = 1
)
as
begin
  set nocount on

  declare @datumvon   datetime
  declare @datumbis   datetime

  create table #dienstserie
  (
     prsid         int,
     dienstid      int,
     abteilungid   int,
     tag           int,
     typ           int,
     von           datetime,
     bis           datetime,
     planvon       datetime,
     planbis       datetime
  )
  
  select @datumvon = Von,
         @datumbis = Bis
    from ArbeitsPlan
   where PlanID = @planid

  insert #dienstserie
  select prsid = ds.PrsID,
         dienstid = ds.DienstID,
         abteilungid = ds.AbteilungID,
         tag = ds.Tag,
         typ = ds.Typ,
         von = ds.Von, 
         bis = ds.bis,
         planvon = @datumvon,
         planbis = @datumbis
   from DienstSerie ds
  where ((Von <= @datumvon and bis >= @datumbis)          -- bis
       or (Von >= @datumvon and bis >= @datumbis and Von <= @datumbis)
       or (Von <= @datumvon and bis >= @datumbis)
			           										-- von
       or (Von <= @datumvon and bis <= @datumbis and bis >= @datumvon)
       or (Von >= @datumvon and bis <= @datumbis))
       
   select * from #dienstserie

end
go
print 'Procedure: get_plandienstserie done ...'
go

grant exec on get_plandienstserie to prsadmins with grant option
go
grant exec on get_plandienstserie to prsusers
go


if exists (select 1 from sysobjects where name = 'get_planrow' and type = 'P')
begin
   drop procedure get_planrow
   print 'Procedure: get_planrow deleted ...'
end
go
create procedure get_planrow(
  @planid            int = 27,
  @prsschemaid       int = 2,
  @prsid             int = 1
)
as
begin
  set nocount on

   declare @stundenprotag int
--   declare @prsid int
   declare @abteilungid int
   declare @datumvon datetime
   declare @mediandate datetime

   create table #dienste(
     datum    datetime,
     name     varchar(32),
     dienstid int,
     stunden  decimal(4,2),
     rgb      int,
     colorname varchar(32),
     typ      int
   )
   
   set @datumvon = (select Von from ArbeitsPlan where PlanID = @planid)
   set @mediandate = convert(datetime,(select convert(varchar,(datepart(yyyy,@datumvon)))+
                                       case when datepart(MM,@datumvon) <10 
                                            then '0'+convert(varchar,(datepart(MM,@datumvon)))+'15'
                                            else convert(varchar,(datepart(MM,@datumvon)))+'15'
                                             end),112)

   select @stundenprotag = stundenprotag
     from PlanSchema ps
    inner join ArbeitsPlan ap
       on ap.SchemaID = ps.SchemaID
    inner join Abteilung a
       on a.AbteilungID = ps.AbteilungID
    where ap.PlanID = @planid
      and ps.PrsSchemaID = @prsschemaid

   select @abteilungid = AbteilungID
     from PlanSchema ps
    inner join ArbeitsPlan ap
       on ap.SchemaID = ps.SchemaID
    where ap.PlanID = @planid
      and ps.PrsSchemaID = @prsschemaid

--   select @prsid = PrsID
--     from PlanSchema ps
--    inner join ArbeitsPlan ap
--       on ap.SchemaID = ps.SchemaID
--    where ap.PlanID = @planid
--      and ps.PrsSchemaID = @prsschemaid

   insert #dienste
   select datum    = Datum,
          name     = d.Name,
          dienstid = dp.DienstID,
          stunden  = 0,
          rgb      = d.RGB,
          colorname = d.ColorName,
          typ      = d.Typ
     from ArbeitsPlan ap
    inner join PlanSchema ps
       on ps.SchemaID = ap.SchemaID
     left outer join Person p
       on p.PrsId = ps.PrsID
    inner join Abteilung a
       on a.AbteilungID = ps.AbteilungID
    inner join Dienstplan dp
       on dp.PlanID = ap.PlanID
      and dp.PrsID = ps.PrsID
      and dp.AbteilungID = ps.AbteilungID
    inner join Dienst d
       on d.DienstID = dp.DienstID
    where ap.PlanID = @planid
      and ps.PrsSchemaID = @prsschemaid
    order by PrsSchemaID

   insert #dienste
   select datum    = Datum,
          name     = d.Name,
          dienstid = dp.DienstID,
          stunden  = 0,
          rgb      = d.RGB,
          colorname = d.ColorName,
          typ      = d.Typ
     from ArbeitsPlanzprs ap
     left outer join Person p
       on p.PrsId = ap.zprsID
    inner join Abteilung a
       on a.AbteilungID = ap.abteilungID
    inner join Dienstplan dp
       on dp.PlanID = ap.planID
      and dp.PrsID = ap.zprsID
      and dp.AbteilungID = ap.abteilungID
    inner join Dienst d
       on d.DienstID = dp.DienstID
    where ap.planID = 24
      and ap.zprsID = @prsid

-- zeiten
   update #dienste
      set stunden = @stundenprotag
     from #dienste
    inner join Dienst d
       on #dienste.dienstid = d.DienstID
    inner join DienstZeit dz
       on dz.DienstID = d.DienstID
      and dz.datvon <= @mediandate
      and dz.datbis > @mediandate
      and dz.abteilungzeit = 1

   update #dienste
      set stunden = convert(varchar(2),datepart(hh, Bis - Von)) + '.' + convert(varchar(2),datepart(n, Bis - Von))
     from #dienste
    inner join Dienst d
       on #dienste.dienstid = d.DienstID
    inner join DienstZeit dz
       on dz.DienstID = d.DienstID
      and dz.datvon <= @mediandate
      and dz.datbis > @mediandate
      and dz.abteilungzeit <> 1

-- farben
   update #dienste
      set rgb = ARGB,
          colorname = cp.Colorname
     from #dienste, Colorplan cp
    where cp.PlanID = @planid
      and cp.Datum = #dienste.datum
      and cp.PrsID = @prsid
      and cp.AbteilungID = @abteilungid


   insert #dienste
   select datum    = Datum,
          name     = '',
          dienstid = 0,
          stunden  = 0,
          rgb      = ARGB,
          colorname = Colorname,
          typ      = 0
     from Colorplan
    where Datum not in (select datum from #dienste)
      and PlanID = @planid
      and PrsID = @prsid
      and AbteilungID = @abteilungid

-- select * from ColorPlan where planid = 9

   select *
     from #dienste
    order by datum

end
go
print 'Procedure: get_planrow done ...'
go

grant exec on get_planrow to prsadmins with grant option
go
grant exec on get_planrow to prsusers
go


if exists (select 1 from sysobjects where name = 'get_planrowcolor' and type = 'P')
begin
   drop procedure get_planrowcolor
   print 'Procedure: get_planrowcolor deleted ...'
end
go
create procedure get_planrowcolor(
  @planid            int = 9,
  @prsschemaid       int = 1
)
as
begin
  set nocount on

   declare @stundenprotag int
   declare @prsid int
   declare @abteilungid int

   create table #dienste(
     datum    datetime,
     rgb      int,
     prsid    int
   )

   select @stundenprotag = stundenprotag
     from PlanSchema ps
    inner join ArbeitsPlan ap
       on ap.SchemaID = ps.SchemaID
    inner join Abteilung a
       on a.AbteilungID = ps.AbteilungID
    where ap.PlanID = @planid
      and ps.PrsSchemaID = @prsschemaid

   select @abteilungid = AbteilungID
     from PlanSchema ps
    inner join ArbeitsPlan ap
       on ap.SchemaID = ps.SchemaID
    where ap.PlanID = @planid
      and ps.PrsSchemaID = @prsschemaid

   select @prsid = PrsID
     from PlanSchema ps
    inner join ArbeitsPlan ap
       on ap.SchemaID = ps.SchemaID
    where ap.PlanID = @planid
      and ps.PrsSchemaID = @prsschemaid

   insert #dienste
   select datum    = Datum,
          rgb      = ARGB,
          prsid    = @prsid
     from ColorPlan
    where PlanID = @planid
      and PrsID = @prsid
      and AbteilungID = @abteilungid

select * from #dienste
   --select datum  = isnull(#dienste.datum,''),
   --       rgb    = isnull(#dienste.rgb,0),
   --       rowrgb = PlanSchema.RGB
   --  from PlanSchema
   -- inner join ArbeitsPlan ap
   --    on ap.SchemaID = PlanSchema.SchemaID
   -- left outer join #dienste
   --    on PlanSchema.PrsID = #dienste.prsid
   -- where PlanSchema.PrsSchemaID = @prsschemaid
   --   and PlanSchema.AbteilungID = @abteilungid
   --   and ap.PlanID = @planid


end
go
print 'Procedure: get_planrowcolor done ...'
go

grant exec on get_planrowcolor to prsadmins with grant option
go
grant exec on get_planrowcolor to prsusers
go



if exists (select 1 from sysobjects where name = 'get_planshema' and type = 'P')
begin
   drop procedure get_planshema
   print 'Procedure: get_planshema deleted ...'
end
go
create procedure get_planshema(
  @schemaid         int               = 25
)
as
begin
  set nocount on

  select schemaname = s.Name,
         schemaid = s.SchemaID,
         prsschemaid = ps.PrsSchemaID,
         prsid = ps.PrsID,
         vorname = isnull(p.Vorname,''),
         nachname = isnull(p.Nachname,''),
         abteilungid = ps.AbteilungID,
         abteilungname = a.Name,
         rgb = ps.RGB,
         color = ps.ColorName
    from PlanSchema ps
   inner join Schemas s
      on s.SchemaID = ps.SchemaID
    left outer join Person p
      on p.PrsId = ps.PrsID
   inner join Abteilung a
      on a.AbteilungID = ps.AbteilungID
   where ps.SchemaID = @schemaid
   order by ps.PrsSchemaID

end
go
print 'Procedure: get_planshema done ...'
go
grant exec on get_planshema to prsadmins with grant option
go
grant exec on get_planshema to prsusers
go



if exists (select 1 from sysobjects where name = 'get_plantermine' and type = 'P')
begin
   drop procedure get_plantermine
   print 'Procedure: get_plantermine deleted ...'
end
go
create procedure get_plantermine(
  @planid             int               = 25,
  @mandid             char(2)           = 'li',
  @planterminid       int               = 0
)
as
begin
  set nocount on

  select terminid = tp.TerminID,
         terminname = tp.Name,
         planid = tp.PlanID, 
         prsid = tp.PrsID, 
         datum = tp.Datum, 
         abteilungid = tp.AbteilungID
    from Terminplan tp 
   inner join Termin t 
      on tp.TerminID = t.TerminID 
   where tp.PlanID = @planid 
     and (tp.PlanTerminID = @planterminid or @planterminid = 0) 
     and t.mandid = @mandid
   order by tp.TerminID, tp.PrsID, tp.AbteilungID

end
go
print 'Procedure: get_plantermine done ...'
go
grant exec on get_plantermine to prsadmins with grant option
go
grant exec on get_plantermine to prsusers
go



if exists (select 1 from sysobjects where name = 'get_saldo' and type = 'P')
begin
   drop procedure get_saldo
   print 'Procedure: get_saldo deleted ...'
end
go
create procedure get_saldo(
  @prsid              int           = 1,
  @planid             int           = 25,
  @abteilungid        int           = 1
)
as
begin
  set nocount on

  declare @date datetime
  declare @vortrag decimal(5,2)
  
  select @date = Von from ArbeitsPlan where PlanID = @planid

  create table #saldo(
    saldo      decimal(7,2)
  )

  insert #saldo
  select saldo = isnull(sum((Haben + Ferien) - Soll),0) from Arbeitszeit az
   inner join ArbeitsPlan ap
      on ap.PlanID = az.PlanID
   where az.PrsID = @prsid 
     and az.AbteilungID = @abteilungid
     and ap.Bis < @date
     and Soll > 0

  select @date = Von from ArbeitsPlan where PlanID = @planid

  insert #saldo
  select saldo = sum(Stunden)
    from Saldoanpassung
   where PrsID = @prsid
     and AbteilungID = @abteilungid
     and Datum < @date

   select @vortrag = sum(isnull(saldo,0))
     from #saldo
          
  declare @soll     decimal(7,2)
  declare @haben    decimal(7,2)
  declare @ferien   decimal(7,2)
  declare @anpassung decimal(7,2)
     
   select @soll = isnull(Soll,0)
     from Arbeitszeit
    where PrsID = @prsid
      and PlanID = @planid
      and AbteilungID = @abteilungid
      
   select @haben = isnull(Haben,0)
     from Arbeitszeit
    where PrsID = @prsid
      and PlanID = @planid
      and AbteilungID = @abteilungid
      
   select @ferien = isnull(Ferien,0)
     from Arbeitszeit
    where PrsID = @prsid
      and PlanID = @planid
      and AbteilungID = @abteilungid
      
   select @anpassung = ISNULL(sum(isnull(Stunden,0)),0)
    from Saldoanpassung 
   where PrsID = @prsid
     and AbteilungID = @abteilungid
     and PlanID = @planid
      
   select vortrag = isnull(@vortrag,0),
          soll = isnull(@soll,0),
          haben = isnull(@haben,0),
          ferien = isnull(@ferien,0),
          anpassung = isnull(@anpassung,0)


end
go
print 'Procedure: get_saldo done ...'
go
grant exec on get_saldo to prsadmins with grant option
go
grant exec on get_saldo to prsusers
go



if exists (select 1 from sysobjects where name = 'get_saldoanpassung' and type = 'P')
begin
   drop procedure get_saldoanpassung
   print 'Procedure: get_saldoanpassung deleted ...'
end
go
create procedure get_saldoanpassung(
  @prsid              int           = 1,
  @abteilungid        int           = 1,
  @planid             int           = 4
)
as
begin
  set nocount on

  select saldoid = SaldoID,
         datum = Datum,
         grund = Grund,
         stunden = Stunden,
         planid = PlanID,
         abteilungid = AbteilungID,
         prsid = PrsID
    from Saldoanpassung 
   where PrsID = @prsid
     and AbteilungID = @abteilungid
     and PlanID = @planid

end
go
print 'Procedure: get_saldoanpassung done ...'
go
grant exec on get_saldoanpassung to prsadmins with grant option
go
grant exec on get_saldoanpassung to prsusers
go



if exists (select 1 from sysobjects where name = 'get_shemas' and type = 'P')
begin
   drop procedure get_shemas
   print 'Procedure: get_shemas deleted ...'
end
go
create procedure get_shemas(
  @mandid             char(2)           = 'li',
  @status             int               = 1 --1:Active   0:All
)
as
begin
  set nocount on

  select schemaid = SchemaID,
         name = Name,
         status = Status
    from Schemas
   where (Status = @status or @status = 0)

end
go
print 'Procedure: get_shemas done ...'
go
grant exec on get_shemas to prsadmins with grant option
go
grant exec on get_shemas to prsusers
go



if exists (select 1 from sysobjects where name = 'get_updprsschema' and type = 'P')
begin
   drop procedure get_updprsschema
   print 'Procedure: get_updprsschema deleted ...'
end
go
create procedure get_updprsschema(
  @schemaid         int     = 23,
  @plandatum        char(8) = '20090301'
)
as
begin
  set nocount on

   create table #schema(
     prsid            int,
     abteilungid      int,
     typ              int,
     pos              int
   )

   create table #abteilung(
      abteilungid     int
   )

   insert #abteilung
   select abteilungid = abteilungid
     from planschema
    where schemaid = @schemaid
    group by abteilungid

   -- kndigungen gesamtes pensum
   insert #schema
   select prsid = ap.prsid,
          abteilungid = pa.abteilungid,
          typ = 1, -- kndigungen
          pos = ps.prsschemaid
     from arbeitspensum ap
    inner join personalabteilung pa
       on pa.prsid = ap.prsid
      and (pa.status = 1 or pa.status is null)
    inner join planschema ps
       on ps.schemaid = @schemaid
      and ps.abteilungid = pa.abteilungid
      and ps.prsid = pa.prsid
    where ap.bis in (select max(ap2.bis) from arbeitspensum ap2 where ap2.prsid = ap.prsid)
      and ap.bis < convert(datetime,@plandatum,112)--getdate() --


   -- abteilungswechsel
   -- nicht in schema enthaltene
   insert #schema
   select prsid = pa.prsid,
          abteilungid = pa.abteilungid,
          typ = 2, -- nicht in schema enthalten
          pos = 0
     from personalabteilung pa
     left outer join planschema ps
       on ps.prsid = pa.prsid
      and ps.abteilungid = pa.abteilungid
      and ps.schemaid = @schemaid
    where pa.bis = (select max(pa2.bis) from personalabteilung pa2 where pa2.prsid = pa.prsid and pa2.abteilungid = pa.abteilungid)
      and ps.schemaid is null
      and convert(datetime,@plandatum,112) between pa.von and pa.bis
      and pa.prsid not in (select s.prsid from #schema s where typ = 1) -- keine gekndigten personen
      and pa.abteilungid in (select abt.abteilungid from #abteilung abt)

   -- personen die aus dem schema/abteilung entfernt werden mssen
   insert #schema
   select prsid = ps.prsid,
          abteilung = pa.abteilungid,
          typ = 3, -- aus schema entfernen
          pos = ps.prsschemaid
     from planschema ps
    inner join personalabteilung pa
       on pa.prsid = ps.prsid
      and pa.abteilungid = ps.abteilungid
      and pa.status = 2
      and pa.bis = (select max(pa2.bis) from personalabteilung pa2 where pa2.prsid = pa.prsid/* and pa2.abteilungid = pa.abteilungid*/)
      and convert(datetime, @plandatum, 112) not between pa.von and pa.bis
    where ps.schemaid = @schemaid



   select prsid = #schema.prsid,
          abteilungid = #schema.abteilungid,
          typ = #schema.typ,
          pos = #schema.pos
     from #schema

end
go
print 'Procedure: get_updprsschema done ...'
go
grant exec on get_updprsschema to prsadmins with grant option
go
grant exec on get_updprsschema to prsusers
go




if exists (select 1 from sysobjects where name = 'set_abteilung' and type = 'P')
begin
   drop procedure set_abteilung
   print 'Procedure: set_abteilung deleted ...'
end
go
create procedure set_abteilung(
  @abteilungname     varchar(32)   = 'Abteilung',
  @kurztext          varchar(60)   = 'Abteilung kurztext',
  @beschreibung      varchar(250)  = 'Abteilung Beschreibung',
  @stundenprotag     decimal(5,2)  = 20,
  @mandid            char(2)       = 'li',
--  @gerechnet         int           = 1,
  @retval          int             = 0 output
)
as
begin
  set nocount on

  declare @beschrid   int

  if @beschreibung is null select @beschreibung = '' 
   if not exists (select 1 from Beschreibung where Beschreibung.BeschrID between 1001 and 1500)
   begin 
     select @beschrid = 1001
    end
   else
   begin
     select @beschrid = max(Beschreibung.BeschrID) + 1
       from Beschreibung
      where Beschreibung.BeschrID between 1001 and 1500
    end
  exec set_beschreibung @beschrid, @kurztext, @beschreibung
  insert into Abteilung (Name, BeschrID,stundenprotag,mandid) values (@abteilungname, @beschrid, @stundenprotag,@mandid)

end
go
print 'Procedure: set_abteilung done ...'
go
grant exec on set_abteilung to prsadmins with grant option
go
grant exec on set_abteilung to prsusers
go


if exists (select 1 from sysobjects where name = 'set_anstellung' and type = 'P')
begin
   drop procedure set_anstellung
   print 'Procedure: set_anstellung deleted ...'
end
go
create procedure set_anstellung(
  @mandid             char(2)       = 'li',
  @anstellungname     varchar(32)   = 'Abteilung',
  @kurztext           varchar(60)   = 'Abteilung',
  @beschreibung       varchar(250)  = 'Abteilung Beschreibung',
  @prio               int = 0,
  @EinsatzProTag      int           = 1,
  @Gerechnet          int           = 1,
  @ausgelernt         int           = 0
)
as
begin
  set nocount on
  declare @beschrid   int
  if @beschreibung is null select @beschreibung = ''
   if not exists (select 1 from Beschreibung where Beschreibung.BeschrID between 501 and 1000)
   begin 
     select @beschrid = 501
    end
   else
   begin
     select @beschrid = max(Beschreibung.BeschrID) + 1
       from Beschreibung
      where Beschreibung.BeschrID between 501 and 1000
    end
  exec set_beschreibung @beschrid, @kurztext, @beschreibung
  insert into Anstellung (Name, Prioritaet, BeschrID, EinsatzProTag, Gerechnet, mandid, ausgelernt) values (@anstellungname, @prio, @beschrid, @EinsatzProTag, @Gerechnet, @mandid,@ausgelernt)

end
go
print 'Procedure: set_anstellung done ...'
go
grant exec on set_anstellung to prsadmins with grant option
go
grant exec on set_anstellung to prsusers
go


if exists (select 1 from sysobjects where name = 'set_arbeitstage' and type = 'P')
begin
   drop procedure set_arbeitstage
   print 'Procedure: set_arbeitstage deleted ...'
end
go
-- setzt anzahl tage pro monat fest
create procedure set_arbeitstage(
  @jahr          int          = 1900,
  @Name          varchar(60)  = 'januar',
  @typwert       int          = 1,
  @Von           varchar(8)   = '20070101',
  @Bis           varchar(8)   = '20070131',
  @Typ           int          = 1, -- 1: Monat   2: Feiertag
  @Tage          decimal(5,2) = 23,   --Anzahl tage im monat an denen gearbeitet wird / anzahl bezahlte tage pro feiertag 
  @mandid        char(2)      = 'li',
  @retval        int          = 0 output
)
as
begin

  set nocount on

  declare @jahrid  int
  declare @datevon datetime
  declare @datebis datetime

  select @jahrid = JahrID from PlanungsJahr where PlanungsJahr.Jahr = @jahr
  select @datevon = convert(datetime, @Von, 112)
  select @datebis = convert(datetime, @Bis, 112)
  
  if @mandid = '' select top 1 @mandid = mandid from mandant where active = 1

  insert into ArbeitsTage (JahrID,Name,typwert,Von,Bis,typ,Tage,mandid) values (@jahrid,@Name,@typwert,@datevon,@datebis,@Typ,@Tage,@mandid)
  
  select @retval = ATID from ArbeitsTage where JahrID = @jahrid and Name = @Name and Von = @Von and typ = @Typ

end
go
print 'Procedure: set_arbeitstage done ...'
go
grant exec on set_arbeitstage to prsadmins with grant option
go
grant exec on set_arbeitstage to prsusers
go


if exists (select 1 from sysobjects where name = 'set_beschreibung' and type = 'P')
begin
   drop procedure set_beschreibung
   print 'Procedure: set_beschreibung deleted ...'
end
go
create procedure set_beschreibung(
  @beschrid          int           = 1,
  @dienstname        varchar(32)   = 'Dienst',
  @beschreibung      varchar(250)  = 'Dienst Beschreibung'
)
as
begin
  set nocount on
   insert into Beschreibung (BeschrID, Kurztext, Langtext) values (@beschrid, @dienstname, @beschreibung)
end
go
print 'Procedure: set_beschreibung done ...'
go
grant exec on set_beschreibung to prsadmins with grant option
go
grant exec on set_beschreibung to prsusers
go


if exists (select 1 from sysobjects where name = 'set_colorplan' and type = 'P')
begin
   drop procedure set_colorplan
   print 'Procedure: set_colorplan deleted ...'
end
go
create procedure set_colorplan(
   @planID         int  = 1,
   @prsID          int  = 1,
   @datum          char(8) = '20070501',
   @abteilungID    int  = 1,
   @colorname      varchar(32)  = '',
   @argb           int = 0
)
as
begin
  set nocount on

  insert into Colorplan (PlanID, PrsID, Datum, AbteilungID, Colorname, ARGB) values (@planID, @prsID, @datum, @abteilungID, @colorname, @argb)

end
go
print 'Procedure: set_colorplan done ...'
go
grant exec on set_colorplan to prsadmins with grant option
go
grant exec on set_colorplan to prsusers
go



if exists (select 1 from sysobjects where name = 'set_copydienstschema' and type = 'P')
begin
   drop procedure set_copydienstschema
   print 'Procedure: set_copydienstschema deleted ...'
end
go
create procedure set_copydienstschema(
  @oldschemaid      int = 1,
  @newschemaid      int = 1
)
as
begin

   set nocount on

   insert DienstSchema
   select schemaid      = @newschemaid,
          abteilungid   = ds.abteilungid,
          dienstid      = ds.dienstid,
          einsatzprotag = ds.einsatzprotag,
          gerechnet     = ds.gerechnet
     from dienstschema ds
    where ds.schemaid = @oldschemaid

end
go
print 'Procedure: set_copydienstschema done ...'
go
grant exec on set_copydienstschema to prsadmins with grant option
go
grant exec on set_copydienstschema to prsusers
go



if exists (select 1 from sysobjects where name = 'set_dienstplan' and type = 'P')
begin
   drop procedure set_dienstplan
   print 'Procedure: set_dienstplan deleted ...'
end
go
create procedure set_dienstplan(
   @planid         int  = 1,
   @prsid          int  = 1,
   @datum          char(8) = '20070501',
   @abteilungid    int  = 1,
   @dienstid       int  = 1,
   @stunden        decimal(5,2) = 0.0
)
as
begin
  set nocount on

declare @ferien   decimal(5,2)
declare @arbeitsstunden decimal(5,2)
declare @typ int

select @typ = Typ from Dienst where DienstID = @dienstid

  if @typ = 2 --Ferien
   begin
    select @ferien = (select Ferien from Arbeitszeit where PrsID = @prsid and PlanID = @planid and AbteilungID = @abteilungid) + @stunden
    select @arbeitsstunden = (select Haben from Arbeitszeit where PrsID = @prsid and PlanID = @planid and AbteilungID = @abteilungid)
   end

  else
   begin
    select @ferien = (select Ferien from Arbeitszeit where PrsID = @prsid and PlanID = @planid and AbteilungID = @abteilungid)
    select @arbeitsstunden = (select Haben from Arbeitszeit where PrsID = @prsid and PlanID = @planid and AbteilungID = @abteilungid) + @stunden
  end

  insert into Dienstplan (PlanID, PrsID, Datum, AbteilungID, DienstID) values (@planid, @prsid, @datum, @abteilungid, @dienstid)

  declare @pensum decimal(5,2)
  declare @arbeitstage decimal(5,2)
  declare @stundenprotag decimal(5,2)
  declare @soll decimal(5,2)

   select @pensum = isnull(nullif(pa.Prozent,0) ,isnull(a.Prozent,100))
     from Arbeitspensum a,
          PersonalAbteilung pa
    where pa.AbteilungID = @abteilungid
      and a.PrsID = @prsid
      and a.Von <= @datum 
      and a.Bis > @datum
      and pa.PrsId = a.PrsID
      and pa.Von <= @datum
      and pa.Bis > @datum

   if @pensum is null
            select @pensum = 100

  select @arbeitstage = Tage from ArbeitsTage at
   inner join PlanungsJahr pj
      on pj.JahrID = at.JahrID
     and @datum between pj.von and pj.bis
   inner join ArbeitsPlan ap
      on ap.ATID = at.ATID
   where ap.PlanID = @planid

  select @stundenprotag = stundenprotag from Abteilung where AbteilungID = @abteilungid

  select @soll = ((@arbeitstage * @pensum)/100) * @stundenprotag


  if exists (select 1 from Arbeitszeit where PrsID = @prsid and PlanID = @planid and AbteilungID = @abteilungid)
   begin
    update Arbeitszeit set Soll = isnull(@soll,0),--0,
           Haben = isnull(@arbeitsstunden,0),
           Ferien = isnull(@ferien,0)
     where PrsID = @prsid and PlanID = @planid and AbteilungID = @abteilungid
   end
  else
   begin
    insert into Arbeitszeit (PrsID,PlanID,AbteilungID,Soll,Haben,Ferien) values (@prsid, @planid, @abteilungid, /*0*/isnull(@soll,0),0,0)
  end

end
go
print 'Procedure: set_dienstplan done ...'
go
grant exec on set_dienstplan to prsadmins with grant option
go
grant exec on set_dienstplan to prsusers
go


if exists (select 1 from sysobjects where name = 'set_dienstschema' and type = 'P')
begin
   drop procedure set_dienstschema
   print 'Procedure: set_dienstschema deleted ...'
end
go
create procedure set_dienstschema(
  @schemaid      int = 1,
  @abteilungid   int = 1,
  @dienstid      int = 1,
  @einsatzprotag int = 1,
  @gerechnet     int = 1
)
as
begin

   set nocount on

   insert into dienstschema (schemaid,abteilungid,dienstid,einsatzprotag,gerechnet) values (@schemaid, @abteilungid, @dienstid, @einsatzprotag, @gerechnet)

end
go
print 'Procedure: set_dienstschema done ...'
go
grant exec on set_dienstschema to prsadmins with grant option
go
grant exec on set_dienstschema to prsusers
go


if exists (select 1 from sysobjects where name = 'set_dienstserie' and type = 'P')
begin
   drop procedure set_dienstserie
   print 'Procedure: set_dienstserie deleted ...'
end
go
create procedure set_dienstserie(
  @PrsID             int           = 1,
  @dienstid          int           = 1,
  @AbteilungID       int           = 1,
  @Tag               int           = 1,
  @Typ               varchar(5)       = 'W',--M: Monat W: Woche
  @von               varchar(8)    = '20080101',
  @bis               varchar(8)    = '20100101'
--  @status            int
)
as
begin
  set nocount on

  declare @datevon datetime
  declare @datebis datetime

  select @datevon = convert(datetime, @von, 112)
  select @datebis = convert(datetime, @bis, 112)

  insert into dienstserie(prsid, dienstid, abteilungid, tag, typ, von, bis) values (@PrsID, @dienstid, @AbteilungID, @Tag, upper(@Typ), @datevon, @datebis)

end
go
print 'Procedure: set_dienstserie done ...'
go
grant exec on set_dienstserie to prsadmins with grant option
go
grant exec on set_dienstserie to prsusers
go


if exists (select 1 from sysobjects where name = 'set_ferienzeit' and type = 'P')
begin
   drop procedure set_ferienzeit
   print 'Procedure: set_ferienzeit deleted ...'
end
go
create procedure set_ferienzeit(
  @prsid             int           = 1,
  @von               char(8)       = '20070101',--'00000000',
  @bis               char(8)       = '20070105',
  @retval            int           = 0 output
)
as
begin
  set nocount on
  declare    @datumvon    datetime
  declare    @datumbis    datetime
   if @prsid = 0 or @prsid = '' select @prsid = null
   if @von <> '00000000' select @datumvon = convert(datetime, @von, 112)
   if @von <> '00000000' select @datumbis = convert(datetime, @bis, 112)
   if (@prsid is not null)
   begin
      insert into Ferien (PrsId, Von, Bis) values (@prsid, @datumvon, @datumbis)
      
      select @retval = MAX(FerienID)
        from Ferien
       where PrsId = @prsid
         and Von = @von
         and Bis = @bis
      
   end
end
go
print 'Procedure: set_ferienzeit done ...'
go
grant exec on set_ferienzeit to prsadmins with grant option
go
grant exec on set_ferienzeit to prsusers
go


if exists (select 1 from sysobjects where name = 'set_mandant' and type = 'P')
begin
   drop procedure set_mandant
   print 'Procedure: set_mandant deleted ...'
end
go
create procedure set_mandant(
  @mandid            char(2)        = 'PA',
  @name              varchar(35)    = 'Personalplaner mandant',
  @beschreibung      varchar(250)   = 'Default mandant',
  @retval            int            = 0 output
)
as
begin
  set nocount on
--test
  insert into mandant 
              (mandid,
               name,
               beschreibung)
       values (@mandid,
               @name,
               isnull(@beschreibung,''))

end
go
print 'Procedure: set_mandant done ...'
go
grant exec on set_mandant to prsadmins with grant option
go
grant exec on set_mandant to prsusers
go


if exists (select 1 from sysobjects where name = 'set_neudienst' and type = 'P')
begin
   drop procedure set_neudienst
   print 'Procedure: set_neudienst deleted ...'
end
go
create procedure set_neudienst(
  @mandid            char(2)       = 'li',
  @dienstname        varchar(32)   = 'Januar',
  @kurztext          varchar(60)   = 'januar test',
  @beschreibung      varchar(250)  = 'januar besch test',
--  @einsatzprotag     int           = 1,
  @gerechnet         int           = 1,
  @RGB               int           = 0,
  @ColorName         varchar(32)   = '',
  @typ               int           = 1,
  @GanzerTag         int           = 0,
  @dienstid          int           = 0 output
)
as
begin
  set nocount on
  declare @beschrid   int
  if @beschreibung is null select @beschreibung = ''
  if not exists (select 1 from Beschreibung where Beschreibung.BeschrID between 1 and 500)
  begin
    select @beschrid = 1
  end
  else
  begin
    select @beschrid = max(Beschreibung.BeschrID) + 1
      from Beschreibung
     where Beschreibung.BeschrID between 1 and 500
  end
  exec set_beschreibung @beschrid, @kurztext, @beschreibung
  insert into Dienst (Name, BeschrID,Gerechnet,RGB,ColorName,Typ,Status,GanzerTag,mandid) values (@dienstname, @beschrid, @gerechnet, @RGB, @ColorName, @typ,0,@GanzerTag,@mandid)
  
  select @dienstid = DienstID
    from Dienst
   where Name = @dienstname
  
end
go
print 'Procedure: set_neudienst done ...'
go
grant exec on set_neudienst to prsadmins with grant option
go
grant exec on set_neudienst to prsusers
go


if exists (select 1 from sysobjects where name = 'set_neuplan' and type = 'P')
begin
   drop procedure set_neuplan
   print 'Procedure: set_neuplan deleted ...'
end
go
create procedure set_neuplan(
  @planname          varchar(60)   = 'Januar',
  @atid              int           = 1,
  @schemaid          int           = 1
)
as
begin
  set nocount on
  declare    @datumvon    datetime
  declare    @datumbis    datetime
  declare    @jahrid      int
  declare    @mandid      char(2)
  
  select @datumvon = Von from ArbeitsTage where ATID = @atid
  select @datumbis = Bis from ArbeitsTage where ATID = @atid
  select @jahrid = JahrID from ArbeitsTage where ATID = @atid
  select @mandid = mandid from ArbeitsTage where ATID = @atid
  
   -- Status 1: Aktiv   2: Inaktiv
   insert into ArbeitsPlan (ATID, Name, JahrID, SchemaID, Von, Bis, Status, mandid) 
        values (@atid, @planname, @jahrid, @schemaid, @datumvon, @datumbis, 1, @mandid)

   -- Returns the planid of the newly created plan
   select planid = ArbeitsPlan.PlanID
     from ArbeitsPlan
    where ArbeitsPlan.Von = @datumvon
      and ArbeitsPlan.Bis = @datumbis
      and ArbeitsPlan.Name = @planname
      and ArbeitsPlan.Status = 1

end
go
print 'Procedure: set_neuplan done ...'
go
grant exec on set_neuplan to prsadmins with grant option
go
grant exec on set_neuplan to prsusers
go



if exists (select 1 from sysobjects where name = 'set_newpensum' and type = 'P')
begin
   drop procedure set_newpensum
   print 'Procedure: set_newpensum deleted ...'
end
go
create procedure set_newpensum(
  @prsid             int           = 1,
  @pensum            int           = 1,
  @von               varchar(8)    = '00000000',
  @bis               varchar(8)    = '00000000'
)
as
begin
  set nocount on

  declare @pvon datetime
  declare @pbis datetime

  select @pvon = convert(datetime,@von,112)
  select @pbis = convert(datetime,@bis,112)

  -- reset pld bis to new von
  update Arbeitspensum
     set Bis = @pvon
   where PrsID = @prsid
     and Status = 1

  -- set all status to 2
  update Arbeitspensum
     set Status = 2
   where PrsID = @prsid

  -- ueberflssige pensen lschen
  delete from Arbeitspensum where Von = Bis and PrsID = @prsid
  delete from Arbeitspensum where Von >= @pvon and Bis >= @pbis and PrsID = @prsid
  delete from Arbeitspensum where Von >= @pvon and Bis >= @pvon and PrsID = @prsid
  
  update Arbeitspensum
     set Bis = @pvon
   where Bis > @pvon
     and Status = 2
     and PrsID = @prsid

  -- insert new pensum
  insert into Arbeitspensum (PrsID, Von, Bis, Prozent, Status) 
      values (@prsid, @pvon, @pbis, @pensum,1)

  -- Anstellungsdatum ndern
  update PersonAnstellung
     set Bis = @pbis--@pvon--@pbis
   where PrsID = @prsid
     and Bis > @pvon
     
  -- Person als angestellte darstellen
  if exists (select 1 from Person where PrsId = @prsid and PersonTyp <> 1)
	  update Person 
		 set PersonTyp = 1
	   where PrsId = @prsid

end
go
print 'Procedure: set_newpensum done ...'
go
grant exec on set_newpensum to prsadmins with grant option
go
grant exec on set_newpensum to prsusers
go

if exists (select 1 from sysobjects where name = 'set_newshemaplan' and type = 'P')
begin
   drop procedure set_newshemaplan
   print 'Procedure: set_newshemaplan deleted ...'
end
go
-- Erstellt ein neues Schema und einen neuen Plan
-- Wird nur benutzt wenn laufende schemas aktiviert sind
create procedure set_newshemaplan(
  @planname          varchar(60)   = 'Januar',
  @atid              int           = 25, --24
  @schemaid          int           = 1
)
as
begin
  set nocount on
  declare    @datumvon    datetime
  declare    @datumbis    datetime
  --declare    @oldshemaid  int
  declare    @newshemaid  int
  declare    @oldplanid   int
  
  declare    @mandid      char(2)
  
    select @mandid = mandid 
      from Schemas 
     where SchemaID = @schemaid
  
   select @datumvon = Von
     from ArbeitsTage
    where ATID = @atid
    
   select @datumbis = Bis
     from ArbeitsTage
    where ATID = @atid
   
   select @oldplanid = PlanID from ArbeitsPlan where ATID = @atid and SchemaID = @schemaid
   --select @oldshemaid = SchemaID from ArbeitsPlan where PlanID = @oldplanid
   
   create table #prs
   (
      prsschemaid int,
      prsid       int,
      abteilungid int,
      rgb         int,
      stat        int
   )
   
   create table #schema
   (
      prsschemaid int identity(1,1),
      prsid       int,
      abteilungid int,
      rgb         int
   )
   
   -- Altes Schema laden nur mit personen die noch angestellt sind
   insert #prs
   select prsschemaid = ps.PrsSchemaID,
          prsid = ps.PrsID,
          abteilungid = ps.AbteilungID,
          rgb = ISNULL(ps.RGB,0),
          stat = 1
     from PlanSchema ps
    inner join PersonalAbteilung pa
       on pa.PrsId = ps.PrsID
      and pa.AbteilungID = ps.AbteilungID
      and pa.Von <= @datumvon
      and pa.Bis >= @datumbis
      and (pa.Status = 1 or pa.Status is null)
    where ps.SchemaID = @schemaid--@oldshemaid

   -- Abteilungen aus dem alten schema laden
   insert #prs
   select prsschemaid = ps.PrsSchemaID,
          prsid = ps.PrsID,
          abteilungid = ps.AbteilungID,
          rgb = ISNULL(ps.RGB,0),
          stat = 1
     from PlanSchema ps
    where ps.PrsID <= 0
      and ps.SchemaID = @schemaid--@oldshemaid

     -- Zusatzpersonen sollten schon nicht mehr enthalten sein da diese kein pensum haben...
   --delete #prs
   --  from #prs
   -- inner join ArbeitsPlanzprs apz
   --    on apz.zprsID = #prs.prsid
   --   and apz.abteilungID = #prs.abteilungid
    
   -- Personen ins schema laden die neu angefangen haben oder im schema fehlen
   insert #prs
   select prsschemaid = 0,
          prsid = p.PrsId,
          abteilungid = pa.AbteilungID,
          rgb = 0,
          stat = 2
     from Person p
    inner join PersonalAbteilung pa
       on pa.PrsId = p.PrsId
      and pa.Von <= @datumvon
      and pa.Bis >= @datumbis
      and pa.AbteilungID in (select AbteilungID from PlanSchema ps where ps.SchemaID = @schemaid)
      
   -- alle Personen lschen die bereits vorhanden sind
   delete #prs
     from #prs p1
    inner join Person
       on Person.PrsId = p1.prsid
    inner join PersonalAbteilung pa
       on pa.PrsId = Person.PrsId
      and pa.Von <= @datumvon
      and pa.Bis >= @datumbis
   where p1.prsid in (select p2.prsid from #prs p2 where p2.stat = 1)
    and p1.stat = 2
    
   update #prs
      set prsschemaid = (select MAX(p3.prsschemaid)+1 from #prs p3 where p3.abteilungid = #prs.abteilungid)
     from #prs 
    where prsschemaid = 0
      and prsid > 0
     
    insert #schema
    select prsid = prsid,
           abteilungid = abteilungid,
           rgb = rgb
      from #prs
     order by abteilungid, prsschemaid
          
   update Schemas
      set Status = 0
    where SchemaID = @schemaid--@oldshemaid
   
   declare @tmpschemaid int
   select @tmpschemaid = MAX(SchemaID)+1 from Schemas
   
   --insert into Schemas (Name,Status,mandid) values ('Schema ' + isnull(CONVERT(varchar(5),@tmpschemaid),'1'),1,@mandid)
   insert into Schemas (Name,Status,mandid) values (@planname,1,@mandid)
   
   select @newshemaid = MAX(SchemaID) from Schemas where mandid = @mandid
      
   insert PlanSchema
   select SchemaID = @newshemaid,
          PrsSchemaID = prsschemaid,
          PrsID = prsid,
          AbteilungID = abteilungid,
          RGB = rgb,
          ColorName = '',
          Bezeichner = ''
     from #schema
          
   declare @jahrid  int
   
   select @jahrid = JahrID
     from ArbeitsTage at
    where at.ATID = @atid
    
   -- Neuer Arbeitsplan erstellen
   insert into ArbeitsPlan (ATID, Name, JahrID, SchemaID, Von, Bis, Status, mandid) 
        values (@atid, @planname, @jahrid, @newshemaid, @datumvon, @datumbis, 1, @mandid)

   -- Create/copy new Dienstschema
   insert DienstSchema
   select schemaid      = @newshemaid,
          abteilungid   = ds.AbteilungID,
          dienstid      = ds.DienstID,
          einsatzprotag = ds.EinsatzProTag,
          gerechnet     = ds.Gerechnet
     from DienstSchema ds
    where ds.SchemaID = @schemaid--@oldshemaid
   
   
   -- Neue PlanID zurck geben
   select planid = max(ArbeitsPlan.PlanID)
     from ArbeitsPlan
    where ArbeitsPlan.Von = @datumvon
      and ArbeitsPlan.Bis = @datumbis
      and ArbeitsPlan.Name = @planname
      and ArbeitsPlan.ATID = @atid
      and ArbeitsPlan.Status = 1

end
go
print 'Procedure: set_newshemaplan done ...'
go
grant exec on set_newshemaplan to prsadmins with grant option
go
grant exec on set_newshemaplan to prsusers
go


if exists (select 1 from sysobjects where name = 'set_options' and type = 'P')
begin
   drop procedure set_options
   print 'Procedure: set_options deleted ...'
end
go
create procedure set_options(
  @typin             varchar(30)   = 'Arbeitszeitprotag',
  @wertin            int           = 1,
  @zahlwertin        decimal(15,2) = 8,
  @textwert          varchar(255)  = 'Stunden pro Tag'
)
as
begin
  set nocount on
  
  if exists (select 1 from Options where typ = @typin and wert = @wertin)
  begin
   delete from Options where typ = @typin and wert = @wertin
  end
  
  insert into Options (typ,wert,zahlwert,textwert) values (@typin, @wertin, @zahlwertin, @textwert)
  
end

go
print 'Procedure: set_options done ...'
go
grant exec on set_options to prsadmins with grant option
go
grant exec on set_options to prsusers
go



if exists (select 1 from sysobjects where name = 'set_person' and type = 'P')
begin
   drop procedure set_person
   print 'Procedure: set_person deleted ...'
end
go
create procedure set_person(
  @mandid          char(2)     = 'li',
  @vorname         varchar(32) = 'name', 
  @nachname        varchar(32) = 'nachname', 
  @strasse         varchar(32) = 'strasse', 
  @strassennr      varchar(5)  = 'nr', 
  @postfach        varchar(10) = 'postfach', 
  @plz             char(5)     = 'plz', 
  @ort             varchar(32) = 'ort', 
  @telnrp          varchar(15) = 'telp', 
  @telnrn          varchar(15) = 'teln', 
  @email           varchar(60) = 'email@email.ch',
  @pensumvon       varchar(8)   = '20070101',
  @pensumbis       varchar(8)   = '20100101',
  @pensumprozent   smallint    = 100,
  @anstellungid    int         = 1,
  @persontyp       int         = 1
)
as
begin
   set nocount on
   declare @prsid         int
   declare @ferientage    decimal(5,2)
   declare @jahrid        int

   if @persontyp is null select @persontyp = 1

   if @strassennr = '' select @strassennr = null
   if @postfach = '' select @postfach =  null
   if @telnrp = '' select @telnrp = null
   if @telnrn = '' select @telnrn = null
   if @email = '' select @email = null
   if @pensumbis = '' select @pensumbis = null

   select @ferientage = isnull(Options.zahlwert,0) from Options where Options.typ = 'Ferientage' and wert = 0

   insert into Person (Vorname, Nachname, Strasse, Strassennr, Postfach, PLZ, Ort, TelnrP, TelnrN, Email, PersonTyp,mandid) values (@vorname, @nachname, @strasse, @strassennr, @postfach, @plz, @ort, @telnrp, @telnrn, @email, isnull(@persontyp,1), @mandid)

   select @prsid = p.PrsId from Person p
   where (p.Vorname = @vorname or @vorname is null)
     and (p.Nachname = @nachname or @nachname is null)
     and (p.Strasse = @strasse or @strasse is null)
     and (p.Strassennr = @strassennr or @strassennr is null)
     and (p.Postfach = @postfach or @postfach is null)
     and (p.PLZ = @plz or @plz is null)
     and (p.Ort = @ort or @ort is null)

if @persontyp = 1
begin
   if exists (select 1 from Arbeitspensum where PrsID = @prsid and (Bis = '' or Bis is null))
      begin
        update Arbeitspensum 
           set Bis = convert(datetime, @pensumbis, 112), 
               Status = 2 
         where PrsID = @prsid 
           and (Bis = '' or Bis is null) 
           and Von = (select max(Von) from Arbeitspensum 
                       where PrsID = @prsid 
                         and (Bis = '' or Bis is null))
      end
   else
      begin
        insert into Arbeitspensum 
                    (PrsID, 
                     Von, 
                     Bis, 
                     Prozent, 
                     Status) 
             values (@prsid, 
                     convert(datetime, 
                     @pensumvon, 112), 
                     convert(datetime, @pensumbis, 112), 
                     @pensumprozent,
                     1)
   end 

   exec set_prsanstellung @prsid, @anstellungid, @pensumvon, @pensumbis

   -- Arbeitszeiten abfllen
   select @jahrid = JahrID from PlanungsJahr where Status = 1
   if @jahrid is null select @jahrid = max(JahrID) from PlanungsJahr

   if exists (select 1 from PersonZeiten where PrsID = @prsid and JahrID = @jahrid)
   begin
      update PersonZeiten 
         set FerienSaldo = ((@ferientage * @pensumprozent)/100) 
       where PersonZeiten.PrsID = @prsid
   end
   else
   begin
--TODO:
--arbeitszeitsaldo noch einbauen...
      insert into PersonZeiten (PrsID,FerienSaldo,ArbeitszeitSaldo,JahrID) 
           values (@prsid,((@ferientage * @pensumprozent)/100),0,@jahrid)
   end
end
   -- Return the PrsID
   select prsid = @prsid

end
go
print 'Procedure: set_person done ...'
go
grant exec on set_person to prsadmins with grant option
go
grant exec on set_person to prsusers
go



if exists (select 1 from sysobjects where name = 'set_personkuendigung' and type = 'P')
begin
   drop procedure set_personkuendigung
   print 'Procedure: set_personkuendigung deleted ...'
end
go

create procedure set_personkuendigung(
  @prsid       int   = 1,
  @bis         varchar(8)   = '20090101'
)
as
begin

   set nocount on
   
   update Arbeitspensum
      set Bis = convert(datetime, @bis, 112),
          Status = 2
    where PrsID = @prsid 
      --and Status = 1
      and Bis >= convert(datetime, @bis, 112)

   update PersonAnstellung
      set Bis = convert(datetime, @bis, 112)
   where PrsID = @prsid
     and Bis >= convert(datetime, @bis, 112)
   
   update PersonalAbteilung
   set Bis = convert(datetime, @bis, 112),
       Status = 2
   where PrsId = @prsid
     and Bis >= convert(datetime, @bis, 112)

end
go
print 'Procedure: set_personkuendigung done ...'
go
grant exec on set_personkuendigung to prsadmins with grant option
go
grant exec on set_personkuendigung to prsusers
go



if exists (select 1 from sysobjects where name = 'set_personzeitennew' and type = 'P')
begin
   drop procedure set_personzeitennew
   print 'Procedure: set_personzeitennew deleted ...'
end
go

create procedure set_personzeitennew(
  @prsid         int   = 1,
  @oldjahrid     int   = 1
)
as
begin

   set nocount on

   declare @ferientage    decimal(5,2)
   declare @jahrid        int
   declare @proz          smallint

   declare @oldsaldo      decimal(5,2)
   declare @oldferien     decimal(5,2)

   select @jahrid = JahrID 
     from PlanungsJahr 
    where Status = 1
    
   if @jahrid is null select @jahrid = max(JahrID) from PlanungsJahr
   
   select @proz = ap.Prozent 
     from Arbeitspensum ap 
    where ap.Status = 1 
      and ap.PrsID = @prsid
      
   select @ferientage = isnull(Options.zahlwert,0) 
     from Options 
    where Options.typ = 'Ferientage' 
      and wert = 0

   select @oldsaldo = isnull(ArbeitszeitSaldo,0) 
     from PersonZeiten 
    where JahrID = @oldjahrid 
      and PrsID = @prsid
      
   select @oldferien = isnull(FerienSaldo,0) 
     from PersonZeiten 
    where JahrID = @oldjahrid 
      and PrsID = @prsid

   if not exists (select 1 from PersonZeiten where PrsID = @prsid and JahrID = @jahrid)
   begin
     insert into PersonZeiten
                 (PrsID, 
                  FerienSaldo, 
                  ArbeitszeitSaldo, 
                  JahrID) 
          values (@prsid,
                 ((@ferientage * ISNULL(@proz,100))/100) + isnull(@oldferien,0), 
                 isnull(@oldsaldo,0), 
                 @jahrid)
   end

end
go
print 'Procedure: set_personzeitennew done ...'
go
grant exec on set_personzeitennew to prsadmins with grant option
go
grant exec on set_personzeitennew to prsusers
go


if exists (select 1 from sysobjects where name = 'set_plandienstserie' and type = 'P')
begin
   drop procedure set_plandienstserie
   print 'Procedure: set_plandienstserie deleted ...'
end
go
create procedure set_plandienstserie(
    @planid     int
)
as
begin
  set nocount on

  declare @datumvon   datetime
  declare @datumbis   datetime

  create table #dienstserie
  (
     prsid         int,
     dienstid      int,
     abteilungid   int,
     tag           int,
     typ           int,
     von           datetime,
     bis           datetime
  )
  
  select @datumvon = Von,
         @datumbis = Bis
    from ArbeitsPlan
   where PlanID = @planid

  insert #dienstserie
  select prsid = ds.PrsID,
         dienstid = ds.DienstID,
         abteilungid = ds.AbteilungID,
         tag = ds.Tag,
         typ = ds.Typ,
         von = ds.Von, 
         bis = ds.bis 
   from DienstSerie ds
  where ((Von <= @datumvon and bis >= @datumbis)          -- bis
       or (Von >= @datumvon and bis >= @datumbis and Von <= @datumbis)
       or (Von <= @datumvon and bis >= @datumbis)
			           										-- von
       or (Von <= @datumvon and bis <= @datumbis and bis >= @datumvon)
       or (Von >= @datumvon and bis <= @datumbis))
--    and ds.abteilungid = @abteilungid
--    and ds.prsid = @prsid

end
go
print 'Procedure: set_plandienstserie done ...'
go

grant exec on set_plandienstserie to prsadmins with grant option
go
grant exec on set_plandienstserie to prsusers
go


if exists (select 1 from sysobjects where name = 'set_planschema' and type = 'P')
begin
   drop procedure set_planschema
   print 'Procedure: set_planschema deleted ...'
end
go
-- setzt anzahl tage pro monat fest
create procedure set_planschema(
  @SchemaID        int = 1,
  @PrsSchemaID     int = 1,-- nummer fortlaufend, fngt bei jedem schema neu an
  @PrsID           int = 1,
  @AbteilungID     int = 1,
  @RGB             int = null,
  @ColorName       varchar(32) = '',
  @Bezeichner      varchar(32) = ''
)
as
begin

  set nocount on

  insert into PlanSchema (SchemaID, PrsSchemaID,PrsID,AbteilungID,RGB,ColorName,Bezeichner) values (@SchemaID,@PrsSchemaID,@PrsID,@AbteilungID,@RGB,@ColorName,@Bezeichner)

end
go

print 'Procedure: set_planschema done ...'
go

grant exec on set_planschema to prsadmins with grant option
go
grant exec on set_planschema to prsusers
go


if exists (select 1 from sysobjects where name = 'set_planungsjahr' and type = 'P')
begin
   drop procedure set_planungsjahr
   print 'Procedure: set_planungsjahr deleted ...'
end
go

create procedure set_planungsjahr(
  @jahr          int   = 1900,
  @mandid        char(2) = 'li',
  @von           varchar(8) = '20070101',
  @bis           varchar(8) = '20071231'
)
as
begin

  set nocount on

  declare @datevon datetime
  declare @datebis datetime

  select @datevon = convert(datetime, @von, 112)
  select @datebis = convert(datetime, @bis, 112)

  -- Alle alten jahre status auf 2 setzten
  update PlanungsJahr set Status = 2 where Status = 1

  -- neues jahr erstellen mit aktivem status
  insert into PlanungsJahr (Jahr,von,bis,mandid,Status) values (@jahr,@datevon,@datebis,@mandid,1)
  
  select * from PlanungsJahr

end
go
print 'Procedure: set_planungsjahr done ...'
go
grant exec on set_planungsjahr to prsadmins with grant option
go
grant exec on set_planungsjahr to prsusers
go



if exists (select 1 from sysobjects where name = 'set_prsabteilung' and type = 'P')
begin
   drop procedure set_prsabteilung
   print 'Procedure: set_prsabteilung deleted ...'
end
go
create procedure set_prsabteilung(
  @abteilungid      int   = 1,
  @prsid            int   = 1,
  @von              char(8) = '20070501',
  @bis              char(8) = '20070501',
  @prozent          int     = 0
)
as
begin
  set nocount on
--   if exists (select 1 from personalabteilung where abteilungid = @abteilungid and prsid = @prsid)
--   begin
--      update personalabteilung
--         set bis = @von
--       where abteilungid = @abteilungid 
--         and prsid = @prsid
--         and bis = (select max(pa.bis) from personalabteilung pa where pa.abteilungid = @abteilungid and pa.prsid = @prsid)
--   end

   insert into PersonalAbteilung (AbteilungID, PrsId, Von, Bis, Prozent, Status) values (@abteilungid, @prsid, @von, @bis, @prozent, 1)
end
go
print 'Procedure: set_prsabteilung done ...'
go
grant exec on set_prsabteilung to prsadmins with grant option
go
grant exec on set_prsabteilung to prsusers
go


if exists (select 1 from sysobjects where name = 'set_prsanstellung' and type = 'P')
begin
   drop procedure set_prsanstellung
   print 'Procedure: set_prsanstellung deleted ...'
end
go
create procedure set_prsanstellung(
  @prsid             int   = 1,
  @anstellungid      int   = 1,
  @von               varchar(8) = '20070101',
  @bis               varchar(8) = '20200101'
)
as
begin
  set nocount on

  declare @datevon  datetime
  declare @datebis  datetime
 
  select @datevon = convert(datetime, @von, 112)
  select @datebis = convert(datetime, @bis, 112)

   insert into PersonAnstellung (PrsID,AnstellungID,Von,Bis) values (@prsid, @anstellungid, @datevon,@datebis)
end
go
print 'Procedure: set_prsanstellung done ...'
go
grant exec on set_prsanstellung to prsadmins with grant option
go
grant exec on set_prsanstellung to prsusers
go


if exists (select 1 from sysobjects where name = 'set_recalculateplanzeit' and type = 'P')
begin
   drop procedure set_recalculateplanzeit
   print 'Procedure: set_recalculateplanzeit deleted ...'
end
go

create procedure set_recalculateplanzeit(
  @planid         int = 1,
  @prsid          int = 1,
  @abteilungid    int = 1
)
as
begin
  set nocount on

   declare @datum datetime
   --select @dte = convert(datetime,@datum,112)
   select @datum = Bis from ArbeitsPlan where PlanID = @planid
   
     --recalculate saldos
     create table #dienstzeiten(
        dienstid       int,
        zeit           decimal(8,2),
        typ            char(1)
     )
     
    -- Zeiten berechnen
    declare @abteilungszeit  decimal(5,2)
    declare @arbeitstage     decimal(5,2)
    declare @pensum          decimal(5,2)
    declare @soll            decimal(5,2)
    declare @haben           decimal(5,2)
    declare @ferien          decimal(5,2)

     select @abteilungszeit = stundenprotag from Abteilung where AbteilungID = @abteilungid

     -- zeiten in decimal konvertieren
     insert #dienstzeiten
     select dienstid = dz.DienstID,
            zeit     = case when dz.Von < dz.Bis then
                                --datepart(hh,dz.bis - dz.von) + (convert(decimal(5,2), datepart(mi,dz.bis - dz.von))/100)
                                datepart(hh,dz.Bis - dz.Von) + (convert(decimal(5,2), ((datepart(mi,dz.Bis)*100/60)- (datepart(mi,dz.Von)*100/60) ))/100)
                            else
                                --datepart(hh,dz.von - dz.bis) + (convert(decimal(5,2), datepart(mi,dz.von - dz.bis))/100)
                                datepart(hh,dz.Von - dz.Bis) + (convert(decimal(5,2), ((datepart(mi,dz.Von)*100/60) - (datepart(mi,dz.Bis)*100/60) ))/100)
                            end,
            typ      = dz.Zeiteinheit
       from DienstZeit dz
      where dz.datvon <= @datum
        and dz.datbis > @datum --bis und von knnen am 1. gleich gross sein wenn dienstzeit gendert wird
      
     -- Zeiten aus selbem dienst addieren
     insert #dienstzeiten
     select dienstid = dz.dienstid,
            zeit     = sum(zeit),
            typ      = '1'
       from #dienstzeiten dz
      where dz.typ = 'Z'
      group by dz.dienstid, typ
      
      -- pausen aus selbem dienst addieren
      insert #dienstzeiten
     select dienstid = dz.dienstid,
            zeit     = sum(zeit),
            typ      = '2'
       from #dienstzeiten dz
      where dz.typ = 'P'
      group by dz.dienstid, typ
      
      -- zeiten - pausen
      insert #dienstzeiten
     select dienstid = dz1.dienstid,
            zeit     = isnull(dz1.zeit,0)-isnull(dz2.zeit,0),
            typ      = 'F'
       from #dienstzeiten dz1
       left outer join #dienstzeiten dz2
         on dz2.dienstid = dz1.dienstid
        and dz2.typ = '2'
      where dz1.typ = '1'
        
     -- berschssige eintrge lschen
     delete from #dienstzeiten where typ <> 'F'

     -- Zeit von Abteilung nehmen wo ntig
     update #dienstzeiten
        set zeit = @abteilungszeit
       from #dienstzeiten
      inner join Dienst d
         on d.DienstID = #dienstzeiten.dienstid
      inner join DienstZeit dz
         on dz.DienstID = d.DienstID
        and dz.datvon <= @datum
        and dz.datbis > @datum
      where dz.abteilungzeit = 1

       -- haben
   select @haben = isnull(sum(isnull(dz.zeit,0)),0)
     from Dienstplan dp 
    inner join Dienst d
       on d.DienstID = dp.DienstID
    inner join #dienstzeiten dz
       on dz.dienstid = dp.DienstID
    where dp.PlanID = @planid
      and dp.PrsID = @prsid
      and dp.AbteilungID = @abteilungid
      and d.Typ <> 3 -- Ferien
      and d.Gerechnet = 1
        
     -- ferien
   select @ferien = isnull(sum(isnull(dz.zeit,0)),0)
     from Dienstplan dp 
    inner join Dienst d
       on d.DienstID = dp.DienstID
    inner join #dienstzeiten dz
       on dz.dienstid = dp.DienstID
    where dp.PlanID = @planid
      and dp.PrsID = @prsid
      and dp.AbteilungID = @abteilungid
      and d.Typ = 3 --Ferien
      and d.Gerechnet = 1


       -- Soll berechnen
       select @arbeitstage = Tage 
         from ArbeitsTage at
        inner join PlanungsJahr pj
           on pj.JahrID = at.JahrID
          and (@datum >= pj.von and @datum <= pj.bis)
        inner join ArbeitsPlan ap
           on ap.ATID = at.ATID
        where ap.PlanID = @planid
   
       select @pensum = isnull(nullif(pa.Prozent,0) ,isnull(a.Prozent,0))
         from Arbeitspensum a,
              PersonalAbteilung pa
        where pa.AbteilungID = @abteilungid
          and a.PrsID = @prsid
          and a.Von <= @datum
          and a.Bis >= @datum
          and pa.PrsId = a.PrsID
          and pa.Von <= @datum
          and pa.Bis >= @datum
          
        if @pensum is null
            select @pensum = 0--100
      
       --(Data.Monat.MonatData[atIndex].Tage / 100) * pensum * hours
       select @abteilungszeit = max(stundenprotag) from Abteilung where AbteilungID = @abteilungid

       select @soll = isnull(((@arbeitstage * @pensum)/100) * @abteilungszeit,0)
       
       if @pensum = 0 select @soll = 0
       
     -- Zeiten einfgen
     if exists (select * from Arbeitszeit 
                        where PrsID = @prsid 
                          and PlanID = @planid 
                          and AbteilungID = @abteilungid)
     begin
     
       update Arbeitszeit
          set Haben = @haben,
              Ferien = @ferien,
              Soll = @soll
        where PlanID = @planid
          and PrsID = @prsid
          and AbteilungID = @abteilungid
        
     end
     else
     begin -- Insert
     
      insert Arbeitszeit
             (PrsID,
              PlanID,
              AbteilungID,
              Soll,
              Haben,
              Ferien)
      values (@prsid,
              @planid,
              @abteilungid,
              @soll,
              @haben,
              @ferien)
     end
     
end
go
print 'Procedure: set_recalculateplanzeit done ...'
go
grant exec on set_recalculateplanzeit to prsadmins with grant option
go
grant exec on set_recalculateplanzeit to prsusers
go


if exists (select 1 from sysobjects where name = 'set_saldoanpassung' and type = 'P')
begin
   drop procedure set_saldoanpassung
   print 'Procedure: set_saldoanpassung deleted ...'
end
go
-- setzt anzahl tage pro monat fest
create procedure set_saldoanpassung(
  @PlanID          int = 1,
  @abteilungid     int = 1,
  @PrsID           int = 1,
  @Datum           varchar(8) = '20071111',
  @Grund           varchar(250) = 'zu spt',
  @Stunden         decimal(7,2) = 5,
  @retval          int = 0 output 
)
as
begin

  set nocount on

  declare @date datetime

  select @date = convert(datetime, @Datum, 112)
--sp_help saldoanpassung
  insert into Saldoanpassung (PlanID,AbteilungID,PrsID,Datum,Grund,Stunden) values (@PlanID,@abteilungid,@PrsID,@date,@Grund,@Stunden)
  
  select @retval = max(SaldoID)
    from Saldoanpassung
   where PlanID = @PlanID
     and AbteilungID = @abteilungid
     and PrsID = @PrsID
     and Datum = @date
     and Grund = @Grund
     and Stunden = @Stunden

end
go

print 'Procedure: set_saldoanpassung done ...'
go

grant exec on set_saldoanpassung to prsadmins with grant option
go
grant exec on set_saldoanpassung to prsusers
go



if exists (select 1 from sysobjects where name = 'set_schemas' and type = 'P')
begin
   drop procedure set_schemas
   print 'Procedure: set_schemas deleted ...'
end
go
-- setzt anzahl tage pro monat fest
create procedure set_schemas(
  @name            varchar(32) = 'oktober',
  @mandid          char(2)     = 'li'
)
as
begin

  set nocount on

  insert into Schemas (Name, Status, mandid) values (@name, 1, @mandid)
  
  select schemaid = s.SchemaID 
    from Schemas s
   where s.Name = @name 
     and s.Status = 1 
     and s.SchemaID = (select max(s2.SchemaID) from Schemas s2 where s2.Name = s.Name and s2.Status = 1 and s2.mandid = @mandid)
     and s.mandid = @mandid

end
go

print 'Procedure: set_schemas done ...'
go

grant exec on set_schemas to prsadmins with grant option
go
grant exec on set_schemas to prsusers
go


if exists (select 1 from sysobjects where name = 'set_termin' and type = 'P')
begin
   drop procedure set_termin
   print 'Procedure: set_termin deleted ...'
end
go
create procedure set_termin(
  @terminname        varchar(32)   = 'Sitzung',
  @kurztext          varchar(60)   = 'Sitzung test',
  @beschreibung      varchar(250)  = 'Sitzung besch test',
  @rgbin             int           = 0,
  @colorname         varchar(32)   = 'white',
  @shgin             char(1)       = 'S', -- S: Schrift   H: Hintergrund
  @mandid            char(2)       = 'li'
)
as
begin
  set nocount on
  declare @beschrid   int
  if @beschreibung is null select @beschreibung = ''
  if not exists (select 1 from Beschreibung where Beschreibung.BeschrID between 1501 and 2000)
  begin
    select @beschrid = 1501
  end
  else
  begin
    select @beschrid = max(Beschreibung.BeschrID) + 1
      from Beschreibung
     where Beschreibung.BeschrID between 1501 and 2000
  end
  
  exec set_beschreibung @beschrid, @kurztext, @beschreibung
  
  insert into Termin (Name, BeschrID, Status, DefRGB, Colorname, SHG, mandid) values (@terminname, @beschrid, 1, @rgbin, @colorname, upper(@shgin), @mandid)
  
end
go
print 'Procedure: set_termin done ...'
go
grant exec on set_termin to prsadmins with grant option
go
grant exec on set_termin to prsusers
go


if exists (select 1 from sysobjects where name = 'set_terminplan' and type = 'P')
begin
   drop procedure set_terminplan
   print 'Procedure: set_terminplan deleted ...'
end
go
create procedure set_terminplan(
   @planterminid   int  = 1,
   @planID         int  = 1,
   @prsID          int  = 1,
   @name           varchar(32) = 'Erster',
   @datum          char(8) = '20070501',
   @abteilungID    int  = 1,
   @terminid       int  = 1
--   @rgbin          varchar(32) = '00000000'
)
as
begin
  set nocount on

--   delete from terminplan where planterminid = @planterminid
   insert into Terminplan (PlanTerminID, PlanID, PrsID, Datum, AbteilungID, TerminID, Name) values (@planterminid, @planID, @prsID, @datum, @abteilungID, @terminid, @name)


end

go
print 'Procedure: set_terminplan done ...'
go
grant exec on set_terminplan to prsadmins with grant option
go
grant exec on set_terminplan to prsusers
go


	if exists (select 1 from sysobjects where name = 'set_updsetcolor' and type = 'P')
begin
   drop procedure set_updsetcolor
   print 'Procedure: set_updsetcolor deleted ...'
end
go
create procedure set_updsetcolor(
  @planid         int = 1,
  @prsid          int = 1,
  @abteilungid    int = 1,
  @datum          char(8) = '20080702',
  @argb           int = 0
)
as
begin
  set nocount on

   declare @dte datetime
   select @dte = convert(datetime,@datum,112)

   if @argb = 0
   begin --delete dienst
     delete from ColorPlan
     where PlanID = @planid
       and PrsID = @prsid
       and Datum = @dte
       and AbteilungID = @abteilungid
   end
   else --add dienst
   begin
     if exists (select * from ColorPlan dp
                 where dp.PlanID = @planid
                   and dp.PrsID = @prsid
                   and dp.Datum = @dte
                   and dp.AbteilungID = @abteilungid)
       begin
         update ColorPlan
            set ARGB = @argb,
                Colorname = ''
          where PlanID = @planid
            and PrsID = @prsid
            and Datum = @dte
            and AbteilungID = @abteilungid
       end
       else --new record
       begin
         insert ColorPlan
                (PlanID,
                 PrsID,
                 Datum,
                 AbteilungID,
                 Colorname,
                 ARGB)
         values (@planid,
                 @prsid,
                 @dte,
                 @abteilungid,
                 '',
                 @argb)
       end
     end
     
end
go
print 'Procedure: set_updsetcolor done ...'
go

grant exec on set_updsetcolor to prsadmins with grant option
go
grant exec on set_updsetcolor to prsusers
go


if exists (select 1 from sysobjects where name = 'set_updsetdienst' and type = 'P')
begin
   drop procedure set_updsetdienst
   print 'Procedure: set_updsetdienst deleted ...'
end
go
-- exec set_updsetdienst 27,30,1,'20091109',0
-- delete from Arbeitszeit where PrsID = 30 and PlanID = 27 and AbteilungID = 1
-- select * from Arbeitszeit where PrsID = 30 and PlanID = 27 and AbteilungID = 1
create procedure set_updsetdienst(
  @planid         int = 1,
  @prsid          int = 1,
  @abteilungid    int = 1,
  @datum          char(8) = '20080702',
  @dienstid       int = 8
)
as
begin
  set nocount on

   declare @dte datetime
   select @dte = convert(datetime,@datum,112)
   
   if @dienstid = 0
   begin --delete dienst
     delete from Dienstplan
     where PlanID = @planid
       and PrsID = @prsid
       and Datum = @dte
       and AbteilungID = @abteilungid
   end
   else --add dienst
   begin
     if exists (select * from Dienstplan dp
                 where dp.PlanID = @planid
                   and dp.PrsID = @prsid
                   and dp.Datum = @dte
                   and dp.AbteilungID = @abteilungid)
       begin
         update Dienstplan
            set DienstID = @dienstid
          where PlanID = @planid
            and PrsID = @prsid
            and Datum = @dte
            and AbteilungID = @abteilungid
       end
       else --new record
       begin
         insert Dienstplan
                (PlanID,
                 PrsID,
                 Datum,
                 AbteilungID,
                 DienstID)
         values (@planid,
                 @prsid,
                 @dte,
                 @abteilungid,
                 @dienstid)
       end
     end
     
     
     --recalculate saldos
     create table #dienstzeiten(
        dienstid       int,
        zeit           decimal(8,2),
        typ            char(1)
     )
     
    -- Zeiten berechnen
    declare @abteilungszeit  decimal(5,2)
    declare @arbeitstage     decimal(5,2)
    declare @pensum          decimal(5,2)
    declare @soll            decimal(5,2)
    declare @haben           decimal(5,2)
    declare @ferien          decimal(5,2)

     select @abteilungszeit = stundenprotag from Abteilung where AbteilungID = @abteilungid

     -- zeiten in decimal konvertieren
     insert #dienstzeiten
     select dienstid = dz.DienstID,
            zeit     = case when dz.Von < dz.Bis then
                                --datepart(hh,dz.bis - dz.von) + (convert(decimal(5,2), datepart(mi,dz.bis - dz.von))/100)
                                datepart(hh,dz.Bis - dz.Von) + (convert(decimal(5,2), ((datepart(mi,dz.Bis)*100/60)- (datepart(mi,dz.Von)*100/60) ))/100)
                            else
                                --datepart(hh,dz.von - dz.bis) + (convert(decimal(5,2), datepart(mi,dz.von - dz.bis))/100)
                                datepart(hh,dz.Von - dz.Bis) + (convert(decimal(5,2), ((datepart(mi,dz.Von)*100/60) - (datepart(mi,dz.Bis)*100/60) ))/100)
                            end,
            typ      = dz.Zeiteinheit
       from DienstZeit dz
      where dz.datvon <= @datum
        and dz.datbis > @datum --bis und von knnen am 1. gleich gross sein wenn dienstzeit gendert wird
      
     -- Zeiten aus selbem dienst addieren
     insert #dienstzeiten
     select dienstid = dz.dienstid,
            zeit     = sum(zeit),
            typ      = '1'
       from #dienstzeiten dz
      where dz.typ = 'Z'
      group by dz.dienstid, typ
      
      -- pausen aus selbem dienst addieren
      insert #dienstzeiten
     select dienstid = dz.dienstid,
            zeit     = sum(zeit),
            typ      = '2'
       from #dienstzeiten dz
      where dz.typ = 'P'
      group by dz.dienstid, typ
      
      -- zeiten - pausen
      insert #dienstzeiten
     select dienstid = dz1.dienstid,
            zeit     = isnull(dz1.zeit,0)-isnull(dz2.zeit,0),
            typ      = 'F'
       from #dienstzeiten dz1
       left outer join #dienstzeiten dz2
         on dz2.dienstid = dz1.dienstid
        and dz2.typ = '2'
      where dz1.typ = '1'
        
     -- berschssige eintrge lschen
     delete from #dienstzeiten where typ <> 'F'

     -- Zeit von Abteilung nehmen wo ntig
     update #dienstzeiten
        set zeit = @abteilungszeit
       from #dienstzeiten
      inner join Dienst d
         on d.DienstID = #dienstzeiten.dienstid
      inner join DienstZeit dz
         on dz.DienstID = d.DienstID
        and dz.datvon <= @datum
        and dz.datbis > @datum
      where dz.abteilungzeit = 1

       -- haben
   select @haben = isnull(sum(isnull(dz.zeit,0)),0)
     from Dienstplan dp 
    inner join Dienst d
       on d.DienstID = dp.DienstID
    inner join #dienstzeiten dz
       on dz.dienstid = dp.DienstID
    where dp.PlanID = @planid
      and dp.PrsID = @prsid
      and dp.AbteilungID = @abteilungid
      and d.Typ <> 3 -- Ferien
      and d.Gerechnet = 1
        
     -- ferien
   select @ferien = isnull(sum(isnull(dz.zeit,0)),0)
     from Dienstplan dp 
    inner join Dienst d
       on d.DienstID = dp.DienstID
    inner join #dienstzeiten dz
       on dz.dienstid = dp.DienstID
    where dp.PlanID = @planid
      and dp.PrsID = @prsid
      and dp.AbteilungID = @abteilungid
      and d.Typ = 3 --Ferien
      and d.Gerechnet = 1


       -- Soll berechnen
       select @arbeitstage = Tage 
         from ArbeitsTage at
        inner join PlanungsJahr pj
           on pj.JahrID = at.JahrID
          and @dte between pj.von and pj.bis
        inner join ArbeitsPlan ap
           on ap.ATID = at.ATID
        where ap.PlanID = @planid
   
       select @pensum = isnull(nullif(pa.Prozent,0) ,isnull(a.Prozent,0))
         from Arbeitspensum a,
              PersonalAbteilung pa
        where pa.AbteilungID = @abteilungid
          and a.PrsID = @prsid
          and a.Von <= @dte
          and a.Bis > @dte
          and pa.PrsId = a.PrsID
          and pa.Von <= @dte
          and pa.Bis > @dte
          
        if @pensum is null
            select @pensum = 0--100
      
       --(Data.Monat.MonatData[atIndex].Tage / 100) * pensum * hours
       select @abteilungszeit = max(stundenprotag) from Abteilung where AbteilungID = @abteilungid

       select @soll = isnull(((@arbeitstage * @pensum)/100) * @abteilungszeit,0)
       
       if @pensum = 0 select @soll = 0
       
     -- Zeiten einfgen
     if exists (select * from Arbeitszeit 
                        where PrsID = @prsid 
                          and PlanID = @planid 
                          and AbteilungID = @abteilungid)
     begin
     
       update Arbeitszeit
          set Haben = @haben,
              Ferien = @ferien,
              Soll = @soll
        where PlanID = @planid
          and PrsID = @prsid
          and AbteilungID = @abteilungid
        
     end
     else
     begin -- Insert
     
       ---- Soll berechnen
       --select @arbeitstage = Tage 
       --  from ArbeitsTage at
       -- inner join PlanungsJahr pj
       --    on pj.JahrID = at.JahrID
       --   and @dte between pj.von and pj.bis
       -- inner join ArbeitsPlan ap
       --    on ap.ATID = at.ATID
       -- where ap.PlanID = @planid
   
       --select @pensum = isnull(nullif(pa.Prozent,0) ,isnull(a.Prozent,0))
       --  from Arbeitspensum a,
       --       PersonalAbteilung pa
       -- where pa.AbteilungID = @abteilungid
       --   and a.PrsID = @prsid
       --   and a.Von <= @dte
       --   and a.Bis > @dte
       --   and pa.PrsId = a.PrsID
       --   and pa.Von <= @dte
       --   and pa.Bis > @dte
          
       -- if @pensum is null
       --     select @pensum = 100
      
       ----(Data.Monat.MonatData[atIndex].Tage / 100) * pensum * hours
       --select @abteilungszeit = max(stundenprotag) from Abteilung --where AbteilungID = @abteilungid

       --select @soll = isnull(((@arbeitstage * @pensum)/100) * @abteilungszeit,0)
       
      insert Arbeitszeit
             (PrsID,
              PlanID,
              AbteilungID,
              Soll,
              Haben,
              Ferien)
      values (@prsid,
              @planid,
              @abteilungid,
              @soll,
              @haben,
              @ferien)
     end
     
end
go
print 'Procedure: set_updsetdienst done ...'
go
grant exec on set_updsetdienst to prsadmins with grant option
go
grant exec on set_updsetdienst to prsusers
go


if exists (select 1 from sysobjects where name = 'set_userrights' and type = 'P')
begin
   drop procedure set_userrights
   print 'Procedure: set_userrights deleted ...'
end
go
create procedure set_userrights(
  @username          varchar(35)   = 'planer',
  @loginname         varchar(35)   = 'planer',
  @password          varchar(256)  = 'personalplaner',
  @beschreibung      varchar(250)  = 'Haupt User',
  @userrights        int           = 1,
  @mandid            char(2)       = 'PA',
  @retval            int           = 0 output
)
as
begin
  set nocount on

  insert into userrights 
              (username,
               loginname,
               password,
               beschreibung,
               userrights,
               mandid)
       values (@username,
               @loginname,
               convert(varbinary(256), pwdencrypt(ISNULL(@password,''))),
               ISNULL(@beschreibung,''),
               @userrights,
               @mandid)

end
go
print 'Procedure: set_userrights done ...'
go
grant exec on set_userrights to prsadmins with grant option
go
grant exec on set_userrights to prsusers
go


if exists (select 1 from sysobjects where name = 'set_zeit' and type = 'P')
begin
   drop procedure set_zeit
   print 'Procedure: set_zeit deleted ...'
end
go
create procedure set_zeit(
  @dienstID             int           = 1,
  @von               char(8)       = '20:30:01',--'00000000',
  @bis               char(8)       = '20:30:05',
  @zeiteinheit       char(1)       = 'P',
  @datevon           char(8)       = '20090101',
  @datebis           char(8)       = '20300101',
  @abteilungszeit    int           = 0
)
as
begin
  set nocount on
   if @dienstID = 0 or @dienstID = '' select @dienstID = null




---- delete all that have the same begin date as end date
--  delete from DienstZeit where DienstID = @dienstID and datbis = datvon
--  delete from DienstZeit where DienstID = @dienstID and datvon >= @datevon

--  update DienstZeit
--     set datbis = @datevon
--   where DienstID = @dienstID
--     and datbis >= @datevon
--     --and Von <= @datebis

--select * from DienstZeit

   if (@dienstID is not null)
   begin
      insert into DienstZeit (DienstID, Von, Bis, Zeiteinheit,datvon,datbis,abteilungzeit) 
          values (@dienstID, convert(datetime, @von, 108), convert(datetime, @bis, 108), @zeiteinheit, @datevon,@datebis,@abteilungszeit)
   end
   
   
   
   
   
end
go
print 'Procedure: set_zeit done ...'
go
grant exec on set_zeit to prsadmins with grant option
go
grant exec on set_zeit to prsusers
go


if exists (select 1 from sysobjects where name = 'set_zusatzperson' and type = 'P')
begin
   drop procedure set_zusatzperson
   print 'Procedure: set_zusatzperson deleted ...'
end
go
create procedure set_zusatzperson(
  @nachname          varchar(32)   = 'Nachname',
  @vorname           varchar(32)   = 'Vorname',
  @planid            int           = 1,
  @abteilungid       int           = 1
)
as
begin
  set nocount on

  declare @prsid int

   if not exists (select 1 from Person where Nachname = @nachname and Vorname = @vorname and PersonTyp = 2)
   begin
     insert into Person (Vorname, Nachname, Strasse, Strassennr, Postfach, PLZ, Ort, TelnrP, TelnrN, Email, PersonTyp) values (@vorname, @nachname, '', '', '', '', '', '', '', '', 2/*zusatzperson*/)
   end

   select @prsid = PrsId from Person where Nachname = @nachname and Vorname = @vorname and PersonTyp = 2

   if not exists (select 1 from ArbeitsPlanzprs where planID = @planid and zprsID = isnull(@prsid,0) and abteilungID = @abteilungid)
   begin
     insert into ArbeitsPlanzprs(planID, zprsID, abteilungID) values (@planid, @prsid, @abteilungid)
     insert into Arbeitszeit(PrsID,PlanID,AbteilungID,Soll,Haben,Ferien) values (@prsid,@planid,@abteilungid,0,0,0)
   end

end
go
print 'Procedure: set_zusatzperson done ...'
go
grant exec on set_zusatzperson to prsadmins with grant option
go
grant exec on set_zusatzperson to prsusers
go


if exists (select 1 from sysobjects where name = 'set_zusatzpersonplan' and type = 'P')
begin
   drop procedure set_zusatzpersonplan
   print 'Procedure: set_zusatzpersonplan deleted ...'
end
go
create procedure set_zusatzpersonplan(
  @planid            int           = 18,
  @prsid            int            = 26,
  @abteilungid       int           = 1
)
as
begin
  set nocount on

  --insert into arbeitsplanzprs(planid, zprsid, abteilungid) values (@planid, @zprsid, @abteilungid)
  --insert into Arbeitszeit(prsid,planid,abteilungid,soll,haben,ferien) values (@zprsid,@planid,@abteilungid,0,0,0)

  declare @maxprsschemaid    int

  select @maxprsschemaid = max(ps.PrsSchemaID)
    from ArbeitsPlan ap
   inner join PlanSchema ps
      on ps.SchemaID = ap.SchemaID
   where ap.PlanID = @planid
     and ps.AbteilungID = @abteilungid

  update PlanSchema
     set PrsSchemaID = ps.PrsSchemaID + 1
    from PlanSchema ps
   inner join ArbeitsPlan ap
      on ps.SchemaID = ap.SchemaID
   where ap.PlanID = @planid
     and ps.PrsSchemaID > @maxprsschemaid

  insert PlanSchema
         (SchemaID,
          PrsSchemaID,
          PrsID,
          AbteilungID,
          RGB,
          ColorName,
          Bezeichner)
   select ap.SchemaID,
          @maxprsschemaid + 1,
          @prsid,
          @abteilungid,
          -1,
          '',''
      from ArbeitsPlan ap
   inner join PlanSchema ps
      on ps.SchemaID = ap.SchemaID
   where ap.PlanID = @planid
   group by ap.SchemaID

end
go
print 'Procedure: set_zusatzpersonplan done ...'
go
grant exec on set_zusatzpersonplan to prsadmins with grant option
go
grant exec on set_zusatzpersonplan to prsusers
go


if exists (select 1 from sysobjects where name = 'upd_abteilung' and type = 'P')
begin
   drop procedure upd_abteilung
   print 'Procedure: upd_abteilung deleted ...'
end
go
create procedure upd_abteilung(
  @abteilungid       int           = 1,
  @abteilungname     varchar(32)   = 'Abteilung neu',
  @beschreibung      varchar(250)  = 'Abteilung Beschreibung neu',
  @stundenprotag     decimal(5,2)  = 20
--  @gerechnet         int           = 1
)
as
begin
  set nocount on
  declare @beschrid   int
   select @beschrid = beschrid
     from abteilung
    where abteilungid = @abteilungid
  exec upd_beschreibung @beschrid, @abteilungname, @beschreibung
   update abteilung 
      set abteilung.name = @abteilungname,
          abteilung.stundenprotag = @stundenprotag
    where abteilung.abteilungid = @abteilungid
end
go
print 'Procedure: upd_abteilung done ...'
go
grant exec on upd_abteilung to prsadmins with grant option
go
grant exec on upd_abteilung to prsusers
go


if exists (select 1 from sysobjects where name = 'upd_anstellung' and type = 'P')
begin
   drop procedure upd_anstellung
   print 'Procedure: upd_anstellung deleted ...'
end
go
create procedure upd_anstellung(
  @anstellungid       int           = 1,
  @anstellungname     varchar(32)   = 'Abteilung neu',
  @beschreibung       varchar(250)  = 'Abteilung Beschreibung neu',
  @prio               int           = 1,
  @EinsatzProTag      int           = 1,
  @gerechnet          int           = 1,
  @ausgelernt         int           = 0
)
as
begin
  set nocount on
  declare @beschrid   int
   select @beschrid = BeschrID
     from Anstellung
    where AnstellungID = @anstellungid
    
     exec upd_beschreibung @beschrid, @anstellungname, @beschreibung
  
   update Anstellung 
      set Name = @anstellungname,
          Prioritaet = @prio,
          EinsatzProTag = @EinsatzProTag,
          Gerechnet = @gerechnet,
          ausgelernt = @ausgelernt
--          anstellugn.gerechnet = @Gerechnet 
    where AnstellungID = @anstellungid
end
go
print 'Procedure: upd_anstellung done ...'
go
grant exec on upd_anstellung to prsadmins with grant option
go
grant exec on upd_anstellung to prsusers
go


if exists (select 1 from sysobjects where name = 'upd_arbeitstage_feiertag' and type = 'P')
begin
   drop procedure upd_arbeitstage_feiertag
   print 'Procedure: upd_arbeitstage_feiertag deleted ...'
end
go
-- setzt anzahl tage pro monat fest
create procedure upd_arbeitstage_feiertag(
  @name          varchar(32)  = 'Ostern',
  @von           varchar(8)   = '00000000',
  @bis           varchar(8)   = '00000000',
  @tage          decimal(5,2) = 23,   --Anzahl tage im monat an denen gearbeitet wird / anzahl bezahlte tage pro feiertag 
  @atid          int          = 1
)
as
begin

  set nocount on

  update ArbeitsTage
     set Name = @name,
         Von  = convert(datetime, @von, 108),
         Bis  = convert(datetime, @bis, 108),
         Tage = @tage
   where ATID = @atid

end
go
print 'Procedure: upd_arbeitstage_feiertag done ...'
go
grant exec on upd_arbeitstage_feiertag to prsadmins with grant option
go
grant exec on upd_arbeitstage_feiertag to prsusers
go


if exists (select 1 from sysobjects where name = 'upd_arbeitstage_tage' and type = 'P')
begin
   drop procedure upd_arbeitstage_tage
   print 'Procedure: upd_arbeitstage_tage deleted ...'
end
go
-- setzt anzahl tage pro monat fest
create procedure upd_arbeitstage_tage(
  @jahrid        int          = 1,
  @typwert       int          = 1,
  @Tage          decimal(5,2) = 23   --Anzahl tage im monat an denen gearbeitet wird / anzahl bezahlte tage pro feiertag 
)
as
begin

  set nocount on

  update ArbeitsTage
     set Tage = @Tage
   where JahrID = @jahrid
     and typwert = @typwert

end
go
print 'Procedure: upd_arbeitstage_tage done ...'
go
grant exec on upd_arbeitstage_tage to prsadmins with grant option
go
grant exec on upd_arbeitstage_tage to prsusers
go


if exists (select 1 from sysobjects where name = 'upd_beschreibung' and type = 'P')
begin
   drop procedure upd_beschreibung
   print 'Procedure: upd_beschreibung deleted ...'
end
go
create procedure upd_beschreibung(
  @beschrid          int           = 6,
  @dienstname        varchar(32)   = 'Dienst',
  @beschreibung      varchar(250)  = 'Dienst Beschreibung'
)
as
begin
  set nocount on
   update Beschreibung 
      set Kurztext = @dienstname,
          Langtext = @beschreibung
    where BeschrID = @beschrid
end
go
print 'Procedure: upd_beschreibung done ...'
go
grant exec on upd_beschreibung to prsadmins with grant option
go
grant exec on upd_beschreibung to prsusers
go


if exists (select 1 from sysobjects where name = 'upd_dienst' and type = 'P')
begin
   drop procedure upd_dienst
   print 'Procedure: upd_dienst deleted ...'
end
go
create procedure upd_dienst(
  @dienstid          int           = 1,
  @dienstname        varchar(32)   = 'Abteilung neu',
  @kurztext          varchar(60)   = 'kurztext',
  @beschreibung      varchar(250)  = 'Abteilung Beschreibung neu',
--  @einsatzprotag     int           = 1,
  @gerechnet         int           = 1,
  @RGB                int           = 0,
  @ColorName          varchar(32)   = '',
  @typ               int           = 1,
  @GanzerTag         int           = 0
)
as
begin
  set nocount on
  declare @beschrid   int
   select @beschrid = BeschrID
     from Dienst
    where DienstID = @dienstid
  exec upd_beschreibung @beschrid, @kurztext, @beschreibung
   update Dienst 
      set Dienst.Name = @dienstname,
--          einsatzprotag = @einsatzprotag,
          Gerechnet = @gerechnet,
          RGB = @RGB,
          ColorName = @ColorName,
          Typ = @typ,
          GanzerTag = @GanzerTag
    where DienstID = @dienstid

end
go
print 'Procedure: upd_dienst done ...'
go
grant exec on upd_dienst to prsadmins with grant option
go
grant exec on upd_dienst to prsusers
go


if exists (select 1 from sysobjects where name = 'upd_dienstserie' and type = 'P')
begin
   drop procedure upd_dienstserie
   print 'Procedure: upd_dienstserie deleted ...'
end
go
create procedure upd_dienstserie(
  @OldPrsID             int           = 1,
  @Olddienstid          int           = 1,
  @OldAbteilungID       int           = 1,
  @OldTag               int           = 1,
  @OldTyp               varchar(5)       = 'W',--M: Monat W: Woche
  @Oldvon               varchar(8)    = '20080101',
  @Oldbis               varchar(8)    = '20100101',
  @PrsID             int           = 1,
  @dienstid          int           = 1,
  @AbteilungID       int           = 1,
  @Tag               int           = 1,
  @Typ               varchar(5)       = 'W',--M: Monat W: Woche
  @von               varchar(8)    = '20080101',
  @bis               varchar(8)    = '20100101'
)
as
begin
  set nocount on

  declare @datevon datetime
  declare @datebis datetime
  declare @olddatevon datetime
  declare @olddatebis datetime

  select @datevon = convert(datetime, @von, 112)
  select @datebis = convert(datetime, @bis, 112)
  select @olddatevon = convert(datetime, @Oldvon, 112)
  select @olddatebis = convert(datetime, @Oldbis, 112)

  update DienstSerie 
     set PrsID = @PrsID, 
         DienstID = @dienstid, 
         AbteilungID = @AbteilungID, 
         Tag = @Tag, 
         Typ = upper(@Typ), 
         Von = @datevon, 
         bis = @datebis
   where PrsID = @OldPrsID
     and DienstID = @Olddienstid
     and AbteilungID = @OldAbteilungID
     and Tag = @OldTag
     and Typ = upper(@OldTyp)
     and Von = @olddatevon
     and bis = @olddatebis

end
go
print 'Procedure: upd_dienstserie done ...'
go
grant exec on upd_dienstserie to prsadmins with grant option
go
grant exec on upd_dienstserie to prsusers
go


if exists (select 1 from sysobjects where name = 'upd_options' and type = 'P')
begin
   drop procedure upd_options
   print 'Procedure: upd_options deleted ...'
end
go
create procedure upd_options(
  @typin             varchar(30)   = 'Arbeitszeitprotag',
  @wertin            int           = 1,
  @zahlwertin        decimal(15,2) = 8,
  @textwert          varchar(255)  = 'Stunden pro Tag'
)
as
begin
  set nocount on

   if @textwert = '' or @textwert is null select @textwert = textwert from Options where typ = @typin and wert = @wertin

   update Options set
          zahlwert       = @zahlwertin,
          textwert       = @textwert
    where typ            = @typin
      and wert           = @wertin 
--      and textwert       = @textwert
end
go
print 'Procedure: upd_options done ...'
go
grant exec on upd_options to prsadmins with grant option
go
grant exec on upd_options to prsusers
go



if exists (select 1 from sysobjects where name = 'upd_person' and type = 'P')
begin
   drop procedure upd_person
   print 'Procedure: upd_person deleted ...'
end
go

create procedure upd_person(
  @vorname         varchar(32) = 'name test', 
  @nachname        varchar(32) = 'nachname', 
  @strasse         varchar(32) = 'strasse', 
  @strassennr      varchar(5)  = 'nr', 
  @postfach        varchar(10) = 'postfach test', 
  @plz             char(5)     = 'plz', 
  @ort             varchar(32) = 'ort', 
  @telnrp          varchar(15) = 'telp', 
  @telnrn          varchar(15) = 'teln', 
  @email           varchar(60) = 'email@email.ch',
  --@pensumvon       varchar(8)   = '20070301',--datetime    = '20070301',
  --@pensumbis       varchar(8)    = '20100101',--datetime    = '20100101',
  --@pensumprozent   smallint    = 100,
  --@anstellungid    int         = 1,
  @prsid           int         = 3
)
as
begin
  set nocount on


   update Person set
          Person.Vorname = @vorname,
          Person.Nachname = @nachname,
          Person.Strasse = @strasse,
          Person.Strassennr = @strassennr,
          Person.Postfach = @postfach,
          Person.PLZ = @plz,
          Person.Ort = @ort,
          Person.TelnrP = @telnrp,
          Person.TelnrN = @telnrn,
          Person.Email = @email
    where Person.PrsId = @prsid


end
go
print 'Procedure: upd_person done ...'
go
grant exec on upd_person to prsadmins with grant option
go
grant exec on upd_person to prsusers
go


if exists (select 1 from sysobjects where name = 'upd_planschema' and type = 'P')
begin
   drop procedure upd_planschema
   print 'Procedure: upd_planschema deleted ...'
end
go
-- setzt anzahl tage pro monat fest
create procedure upd_planschema(
  @SchemaID        int = 1,
  @PrsSchemaID     int = 1,-- nummer fortlaufend, fngt bei jedem schema neu an
  @PrsID           int = 1,
  @AbteilungID     int = 1,
  @RGB             int = null,
  @ColorName       varchar(32) = '',
  @Bezeichner      varchar(32) = ''
)
as
begin

  set nocount on
--select * from planschema
if exists (select 1 from planschema where prsschemaid = @PrsSchemaID and schemaid = @SchemaID)
begin
  update planschema set
         prsid = @PrsID,
         abteilungid = @AbteilungID,
         rgb = @RGB,
         colorname = @ColorName,
         bezeichner = @Bezeichner
   where schemaid = @SchemaID
     and prsschemaid = @PrsSchemaID
end
else
begin
   exec set_planschema @SchemaID, @PrsSchemaID, @PrsID, @AbteilungID, @RGB, @ColorName, @Bezeichner
end

end
go

print 'Procedure: upd_planschema done ...'
go

grant exec on upd_planschema to prsadmins with grant option
go
grant exec on upd_planschema to prsusers
go



if exists (select 1 from sysobjects where name = 'upd_planschemaid' and type = 'P')
begin
   drop procedure upd_planschemaid
   print 'Procedure: upd_planschemaid deleted ...'
end
go
create procedure upd_planschemaid(
  @schemaid          int           = 1,
  @planid            int           = 1
)
as
begin
  set nocount on

   declare @prsid       int
   declare @abteilungid int

   create table #personen(
      planid       int,
      prsid        int,
      abteilungid  int
   )

   -- Schemanummer aktualisieren
   update arbeitsplan
      set schemaid = @schemaid
    where planid = @planid


-- Alles lschen was nicht in schema gehrt
   insert #personen
   select planid = dp.planid,
          prsid = dp.prsid,
          abteilungid = dp.abteilungid
-- select * 
     from dienstplan dp
    inner join arbeitsplan ap
       on ap.planid = dp.planid
     left outer join planschema ps
       on ps.schemaid = ap.schemaid
      and ps.prsid = dp.prsid
      and ps.abteilungid = dp.abteilungid
    where dp.planid = @planid
      and ps.prsid is null
    group by dp.prsid, dp.abteilungid, dp.planid



      declare prs_crs cursor for 
       select prsid,
              abteilungid
         from #personen
          for read only
      
      open prs_crs
      fetch next from prs_crs into @prsid, @abteilungid

      while (@@fetch_status <> -1)
      begin
         if (@@fetch_status <> -2)
         begin
            delete from dienstplan where prsid = @prsid and abteilungid = @abteilungid and planid = @planid
         end

         fetch next from prs_crs into @prsid, @abteilungid
      end

      close prs_crs
      deallocate prs_crs


end
go
print 'Procedure: upd_planschemaid done ...'
go
grant exec on upd_planschemaid to prsadmins with grant option
go
grant exec on upd_planschemaid to prsusers
go


if exists (select 1 from sysobjects where name = 'upd_planstatus' and type = 'P')
begin
   drop procedure upd_planstatus
   print 'Procedure: upd_planstatus deleted ...'
end
go
create procedure upd_planstatus(
  @statusin          int           = 1,  -- 1: Aktiv   2: Inaktiv
  @planidin          int           = 1
)
as
begin
  set nocount on

   if @statusin = 0 select @statusin = 1 -- per default aktiv

   begin
      update ArbeitsPlan set Status = @statusin
       where ArbeitsPlan.PlanID = @planidin
   end
end
go
print 'Procedure: upd_planstatus done ...'
go
grant exec on upd_planstatus to prsadmins with grant option
go
grant exec on upd_planstatus to prsusers
go


if exists (select 1 from sysobjects where name = 'upd_planungsjahr_jahr' and type = 'P')
begin
   drop procedure upd_planungsjahr_jahr
   print 'Procedure: upd_planungsjahr_jahr deleted ...'
end
go

create procedure upd_planungsjahr_jahr(
  @jahr          int   = 1900
)
as
begin

  set nocount on

  -- Alle alten jahre status auf 2 setzten
  update planungsjahr set status = 2 where status = 1

  -- neues jahr mit aktivem status stellen
  update planungsjahr set status = 1 where jahr = @jahr

end
go
print 'Procedure: upd_planungsjahr_jahr done ...'
go
grant exec on upd_planungsjahr_jahr to prsadmins with grant option
go
grant exec on upd_planungsjahr_jahr to prsusers
go



if exists (select 1 from sysobjects where name = 'upd_prsabteilung' and type = 'P')
begin
   drop procedure upd_prsabteilung
   print 'Procedure: upd_prsabteilung deleted ...'
end
go
create procedure upd_prsabteilung(
  @abteilungid      int   = 1,
  @prsid            int   = 1,
  @vonnew           char(8) = '20070501',
  @bisnew           char(8) = '20090501'
)
as
begin
  set nocount on

-- Daten die berschssig sind lschen
-- will cause error if there are more than 1 that have the same begin date because enddate will be set to same...
declare @abtcount table(abteilungid int, cnt int)
 insert @abtcount
 select cnt = count(Von), 
        abteilungid = AbteilungID
   from PersonalAbteilung 
  where (AbteilungID = @abteilungid or @abteilungid =0) 
    and PrsId = @prsid 
    and Bis >= @vonnew 
  group by AbteilungID,PrsId,Von having COUNT(Von) > 1

--if @abtcount > 1
--begin
--  declare @maxbis datetime
--   select @maxbis = MAX(Bis) 
--     from PersonalAbteilung 
--    where AbteilungID = @abteilungid 
--      and PrsId = @prsid 
--      and Bis >= @bisnew having COUNT(Von) > 1
   
--   delete from PersonalAbteilung 
--    where PrsId = @prsid 
--      and AbteilungID = @abteilungid 
--      and Von = (select Von from PersonalAbteilung where AbteilungID = @abteilungid and PrsId = @prsid and Bis >= @bisnew group by AbteilungID,PrsId,Von having COUNT(Von) > 1)
--      and Bis <> @maxbis
--end

  declare @abteilungen table (abteilungid int)
   insert @abteilungen
   select abteilungid = AbteilungID
     from PersonalAbteilung 
    where PrsId = @prsid 
      and (AbteilungID = @abteilungid or @abteilungid = 0) 
      and Von = @vonnew 
      and Bis >= @vonnew
      
if exists (select * from @abteilungen)
begin
   delete PersonalAbteilung
     from PersonalAbteilung pa 
    inner join @abteilungen a
       on a.abteilungid = pa.AbteilungID
    where PrsId = @prsid 
      and Von = @vonnew
      and Bis = @vonnew
end

  -- delete all that have the same begin date as end date
  delete from PersonalAbteilung where PrsId = @prsid and Bis = Von
  --delete from PersonalAbteilung where PrsId = @prsid and Von >= @bisnew and Bis >= @bisnew
  --delete from PersonalAbteilung where PrsId = @prsid and Von >= @vonnew and (Bis <= @bisnew or
  delete from PersonalAbteilung where PrsId = @prsid and Von >= @vonnew
  
  
  update PersonalAbteilung
     set Bis = dateadd(day, -1, @vonnew),-- @vonnew,
         Status = 2
   where (AbteilungID = @abteilungid or @abteilungid = 0)
     and PrsId = @prsid
     and Bis >= @vonnew
     --and Status = 1
--   end

end
go
print 'Procedure: upd_prsabteilung done ...'
go
grant exec on upd_prsabteilung to prsadmins with grant option
go
grant exec on upd_prsabteilung to prsusers
go



if exists (select 1 from sysobjects where name = 'upd_prsanstellung' and type = 'P')
begin
   drop procedure upd_prsanstellung
   print 'Procedure: upd_prsanstellung deleted ...'
end
go
create procedure upd_prsanstellung(
  @prsid             int   = 1,
  @anstellungid      int   = 1,
  @von               varchar(8) = '20070101',
  @bis               varchar(8) = '20200101'
)
as
begin
   set nocount on

   declare @datevon  datetime
   declare @datebis  datetime
 
   select @datevon = convert(datetime, @von, 112)
   select @datebis = convert(datetime, @bis, 112)

   -- Datum von alter Anstellung zurcksetzen
   update personanstellung
      set bis = @datevon
    where prsid = @prsid
      and bis = (select max(pa.bis) from personanstellung pa
                                   where pa.prsid = @prsid)

   insert into personanstellung (prsid,AnstellungID,Von,Bis) values (@prsid, @anstellungid, @datevon,@datebis)

end
go
print 'Procedure: upd_prsanstellung done ...'
go
grant exec on upd_prsanstellung to prsadmins with grant option
go
grant exec on upd_prsanstellung to prsusers
go


if exists (select 1 from sysobjects where name = 'upd_tageimmonat' and type = 'P')
begin
   drop procedure upd_tageimmonat
   print 'Procedure: upd_tageimmonat deleted ...'
end
go
-- setzt anzahl tage pro monat fest
create procedure upd_tageimmonat(
  @atid        int          = 1,
  @tage          decimal(5,2) = 23   --Anzahl tage im monat an denen gearbeitet wird / anzahl bezahlte tage pro feiertag 
)
as
begin

  set nocount on

  update ArbeitsTage
     set Tage = @tage
   where ATID = @atid

end
go
print 'Procedure: upd_tageimmonat done ...'
go
grant exec on upd_tageimmonat to prsadmins with grant option
go
grant exec on upd_tageimmonat to prsusers
go


if exists (select 1 from sysobjects where name = 'upd_termin' and type = 'P')
begin
   drop procedure upd_termin
   print 'Procedure: upd_termin deleted ...'
end
go
create procedure upd_termin(
  @terminidin        int           = 1,
  @dienstname        varchar(32)   = 'Sitzung',
  @kurztext          varchar(60)   = 'kurztext',
  @beschreibung      varchar(250)  = 'Sitzung Beschreibung neu',
  @statusin          int           = 1,
  @rgbin             int           = 0,
  @colorname         varchar(32)   = '00000000',
  @shgin             char(1)       = 'S'
)
as
begin
  set nocount on
  declare @beschrid   int
   select @beschrid = beschrid
     from termin
    where terminid = @terminidin
  exec upd_beschreibung @beschrid, @kurztext, @beschreibung
   update termin 
      set name        = @dienstname,
          status      = @statusin,
          defrgb      = @rgbin,
          colorname   = @colorname,
          shg         = upper(@shgin)
    where terminid    = @terminidin

end
go
print 'Procedure: upd_termin done ...'
go
grant exec on upd_termin to prsadmins with grant option
go
grant exec on upd_termin to prsusers
go


