if exists (select 1 from sysobjects where name = 'vw_abteilung' and type = 'V')
begin
   drop view vw_abteilung
   print 'View: vw_abteilung deleted ...'
end
go
create view vw_abteilung
as
   select 
    a.AbteilungID,
    a.Name,
    b.Kurztext,
    b.Langtext,
    b.BeschrID
   from Abteilung a
   left outer join Beschreibung b
   on b.BeschrID = a.BeschrID
go
print 'View: vw_abteilung done ...'
go

if exists (select 1 from sysobjects where name = 'vw_activearbeitsplan' and type = 'V')
begin
   drop view vw_activearbeitsplan
   print 'View: vw_activearbeitsplan deleted ...'
end
go
create view vw_activearbeitsplan
as
     select PlanID 
       from ArbeitsPlan
      where ArbeitsPlan.Status = 1
go
print 'View: vw_activearbeitsplan done ...'
go
grant select on vw_activearbeitsplan to prsadmins with grant option
go
grant select on vw_activearbeitsplan to prsusers
go


if exists (select 1 from sysobjects where name = 'vw_arbeitsplanactive' and type = 'V')
begin
   drop view vw_arbeitsplanactive
   print 'View: vw_arbeitsplanactive deleted ...'
end
go
create view vw_arbeitsplanactive
as

   select planid = ap.PlanID, 
          atid = ap.ATID, 
          planname = ap.Name, 
          jahrid = ap.JahrID, 
          schemaid = ap.SchemaID, 
          von = ap.Von, 
          bis = ap.Bis,
          schemaname = s.Name
     from ArbeitsPlan ap
    inner join Schemas s
       on s.SchemaID = ap.SchemaID 
    where ap.Status = 1 

go
print 'View: vw_arbeitsplanactive done ...'
go

grant select on vw_arbeitsplanactive to prsadmins with grant option
go
grant select on vw_arbeitsplanactive to prsusers
go


if exists (select 1 from sysobjects where name = 'vw_dienst' and type = 'V')
begin
   drop view vw_dienst
   print 'View: vw_dienst deleted ...'
end
go
create view vw_dienst
as
   select 
    d.DienstID,
    d.Name,
    b.Kurztext,
    b.Langtext,
    b.BeschrID
   from Dienst d
   left outer join Beschreibung b
   on b.BeschrID = d.BeschrID
go
print 'View: vw_dienst done ...'
go

if exists (select 1 from sysobjects where name = 'vw_inactivarbeitsplan' and type = 'V')
begin
   drop view vw_inactivarbeitsplan
   print 'View: vw_inactivarbeitsplan deleted ...'
end
go
create view vw_inactivarbeitsplan
as
     select PlanID 
       from ArbeitsPlan
      where ArbeitsPlan.Status = 2
go
print 'View: vw_inactivarbeitsplan done ...'
go
grant select on vw_inactivarbeitsplan to prsadmins with grant option
go
grant select on vw_inactivarbeitsplan to prsusers
go


if exists (select 1 from sysobjects where name = 'vw_initgrid' and type = 'V')
begin
   drop view vw_initgrid
   print 'View: vw_initgrid deleted ...'
end
go
create view vw_initgrid
as
     select top 1000
          d.PlanID, 
          d.PrsID, 
          p.Nachname, 
          p.Vorname, 
          a.Prozent 
       from Dienstplan d 
 inner join Person p 
         on p.PrsId = d.PrsID 
and p.PersonTyp = 1
 inner join Arbeitspensum a 
         on a.PrsID = p.PrsId 
--        and a.status = 1 
--and a.von <= @datum 
and a.Bis = (select max(Bis) from Arbeitspensum ab where ab.PrsID = p.PrsId)
   group by d.PrsID, d.PlanID, p.Nachname, p.Vorname, a.Prozent
   order by d.PlanID
go
print 'View: vw_initgrid done ...'
go

grant select on vw_initgrid to prsadmins with grant option
go
grant select on vw_initgrid to prsusers
go




if exists (select 1 from sysobjects where name = 'vw_person' and type = 'V')
begin
   drop view vw_person
   print 'View: vw_person deleted ...'
end
go
create view vw_person
as
   select 
      prsid  = p.PrsId,
   nachname  = p.Nachname,
    vorname  = p.Vorname, 
    strasse  = p.Strasse + ' ' + p.Strassennr,
   postfach  = p.Postfach,
        PLZ  = p.PLZ,
        ort  = p.Ort,
     telnrp  = p.TelnrP,
     telnrn  = p.TelnrN,
      email  = p.Email,
    prozent  = a.Prozent, 
abteilungid  = abt.AbteilungID,
  abteilung  = abt.Name,
anstellungid = ans.AnstellungID,
 anstellung  = ans.Name
   from Person p 
  inner join PersonalAbteilung pa 
     on p.PrsId = pa.PrsId 
and pa.Bis = (select max(pa2.Bis) from PersonalAbteilung pa2 where pa2.PrsId = p.PrsId)
   left outer join Abteilung abt
     on abt.AbteilungID = pa.AbteilungID
  inner join Arbeitspensum a 
     on a.PrsID = p.PrsId 
--    and a.status = 1 
--and a.von <= @datum 
and a.Bis = (select max(Bis) from Arbeitspensum ab where ab.PrsID = p.PrsId)
   left outer join PersonAnstellung pans
     on pans.PrsID = p.PrsId
   left outer join Anstellung ans
     on ans.AnstellungID = pans.AnstellungID
where p.PersonTyp = 1
go
print 'View: vw_person done ...'
go

grant select on vw_person to prsadmins with grant option
go
grant select on vw_person to prsusers
go


if exists (select 1 from sysobjects where name = 'vw_personferien' and type = 'V')
begin
   drop view vw_personferien
   print 'View: vw_personferien deleted ...'
end
go
create view vw_personferien
as
   select 
      prsid  = p.PrsId,
   nachname  = p.Nachname,
    vorname  = p.Vorname, 
    strasse  = p.Strasse + ' ' + p.Strassennr,
   postfach  = p.Postfach,
        PLZ  = p.PLZ,
        ort  = p.Ort,
     telnrp  = p.TelnrP,
     telnrn  = p.TelnrN,
      email  = p.Email,
   ferienid = f.FerienID,
  ferienvon = f.Von,
  ferienbis = f.Bis
     from Person p
     left outer join Ferien f
       on f.PrsId = p.PrsId
where p.PersonTyp = 1

go
print 'View: vw_personferien done ...'
go

grant select on vw_personferien to prsadmins with grant option
go
grant select on vw_personferien to prsusers
go


if exists (select 1 from sysobjects where name = 'vw_planrow' and type = 'V')
begin
   drop view vw_planrow
   print 'View: vw_planrow deleted ...'
end
go
create view vw_planrow
as

   select name = ap.Name,
          schemaid = ap.SchemaID,
          prsschemaid = ps.PrsSchemaID,
          prsid = ps.PrsID,
          rowcolor = ps.RGB,
          rowcolorname = ps.ColorName,
          tage = datepart(day,ap.Bis)
     from ArbeitsPlan ap
    inner join PlanSchema ps
       on ps.SchemaID = ap.SchemaID
--where ap.planid = 9

go
print 'View: vw_planrow done ...'
go

grant select on vw_planrow to prsadmins with grant option
go
grant select on vw_planrow to prsusers
go








if exists (select 1 from sysobjects where name = 'vw_schemadata' and type = 'V')
begin
   drop view vw_schemadata
   print 'View: vw_schemadata deleted ...'
end
go
create view vw_schemadata
as

   select schemaid = ps.SchemaID, 
          schemaname = s.Name, 
          prsschemaid = ps.PrsSchemaID,  
          prsid = ps.PrsID,
          nachname = isnull(p.Nachname,''),
          vorname = isnull(p.Vorname,''),
          abteilungid = ps.AbteilungID,
          abteilungname = ab.Name,
          rgb = ps.RGB,
          color = ps.ColorName
     from PlanSchema ps 
    inner join Schemas s on ps.SchemaID = s.SchemaID
     left outer join Person p
       on p.PrsId = ps.PrsID
    inner join Abteilung ab
       on ab.AbteilungID = ps.AbteilungID

go
print 'View: vw_schemadata done ...'
go

grant select on vw_schemadata to prsadmins with grant option
go
grant select on vw_schemadata to prsusers
go


