
if exists (select 1 from sysobjects where name = 'get_updprsschema' and type = 'P')
begin
   drop procedure get_updprsschema
   print 'Procedure: get_updprsschema deleted ...'
end
go
create procedure get_updprsschema(
  @schemaid         int     = 23,
  @plandatum        char(8) = '20090301'
)
as
begin
  set nocount on

   create table #schema(
     prsid            int,
     abteilungid      int,
     typ              int,
     pos              int
   )

   create table #abteilung(
      abteilungid     int
   )

   insert #abteilung
   select abteilungid = abteilungid
     from planschema
    where schemaid = @schemaid
    group by abteilungid

   -- kndigungen gesamtes pensum
   insert #schema
   select prsid = ap.prsid,
          abteilungid = pa.abteilungid,
          typ = 1, -- kndigungen
          pos = ps.prsschemaid
     from arbeitspensum ap
    inner join personalabteilung pa
       on pa.prsid = ap.prsid
      and (pa.status = 1 or pa.status is null)
    inner join planschema ps
       on ps.schemaid = @schemaid
      and ps.abteilungid = pa.abteilungid
      and ps.prsid = pa.prsid
    where ap.bis in (select max(ap2.bis) from arbeitspensum ap2 where ap2.prsid = ap.prsid)
      and ap.bis < convert(datetime,@plandatum,112)--getdate() --


   -- abteilungswechsel
   -- nicht in schema enthaltene
   insert #schema
   select prsid = pa.prsid,
          abteilungid = pa.abteilungid,
          typ = 2, -- nicht in schema enthalten
          pos = 0
     from personalabteilung pa
     left outer join planschema ps
       on ps.prsid = pa.prsid
      and ps.abteilungid = pa.abteilungid
      and ps.schemaid = @schemaid
    where pa.bis = (select max(pa2.bis) from personalabteilung pa2 where pa2.prsid = pa.prsid and pa2.abteilungid = pa.abteilungid)
      and ps.schemaid is null
      and convert(datetime,@plandatum,112) between pa.von and pa.bis
      and pa.prsid not in (select s.prsid from #schema s where typ = 1) -- keine gekndigten personen
      and pa.abteilungid in (select abt.abteilungid from #abteilung abt)

   -- personen die aus dem schema/abteilung entfernt werden mssen
   insert #schema
   select prsid = ps.prsid,
          abteilung = pa.abteilungid,
          typ = 3, -- aus schema entfernen
          pos = ps.prsschemaid
     from planschema ps
    inner join personalabteilung pa
       on pa.prsid = ps.prsid
      and pa.abteilungid = ps.abteilungid
      and pa.status = 2
      and pa.bis = (select max(pa2.bis) from personalabteilung pa2 where pa2.prsid = pa.prsid/* and pa2.abteilungid = pa.abteilungid*/)
      and convert(datetime, @plandatum, 112) not between pa.von and pa.bis
    where ps.schemaid = @schemaid



   select prsid = #schema.prsid,
          abteilungid = #schema.abteilungid,
          typ = #schema.typ,
          pos = #schema.pos
     from #schema

end
go
print 'Procedure: get_updprsschema done ...'
go
grant exec on get_updprsschema to prsadmins with grant option
go
grant exec on get_updprsschema to prsusers
go



