if exists (select 1 from sysobjects where name = 'set_newshemaplan' and type = 'P')
begin
   drop procedure set_newshemaplan
   print 'Procedure: set_newshemaplan deleted ...'
end
go
-- Erstellt ein neues Schema und einen neuen Plan
-- Wird nur benutzt wenn laufende schemas aktiviert sind
create procedure set_newshemaplan(
  @planname          varchar(60)   = 'Januar',
  @atid              int           = 25, --24
  @schemaid          int           = 1
)
as
begin
  set nocount on
  declare    @datumvon    datetime
  declare    @datumbis    datetime
  --declare    @oldshemaid  int
  declare    @newshemaid  int
  declare    @oldplanid   int
  
  declare    @mandid      char(2)
  
    select @mandid = mandid 
      from Schemas 
     where SchemaID = @schemaid
  
   select @datumvon = Von
     from ArbeitsTage
    where ATID = @atid
    
   select @datumbis = Bis
     from ArbeitsTage
    where ATID = @atid
   
   select @oldplanid = PlanID from ArbeitsPlan where ATID = @atid and SchemaID = @schemaid
   --select @oldshemaid = SchemaID from ArbeitsPlan where PlanID = @oldplanid
   
   create table #prs
   (
      prsschemaid int,
      prsid       int,
      abteilungid int,
      rgb         int,
      stat        int
   )
   
   create table #schema
   (
      prsschemaid int identity(1,1),
      prsid       int,
      abteilungid int,
      rgb         int
   )
   
   -- Altes Schema laden nur mit personen die noch angestellt sind
   insert #prs
   select prsschemaid = ps.PrsSchemaID,
          prsid = ps.PrsID,
          abteilungid = ps.AbteilungID,
          rgb = ISNULL(ps.RGB,0),
          stat = 1
     from PlanSchema ps
    inner join PersonalAbteilung pa
       on pa.PrsId = ps.PrsID
      and pa.AbteilungID = ps.AbteilungID
      and pa.Von <= @datumvon
      and pa.Bis >= @datumbis
      and (pa.Status = 1 or pa.Status is null)
    where ps.SchemaID = @schemaid--@oldshemaid

   -- Abteilungen aus dem alten schema laden
   insert #prs
   select prsschemaid = ps.PrsSchemaID,
          prsid = ps.PrsID,
          abteilungid = ps.AbteilungID,
          rgb = ISNULL(ps.RGB,0),
          stat = 1
     from PlanSchema ps
    where ps.PrsID <= 0
      and ps.SchemaID = @schemaid--@oldshemaid

     -- Zusatzpersonen sollten schon nicht mehr enthalten sein da diese kein pensum haben...
   --delete #prs
   --  from #prs
   -- inner join ArbeitsPlanzprs apz
   --    on apz.zprsID = #prs.prsid
   --   and apz.abteilungID = #prs.abteilungid
    
   -- Personen ins schema laden die neu angefangen haben oder im schema fehlen
   insert #prs
   select prsschemaid = 0,
          prsid = p.PrsId,
          abteilungid = pa.AbteilungID,
          rgb = 0,
          stat = 2
     from Person p
    inner join PersonalAbteilung pa
       on pa.PrsId = p.PrsId
      and pa.Von <= @datumvon
      and pa.Bis >= @datumbis
      and pa.AbteilungID in (select AbteilungID from PlanSchema ps where ps.SchemaID = @schemaid)
      
   -- alle Personen lschen die bereits vorhanden sind
   delete #prs
     from #prs p1
    inner join Person
       on Person.PrsId = p1.prsid
    inner join PersonalAbteilung pa
       on pa.PrsId = Person.PrsId
      and pa.Von <= @datumvon
      and pa.Bis >= @datumbis
   where p1.prsid in (select p2.prsid from #prs p2 where p2.stat = 1)
    and p1.stat = 2
    
   update #prs
      set prsschemaid = (select MAX(p3.prsschemaid)+1 from #prs p3 where p3.abteilungid = #prs.abteilungid)
     from #prs 
    where prsschemaid = 0
      and prsid > 0
     
    insert #schema
    select prsid = prsid,
           abteilungid = abteilungid,
           rgb = rgb
      from #prs
     order by abteilungid, prsschemaid
          
   update Schemas
      set Status = 0
    where SchemaID = @schemaid--@oldshemaid
   
   declare @tmpschemaid int
   select @tmpschemaid = MAX(SchemaID)+1 from Schemas
   
   --insert into Schemas (Name,Status,mandid) values ('Schema ' + isnull(CONVERT(varchar(5),@tmpschemaid),'1'),1,@mandid)
   insert into Schemas (Name,Status,mandid) values (@planname,1,@mandid)
   
   select @newshemaid = MAX(SchemaID) from Schemas where mandid = @mandid
      
   insert PlanSchema
   select SchemaID = @newshemaid,
          PrsSchemaID = prsschemaid,
          PrsID = prsid,
          AbteilungID = abteilungid,
          RGB = rgb,
          ColorName = '',
          Bezeichner = ''
     from #schema
          
   declare @jahrid  int
   
   select @jahrid = JahrID
     from ArbeitsTage at
    where at.ATID = @atid
    
   -- Neuer Arbeitsplan erstellen
   insert into ArbeitsPlan (ATID, Name, JahrID, SchemaID, Von, Bis, Status, mandid) 
        values (@atid, @planname, @jahrid, @newshemaid, @datumvon, @datumbis, 1, @mandid)

   -- Create/copy new Dienstschema
   insert DienstSchema
   select schemaid      = @newshemaid,
          abteilungid   = ds.AbteilungID,
          dienstid      = ds.DienstID,
          einsatzprotag = ds.EinsatzProTag,
          gerechnet     = ds.Gerechnet
     from DienstSchema ds
    where ds.SchemaID = @schemaid--@oldshemaid
   
   
   -- Neue PlanID zurck geben
   select planid = max(ArbeitsPlan.PlanID)
     from ArbeitsPlan
    where ArbeitsPlan.Von = @datumvon
      and ArbeitsPlan.Bis = @datumbis
      and ArbeitsPlan.Name = @planname
      and ArbeitsPlan.ATID = @atid
      and ArbeitsPlan.Status = 1

end
go
print 'Procedure: set_newshemaplan done ...'
go
grant exec on set_newshemaplan to prsadmins with grant option
go
grant exec on set_newshemaplan to prsusers
go

