
if exists (select 1 from sysobjects where name = 'set_person' and type = 'P')
begin
   drop procedure set_person
   print 'Procedure: set_person deleted ...'
end
go
create procedure set_person(
  @mandid          char(2)     = 'li',
  @vorname         varchar(32) = 'name', 
  @nachname        varchar(32) = 'nachname', 
  @strasse         varchar(32) = 'strasse', 
  @strassennr      varchar(5)  = 'nr', 
  @postfach        varchar(10) = 'postfach', 
  @plz             char(5)     = 'plz', 
  @ort             varchar(32) = 'ort', 
  @telnrp          varchar(15) = 'telp', 
  @telnrn          varchar(15) = 'teln', 
  @email           varchar(60) = 'email@email.ch',
  @pensumvon       varchar(8)   = '20070101',
  @pensumbis       varchar(8)   = '20100101',
  @pensumprozent   smallint    = 100,
  @anstellungid    int         = 1,
  @persontyp       int         = 1
)
as
begin
   set nocount on
   declare @prsid         int
   declare @ferientage    decimal(5,2)
   declare @jahrid        int

   if @persontyp is null select @persontyp = 1

   if @strassennr = '' select @strassennr = null
   if @postfach = '' select @postfach =  null
   if @telnrp = '' select @telnrp = null
   if @telnrn = '' select @telnrn = null
   if @email = '' select @email = null
   if @pensumbis = '' select @pensumbis = null

   select @ferientage = isnull(Options.zahlwert,0) from Options where Options.typ = 'Ferientage' and wert = 0

   insert into Person (Vorname, Nachname, Strasse, Strassennr, Postfach, PLZ, Ort, TelnrP, TelnrN, Email, PersonTyp,mandid) values (@vorname, @nachname, @strasse, @strassennr, @postfach, @plz, @ort, @telnrp, @telnrn, @email, isnull(@persontyp,1), @mandid)

   select @prsid = p.PrsId from Person p
   where (p.Vorname = @vorname or @vorname is null)
     and (p.Nachname = @nachname or @nachname is null)
     and (p.Strasse = @strasse or @strasse is null)
     and (p.Strassennr = @strassennr or @strassennr is null)
     and (p.Postfach = @postfach or @postfach is null)
     and (p.PLZ = @plz or @plz is null)
     and (p.Ort = @ort or @ort is null)

if @persontyp = 1
begin
   if exists (select 1 from Arbeitspensum where PrsID = @prsid and (Bis = '' or Bis is null))
      begin
        update Arbeitspensum 
           set Bis = convert(datetime, @pensumbis, 112), 
               Status = 2 
         where PrsID = @prsid 
           and (Bis = '' or Bis is null) 
           and Von = (select max(Von) from Arbeitspensum 
                       where PrsID = @prsid 
                         and (Bis = '' or Bis is null))
      end
   else
      begin
        insert into Arbeitspensum 
                    (PrsID, 
                     Von, 
                     Bis, 
                     Prozent, 
                     Status) 
             values (@prsid, 
                     convert(datetime, 
                     @pensumvon, 112), 
                     convert(datetime, @pensumbis, 112), 
                     @pensumprozent,
                     1)
   end 

   exec set_prsanstellung @prsid, @anstellungid, @pensumvon, @pensumbis

   -- Arbeitszeiten abfllen
   select @jahrid = JahrID from PlanungsJahr where Status = 1
   if @jahrid is null select @jahrid = max(JahrID) from PlanungsJahr

   if exists (select 1 from PersonZeiten where PrsID = @prsid and JahrID = @jahrid)
   begin
      update PersonZeiten 
         set FerienSaldo = ((@ferientage * @pensumprozent)/100) 
       where PersonZeiten.PrsID = @prsid
   end
   else
   begin
--TODO:
--arbeitszeitsaldo noch einbauen...
      insert into PersonZeiten (PrsID,FerienSaldo,ArbeitszeitSaldo,JahrID) 
           values (@prsid,((@ferientage * @pensumprozent)/100),0,@jahrid)
   end
end
   -- Return the PrsID
   select prsid = @prsid

end
go
print 'Procedure: set_person done ...'
go
grant exec on set_person to prsadmins with grant option
go
grant exec on set_person to prsusers
go

