if exists (select 1 from sysobjects where name = 'set_updsetdienst' and type = 'P')
begin
   drop procedure set_updsetdienst
   print 'Procedure: set_updsetdienst deleted ...'
end
go
-- exec set_updsetdienst 27,30,1,'20091109',0
-- delete from Arbeitszeit where PrsID = 30 and PlanID = 27 and AbteilungID = 1
-- select * from Arbeitszeit where PrsID = 30 and PlanID = 27 and AbteilungID = 1
create procedure set_updsetdienst(
  @planid         int = 1,
  @prsid          int = 1,
  @abteilungid    int = 1,
  @datum          char(8) = '20080702',
  @dienstid       int = 8
)
as
begin
  set nocount on

   declare @dte datetime
   select @dte = convert(datetime,@datum,112)
   
   if @dienstid = 0
   begin --delete dienst
     delete from Dienstplan
     where PlanID = @planid
       and PrsID = @prsid
       and Datum = @dte
       and AbteilungID = @abteilungid
   end
   else --add dienst
   begin
     if exists (select * from Dienstplan dp
                 where dp.PlanID = @planid
                   and dp.PrsID = @prsid
                   and dp.Datum = @dte
                   and dp.AbteilungID = @abteilungid)
       begin
         update Dienstplan
            set DienstID = @dienstid
          where PlanID = @planid
            and PrsID = @prsid
            and Datum = @dte
            and AbteilungID = @abteilungid
       end
       else --new record
       begin
         insert Dienstplan
                (PlanID,
                 PrsID,
                 Datum,
                 AbteilungID,
                 DienstID)
         values (@planid,
                 @prsid,
                 @dte,
                 @abteilungid,
                 @dienstid)
       end
     end
     
     
     --recalculate saldos
     create table #dienstzeiten(
        dienstid       int,
        zeit           decimal(8,2),
        typ            char(1)
     )
     
    -- Zeiten berechnen
    declare @abteilungszeit  decimal(5,2)
    declare @arbeitstage     decimal(5,2)
    declare @pensum          decimal(5,2)
    declare @soll            decimal(5,2)
    declare @haben           decimal(5,2)
    declare @ferien          decimal(5,2)

     select @abteilungszeit = stundenprotag from Abteilung where AbteilungID = @abteilungid

     -- zeiten in decimal konvertieren
     insert #dienstzeiten
     select dienstid = dz.DienstID,
            zeit     = case when dz.Von < dz.Bis then
                                --datepart(hh,dz.bis - dz.von) + (convert(decimal(5,2), datepart(mi,dz.bis - dz.von))/100)
                                datepart(hh,dz.Bis - dz.Von) + (convert(decimal(5,2), ((datepart(mi,dz.Bis)*100/60)- (datepart(mi,dz.Von)*100/60) ))/100)
                            else
                                --datepart(hh,dz.von - dz.bis) + (convert(decimal(5,2), datepart(mi,dz.von - dz.bis))/100)
                                datepart(hh,dz.Von - dz.Bis) + (convert(decimal(5,2), ((datepart(mi,dz.Von)*100/60) - (datepart(mi,dz.Bis)*100/60) ))/100)
                            end,
            typ      = dz.Zeiteinheit
       from DienstZeit dz
      where dz.datvon <= @datum
        and dz.datbis > @datum --bis und von knnen am 1. gleich gross sein wenn dienstzeit gendert wird
      
     -- Zeiten aus selbem dienst addieren
     insert #dienstzeiten
     select dienstid = dz.dienstid,
            zeit     = sum(zeit),
            typ      = '1'
       from #dienstzeiten dz
      where dz.typ = 'Z'
      group by dz.dienstid, typ
      
      -- pausen aus selbem dienst addieren
      insert #dienstzeiten
     select dienstid = dz.dienstid,
            zeit     = sum(zeit),
            typ      = '2'
       from #dienstzeiten dz
      where dz.typ = 'P'
      group by dz.dienstid, typ
      
      -- zeiten - pausen
      insert #dienstzeiten
     select dienstid = dz1.dienstid,
            zeit     = isnull(dz1.zeit,0)-isnull(dz2.zeit,0),
            typ      = 'F'
       from #dienstzeiten dz1
       left outer join #dienstzeiten dz2
         on dz2.dienstid = dz1.dienstid
        and dz2.typ = '2'
      where dz1.typ = '1'
        
     -- berschssige eintrge lschen
     delete from #dienstzeiten where typ <> 'F'

     -- Zeit von Abteilung nehmen wo ntig
     update #dienstzeiten
        set zeit = @abteilungszeit
       from #dienstzeiten
      inner join Dienst d
         on d.DienstID = #dienstzeiten.dienstid
      inner join DienstZeit dz
         on dz.DienstID = d.DienstID
        and dz.datvon <= @datum
        and dz.datbis > @datum
      where dz.abteilungzeit = 1

       -- haben
   select @haben = isnull(sum(isnull(dz.zeit,0)),0)
     from Dienstplan dp 
    inner join Dienst d
       on d.DienstID = dp.DienstID
    inner join #dienstzeiten dz
       on dz.dienstid = dp.DienstID
    where dp.PlanID = @planid
      and dp.PrsID = @prsid
      and dp.AbteilungID = @abteilungid
      and d.Typ <> 3 -- Ferien
      and d.Gerechnet = 1
        
     -- ferien
   select @ferien = isnull(sum(isnull(dz.zeit,0)),0)
     from Dienstplan dp 
    inner join Dienst d
       on d.DienstID = dp.DienstID
    inner join #dienstzeiten dz
       on dz.dienstid = dp.DienstID
    where dp.PlanID = @planid
      and dp.PrsID = @prsid
      and dp.AbteilungID = @abteilungid
      and d.Typ = 3 --Ferien
      and d.Gerechnet = 1


       -- Soll berechnen
       select @arbeitstage = Tage 
         from ArbeitsTage at
        inner join PlanungsJahr pj
           on pj.JahrID = at.JahrID
          and @dte between pj.von and pj.bis
        inner join ArbeitsPlan ap
           on ap.ATID = at.ATID
        where ap.PlanID = @planid
   
       select @pensum = isnull(nullif(pa.Prozent,0) ,isnull(a.Prozent,0))
         from Arbeitspensum a,
              PersonalAbteilung pa
        where pa.AbteilungID = @abteilungid
          and a.PrsID = @prsid
          and a.Von <= @dte
          and a.Bis > @dte
          and pa.PrsId = a.PrsID
          and pa.Von <= @dte
          and pa.Bis > @dte
          
        if @pensum is null
            select @pensum = 0--100
      
       --(Data.Monat.MonatData[atIndex].Tage / 100) * pensum * hours
       select @abteilungszeit = max(stundenprotag) from Abteilung where AbteilungID = @abteilungid

       select @soll = isnull(((@arbeitstage * @pensum)/100) * @abteilungszeit,0)
       
       if @pensum = 0 select @soll = 0
       
     -- Zeiten einfgen
     if exists (select * from Arbeitszeit 
                        where PrsID = @prsid 
                          and PlanID = @planid 
                          and AbteilungID = @abteilungid)
     begin
     
       update Arbeitszeit
          set Haben = @haben,
              Ferien = @ferien,
              Soll = @soll
        where PlanID = @planid
          and PrsID = @prsid
          and AbteilungID = @abteilungid
        
     end
     else
     begin -- Insert
     
       ---- Soll berechnen
       --select @arbeitstage = Tage 
       --  from ArbeitsTage at
       -- inner join PlanungsJahr pj
       --    on pj.JahrID = at.JahrID
       --   and @dte between pj.von and pj.bis
       -- inner join ArbeitsPlan ap
       --    on ap.ATID = at.ATID
       -- where ap.PlanID = @planid
   
       --select @pensum = isnull(nullif(pa.Prozent,0) ,isnull(a.Prozent,0))
       --  from Arbeitspensum a,
       --       PersonalAbteilung pa
       -- where pa.AbteilungID = @abteilungid
       --   and a.PrsID = @prsid
       --   and a.Von <= @dte
       --   and a.Bis > @dte
       --   and pa.PrsId = a.PrsID
       --   and pa.Von <= @dte
       --   and pa.Bis > @dte
          
       -- if @pensum is null
       --     select @pensum = 100
      
       ----(Data.Monat.MonatData[atIndex].Tage / 100) * pensum * hours
       --select @abteilungszeit = max(stundenprotag) from Abteilung --where AbteilungID = @abteilungid

       --select @soll = isnull(((@arbeitstage * @pensum)/100) * @abteilungszeit,0)
       
      insert Arbeitszeit
             (PrsID,
              PlanID,
              AbteilungID,
              Soll,
              Haben,
              Ferien)
      values (@prsid,
              @planid,
              @abteilungid,
              @soll,
              @haben,
              @ferien)
     end
     
end
go
print 'Procedure: set_updsetdienst done ...'
go
grant exec on set_updsetdienst to prsadmins with grant option
go
grant exec on set_updsetdienst to prsusers
go

