using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Data.Odbc;
using Gencode.Common.ODBC;

namespace Gencode.Gengui.Logon
{
    public partial class Logon : Form
    {
        private bool anmldOK = false;

		LoginData login;

		string testTable = "";
		/// <summary>
		/// This table will be checked if it exists in the selected database.
		/// </summary>
		public string ConnectionTestTable
		{
			set
			{
				testTable = value;
			}
		}

		public Logon()
		{
			InitializeComponent();
			
			//set dialog to small
			_expanded = true;
			pictureBox1_Click(null, new EventArgs());

            try
            {
                for (int i = 0; i < ODBCManager.GetUserDSNList().Length; i++)
                {
                    // Nur SQL Server in die Liste einbinden
                    if (ODBCManager.GetUserDSNList()[i].GetDSNDriverName().ToString() == "SQL Server")
                        comboDBServer.Items.Add(ODBCManager.GetUserDSNList()[i].ToString());
                }
                }
            catch (Exception)
            {
                anmldOK = false;
                Gencode.Common.ActionLogger.Logger.LoggMessage(Gencode.Common.ActionLogger.MessageType.ODBC_ERROR,
                    "Keine USER-DSN-Verbindung vorhanden!");
            }
            try
            {
                for (int i = 0; i < ODBCManager.GetSystemDSNList().Length; i++)
                {
                    if (ODBCManager.GetSystemDSNList()[i].GetDSNDriverName().ToString() == "SQL Server")
                        comboDBServer.Items.Add(ODBCManager.GetSystemDSNList()[i].ToString());
                }
                setLoginData();
            }
            catch (Exception)
            {
                anmldOK = false;
                Gencode.Common.ActionLogger.Logger.LoggMessage(Gencode.Common.ActionLogger.MessageType.ODBC_ERROR,
                    "Keine SYSTEM-DSN-Verbindung vorhanden!");
            }
		}

		public bool TestConnection()
		{
			bool connectionOK = true;

			if (login == null)
				return false;

			//New with Gencode.Common
			Gencode.Common.SQL.DatabaseConection.ODBCConnetionString = "DSN=" + login.DSN +
				";UID=" + login.UserName +
				";PWD=" + login.Password +
				";Database=" + login.Database + ";";
			Gencode.Common.SQL.DatabaseConection.ODBCConnection = new OdbcConnection(Gencode.Common.SQL.DatabaseConection.ODBCConnetionString);

			try
			{
				DataView dv = Gencode.Common.SQL.SimpleQuerry.ExecuteForDataView("select 1 from sysobjects where name = '" + testTable + "'and type = 'U'");
				if (dv.Count > 0)
					connectionOK = true;
				else
					connectionOK = false;
			}
			catch (OdbcException)
			{
                connectionOK = false;
			}

			return connectionOK;
		}

		private void setLoginData()
		{
			LoginSerializer ser = new LoginSerializer();
			login = ser.Deserialize();

			if (login != null)
			{
				txtUser.Text = login.UserName;
				txtPwd.Text = login.Password;
				comboDBServer.Text = login.DSN;
				comboDB.Text = login.Database;
			}
		}

        private void btnOK_Click(object sender, EventArgs e)
        {
            if (comboDBServer.Text == "")
            {
                MessageBox.Show("Bitte ein Datenbank-Server auswhlen", "", MessageBoxButtons.OK);
                return;
            }

            if (comboDB.Text == "")
            {
                MessageBox.Show("Bitte eine Datenbank auswhlen", "", MessageBoxButtons.OK);
                return;
            }

            this.SetConectionString();

			try
			{
				// Tsten ob die Datenbank die Tabelle zum Testen besitzt kompatibel ist
				DataView dv = Gencode.Common.SQL.SimpleQuerry.ExecuteForDataView("select 1 from sysobjects where name = '" + testTable + "'and type = 'U'");

				if (dv.Count > 0)
				{
					this.anmldOK = true;
					login = new LoginData();
					login.UserName = txtUser.Text;
					login.Password = txtPwd.Text;
					login.Database = comboDB.Text;
					login.DSN = comboDBServer.Text;

					LoginSerializer ser = new LoginSerializer();
					ser.Serialize(login);
				}
				else
				{
					MessageBox.Show("Die ausgewhlten Datenbank ist nicht \n kompatieble mit dem Personalplaner", "Verbindungsfehler", MessageBoxButtons.OK);
				}
			}
			catch (Exception exp)
			{
				Gencode.Common.Util.ErrorHandler.LoggError("Logon Error", "Logon.btnOK_Click", exp.Message);
				Gencode.Common.Util.ErrorHandler.ShowError("Es ist ein Fehler beim Verbindungsaufbau entstanden.\nBitte berprfen Sie die Datenbankverbindung oder kontaktieren Sie ihren Administrator.","Anmeldefehler");
			}
			finally
			{
				this.DialogResult = DialogResult.OK;
				this.Close();
			}
        }

        private void btnCancle_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void comboDB_SelectedIndexChanged(object sender, EventArgs e)
        {
            getDatabases();
        }

        private void getDatabases()
        {
            string strSQL = "sp_helpdb";
            string odbcStrConnect = "DSN=" + comboDBServer.Text.ToString() + ";UID=" + txtUser.Text + ";PWD=" + txtPwd.Text + ";Database=master";
            OdbcConnection odbcConn = new OdbcConnection(odbcStrConnect);

            DataSet ds = new DataSet();
            OdbcDataAdapter cmd = new OdbcDataAdapter(strSQL, odbcConn);
            try
            {
                cmd.Fill(ds, "dbs");
                odbcConn.Close();
            }
            catch (OdbcException exp)
            {
                Gencode.Common.ActionLogger.Logger.LoggMessage(Gencode.Common.ActionLogger.MessageType.ODBC_ERROR, "Im DSN '" + comboDBServer.Text + "' ist keine Personalplaner Datenbank vorhanden.");
				Gencode.Common.Util.ErrorHandler.LoggError("Logon Error; no database", "Logon.getDatabases()", exp.Message);
				Gencode.Common.Util.ErrorHandler.ShowError("In der gewhlten DSN-Instanz ist keine\nDatenbank fr den Personalplaner vorhanden", "Verbindungsfehler");
				return;
            }
            finally
            {
                DataView dv = new DataView(ds.Tables["dbs"]);

                comboDB.Items.Clear();
                comboDB.Text = "";

                for (int i = 0; i < dv.Count; i++)
                {
                    // Keine System-Datenbanken laden
                    if (dv[i].Row[0].ToString() != "master" && dv[i].Row[0].ToString() != "model" && dv[i].Row[0].ToString() != "msdb" && dv[i].Row[0].ToString() != "tempdb")
                        comboDB.Items.Add(dv[i].Row[0].ToString());
                }

            }
        }

        private void combower_SelectedIndexChanged(object sender, EventArgs e)
        {
            SetConectionString();
        }

        private void SetConectionString()
        {
			//New with Gencode.Common
			Gencode.Common.SQL.DatabaseConection.ODBCConnetionString = "DSN=" + comboDBServer.Text.ToString() + 
				";UID=" + txtUser.Text + 
				";PWD=" + txtPwd.Text + 
				";Database=" + comboDB.Text + ";";
			Gencode.Common.SQL.DatabaseConection.ODBCConnection = new OdbcConnection(Gencode.Common.SQL.DatabaseConection.ODBCConnetionString);
        }

        private void Anmeldung_FormClosing(object sender, FormClosingEventArgs e)
        {
            // wenn nur auf x gedrckt wird 
            // soll Programm beendet werden
            if (!anmldOK)
                DialogResult = DialogResult.Cancel;
        }

		bool _expanded = false;
		private void pictureBox1_Click(object sender, EventArgs e)
		{
			if (_expanded)
			{
				//set size small
				this.Size = new Size(this.Size.Width, 143);
				comboDB.Visible = false;
				comboDBServer.Visible = false;
				label3.Visible = false;
				label4.Visible = false;

				//set image
				pictureBox1.Image = Properties.Resources.Expand_16;

				_expanded = false;
			}
			else
			{
				//set size big
				this.Size = new Size(this.Size.Width, 213);
				comboDB.Visible = true;
				comboDBServer.Visible = true;
				label3.Visible = true;
				label4.Visible = true;

				//set image
				pictureBox1.Image = Properties.Resources.Collapse_16;

				_expanded = true;
			}
		}
    }
}