using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using Gencode.PrsCommon.App;
using Gencode.Common.SQL;

namespace Personalplaner
{
    class PFerienRow : IPersonRow
    {
        List<IPersonColumn> column = new List<IPersonColumn>();
        int atid;
        string name;
        int prsID;
        int monat;
        int countBezogenenFerienTage = 0;

        public void SetData()
        {
            addColumns();
            addReservedFerien();
        }

		private void addColumns()
        {
            if (column.Count > 0)
                column.Clear();

            Gencode.Common.SQL.Querry querry = new Gencode.Common.SQL.Querry();
            querry.SetSTPName("get_personferiencolumndienst");
            querry.AddParameter("@prsid", prsID);
            querry.AddParameter("@atid", atid);
            querry.AddParameter("@jahrid", YearID);

            DataView dv = querry.ExecuteForDataView();

            PDienstColumn dienst = null;

            //Fr jeden Tag im Monat eine machen......
            int days = DateTime.DaysInMonth(Year, monat);
            string date = "";
            string ms;
            bool set;

            if (monat < 10)
            {
                ms = "0" + monat;
            }
            else
            {
                ms = monat.ToString();
            }

            //Monat first
            PMonthColumn monthColumn = new PMonthColumn();
            monthColumn.Name = name;
            column.Add(monthColumn);

            //Tage / Dienste
            for (int d = 1; d <= days; d++)
            {
                set = false;

                if (d < 10)
                {
                    date = "0" + d + "." + ms + "." + Year;
                }
                else
                {
					date = d + "." + ms + "." + Year;
                }
                
                dienst = new PDienstColumn();

                for (int c = 0; c < dv.Count; c++)
                {
					if (DateUtil.ConvertSqlToDate(dv[c].Row[0].ToString()) == date)
                    {
						dienst.Date = DateUtil.ConvertSqlToDate(dv[c].Row[0].ToString());
                        dienst.DienstID = Int32.Parse(dv[c].Row[1].ToString());
                        dienst.Name = dv[c].Row[2].ToString();

                        set = true;
                        c = dv.Count;
                        countBezogenenFerienTage++;
                    }
                }

                if (!set)
                {
                    dienst.Date = date;
                }

				if (new DateTime(Year, monat, d).DayOfWeek == DayOfWeek.Saturday)
                {
                    dienst.Weekend = GPerson.WEEKEND.Saturday;
                }
				else if (new DateTime(Year, monat, d).DayOfWeek == DayOfWeek.Sunday)
                {
                    dienst.Weekend = GPerson.WEEKEND.Sunday;
                }

                column.Add(dienst);
            }
        }

        private void addReservedFerien()
        {
			DataView dv = SimpleQuerry.ExecuteForDataView("select von, bis from ferien where prsid = " + prsID + " and (datepart(year, von) = " + Year /*Verwaltung.Year*/ + " or datepart(year, bis) = " + Year /*Verwaltung.Year*/ + ")");

            if (dv.Count > 0)
            {
                int dVon, dBis, mVon, mBis;
                string von, bis;
                int c = 0;

                //Find first index
                int colI = 0;
                while (colI < column.Count)
                {
                    if (column[c] is PDienstColumn)
                    {
                        colI = column.Count;
                    }
                    else
                    {
                        colI++;
                        c++;
                    }
                }

                for (int i = 0; i < dv.Count; i++)
                {
					von = DateUtil.ConvertSqlToDate(dv[i].Row[0].ToString());
					bis = DateUtil.ConvertSqlToDate(dv[i].Row[1].ToString());
					mVon = Int16.Parse(DateUtil.GetMonth(von));
					mBis = Int16.Parse(DateUtil.GetMonth(bis));
                    if (mVon == monat || mBis == monat)
                    {
						dVon = Int16.Parse(DateUtil.GetDay(von));
						dBis = Int16.Parse(DateUtil.GetDay(bis));

                        PDienstColumn dienst = null;

                        //Ferien im selben Monat
                        if (mVon == mBis)
                        {
                            while (dVon <= dBis)
                            {
                                dienst = (PDienstColumn)column[c + dVon -1];
                                dienst.ColorName = PlanOption.FerienColorName;
                                dienst.RGB = PlanOption.FerienColorARGB;

                                dVon++;
                            }
                        }
                        else //if (mVon < mBis)
                        {
                            if (mVon == monat)
                            {
								if (Int16.Parse(DateUtil.GetYear(von)) == Year /*Verwaltung.Year*/)
                                {
                                    dVon += (c - 1);
                                    //while (column[c + dVon - 1] is PDienstColumn)
                                    while (column[dVon] is PDienstColumn)
                                    {
                                        //dienst = (PDienstColumn)column[c + dVon - 1];
                                        //dienst.ColorName = PlanOption.FerienColorName;
                                        //dienst.RGB = PlanOption.FerienColorARGB;

                                        //c++;

                                        dienst = (PDienstColumn)column[dVon];
                                        dienst.ColorName = PlanOption.FerienColorName;
                                        dienst.RGB = PlanOption.FerienColorARGB;

                                        dVon++;

                                        //if ((c + dVon - 1) >= column.Count)
                                        if ((dVon) >= column.Count)
                                            break;
                                    }
                                }
                            }
                            else //if (mBis == monat)
                            {
								if (Int16.Parse(DateUtil.GetYear(bis)) == Year /*Verwaltung.Year*/)
                                {
                                    //dBis += c;
                                    //while (c < dBis)
                                    //{
                                    //    dienst = (PDienstColumn)column[c];
                                    //    dienst.ColorName = PlanOption.FerienColorName;
                                    //    dienst.RGB = PlanOption.FerienColorARGB;

                                    //    c++;
                                    //}
                                    dBis += c;
                                    dVon = c;
                                    while (dVon < dBis)
                                    {
                                        dienst = (PDienstColumn)column[dVon];
                                        dienst.ColorName = PlanOption.FerienColorName;
                                        dienst.RGB = PlanOption.FerienColorARGB;

                                        dVon++;
                                    }
                                }
                            }
                        }
                    }
                    
                }
                
            }
        }

        #region Properties
        public List<IPersonColumn> Column
        {
            get { return column; }
            set { column = value; }
        }
        public int ATID
        {
            get { return atid; }
            set { atid = value; }
        }
        public string Name
        {
            get { return name; }
            set { name = value; }
        }
        public int PrsID
        {
            get { return prsID; }
            set { prsID = value; }
        }
        public int Monat
        {
            get { return monat; }
            set { monat = value; }
        }
        public int Saldo
        {
            get { return countBezogenenFerienTage; }
            //set { countBezogenenFerienTage = value; }
        }

		public int YearID
		{
			get;
			set;
		}

		public int Year
		{
			get;
			set;
		}

        #endregion
    }
}
