using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Gencode.Gengui.Forms;
using Gencode.Gengui.Display;
using Personalplaner.Gui.Mask;
using Personalplaner.Modules.Personoverview;

namespace Personalplaner
{
    public partial class MainForm : Gencode.Gengui.GUForm
    {
        public MainForm()
        {
            InitializeComponent();

			this.Text = Verwaltung.ProductName;
			Personalplaner.App.Util.RegisterProgressBar(MainProgressBar);
			versionLabel.Text = "v" + Verwaltung.ProductVersion;


			displayManager.AddDisplayControl(new DisplayableViewControler { HasHeader = false, Name = "default", Text = "default" },
				new Personalplaner.Modules.Base.DefaultView { Name = "defaultVeiw", Text = "default" },
				"default");

            displayManager.AddDisplayControl(contentPlan, mainPlan, "DienstPlan");

            displayManager.AddDisplayControl(contentPerson, mainPerson, "PersonaAnsicht");

            SetData();
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
#if DEBUG
            this.WindowState = FormWindowState.Normal;
#endif
        }

        public void SetData()
        {
            DPlanOrPerson.SetData();
        }

        #region PlanOrPerson

        public void SetControlVisibility(int type)
        {
            switch (type)
            {
                case 1:
					displayManager.SetView("DienstPlan");
                    break;
                case 2:
					displayManager.SetView("PersonaAnsicht");
                    break;
            }
        }

        #endregion

        #region Menu
        private void beendenToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void anstellungenToolStripMenuItem_Click(object sender, EventArgs e)
        {
			GUDialogBox dlg = new GUDialogBox(new NewAnstellung(), "Anstellungen");
			dlg.ShowDialog(this);
        }

        private void ferienToolStripMenuItem_Click(object sender, EventArgs e)
        {
			GUDialogBox dlg = new GUDialogBox(new MainFerien(), "Ferien");
			dlg.ShowDialog(this);
        }

        private void abteilungenToolStripMenuItem_Click(object sender, EventArgs e)
        {
			GUDialogBox dlg = new GUDialogBox(new newAbteilung(), "Abteilungen");
			dlg.ShowDialog(this);
        }

        private void diensteToolStripMenuItem_Click(object sender, EventArgs e)
        {
			GUDialogBox dlg = new GUDialogBox(new OpenDienst(), "Dienste");
			dlg.ShowDialog(this);

            contentPlan.DienstSel.InitializeList();
			contentPlan.DienstSel.InitializeDienstColor();
        }

        private void planErstellenToolStripMenuItem_Click(object sender, EventArgs e)
        {
			GUDialogBox dlg = new GUDialogBox(new CreatePlan(contentPlan, mainPlan, this), "Plan erstellen");
			dlg.ShowDialog(this);
        }

        private void abteilungPersonenToolStripMenuItem_Click(object sender, EventArgs e)
        {
			GUDialogBox dlg = new GUDialogBox(new AbtPersOverview(), "Abteilung - Person");
			dlg.ShowDialog(this);
        }

        private void optionenToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            Form childForm = new Personalplaner.Options.OptionsDlg();
            childForm.ShowDialog(this);
        }

        private void abgeschlossenePlneToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Form childForm = new AbgeschlossenenPlane();
            childForm.ShowDialog(this);

			contentPlan.PlanSel.SetComboPlan();
        }

        private void termineBearbeitenToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Form childForm = new TerminNewDlg();
            childForm.ShowDialog(this);
        }

        private void diensteToolStripMenuItem_Click_1(object sender, EventArgs e)
        {
            Form childForm = new DienstSerieDlg();
            childForm.ShowDialog(this);
        }

        private void planungsjahrErstellenToolStripMenuItem_Click(object sender, EventArgs e)
        {
            CreateYearDlg childForm = new CreateYearDlg();
            if (childForm.ShowDialog(this) == DialogResult.OK)
            {
                Verwaltung.SetYear();
            }
        }

        private void schemasToolStripMenuItem_Click(object sender, EventArgs e)
        {
			GUDialogBox dlg = new GUDialogBox(new CreateSchemaDlg(), "Schema");
			dlg.ShowDialog(this);
        }

        private void monateToolStripMenuItem_Click(object sender, EventArgs e)
        {
			GUDialogBox dlg = new GUDialogBox(new CreateMonthDlg(), "Monate bearbeiten");
            dlg.ShowDialog(this);
        }

		private void feiertageToolStripMenuItem_Click(object sender, EventArgs e)
		{
			GUDialogBox dlg = new GUDialogBox(new FeiertageEditDlg(), "Feiertage");
			dlg.ShowDialog(this);
		}

		private void personHinzufgenToolStripMenuItem_Click(object sender, EventArgs e)
		{
			Gencode.PrsUtil.Dialog.Stammdaten.PersonWizard.AddPersonWizard addprs = new Gencode.PrsUtil.Dialog.Stammdaten.PersonWizard.AddPersonWizard(Verwaltung.Mandant);
			addprs.Show(this);
		}

		private void PersonalstammToolStripMenuItem_Click(object sender, EventArgs e)
		{
			GUDialogBox selection = new GUDialogBox(new Gui.Mask.PersonalStamm(), "Personalstamm");
			selection.ShowDialog(this);
		}

		private void planToolStripMenuItem_Click(object sender, EventArgs e)
		{
			SetControlVisibility(1);
		}

		private void personanzeigenToolStripMenuItem_Click(object sender, EventArgs e)
		{
			SetControlVisibility(2);
		}

		private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
		{
			Verwaltung.Dispose();
		}

		private void newToolStripButton_Click(object sender, EventArgs e)
		{
			GUDialogBox dlg = new GUDialogBox(new CreatePlan(contentPlan, mainPlan, this), "Plan erstellen");
			dlg.ShowDialog(this);
		}

        #endregion

        #region Properties
		//public ContentPerson ContentPerson
		//{
		//    get { return contentPerson; }
		//    set { contentPerson = value; }
		//}

		//internal OverviewBaseView MainPerson
		//{
		//    get { return mainPerson; }
		//    set { mainPerson = value; }
		//}

        #endregion

		
    }
}