using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Gencode.Common.SQL;
using Gencode.Gengui.Forms;
using Gencode.PrsCommon.Data.Person;
using Gencode.PrsCommon.App;

namespace Personalplaner
{
    [ToolboxItem(false)]
	public partial class ArbeitspensumDlg : UserControl, IDialogControl
	{
		int prsID;
		DataView _pensum;
		bool _initialized = false;

		public ArbeitspensumDlg(int prsID)
		{
			this.prsID = prsID;
			InitializeComponent();
			SetData();
			InitializePensumGrid();



			_initialized = true;

			pensVon.Value = DateTime.Now;
			pensBis.Value = new DateTime(DateTime.Now.AddYears(20).Year, DateTime.Now.Month, DateTime.Now.Day);
		}

		private void SetData()
		{
			DataView dv = PersonData.GetGesamtPensum(prsID);
			if (dv.Count > 0)
			{
				txtProzent.Text = dv[0].Row["pensum"].ToString();
				pensVon.Text = DateUtil.ConvertSqlToDate(dv[0].Row["von"].ToString());
				pensBis.Text = DateUtil.ConvertSqlToDate(dv[0].Row["bis"].ToString());
			}
			dv.Dispose();
		}

		private void InitializePensumGrid()
		{
			_pensum = Gencode.PrsCommon.Data.Person.PersonData.GetPersonArbeitspensumHistory(prsID);
			pensumGrid.AutoGenerateColumns = false;
			pensumGrid.DataSource = _pensum;

			pensumGrid.Columns["von"].DataPropertyName = "Von";
			pensumGrid.Columns["bis"].DataPropertyName = "Bis";
			pensumGrid.Columns["prozent"].DataPropertyName = "Prozent";
			pensumGrid.Columns["status"].DataPropertyName = "Statustext";
		}

		private bool SaveData()
		{
			bool retval = true;
			int tmpProz = 0;

			try
			{
				tmpProz = int.Parse(txtProzent.Text);
			}
			catch
			{
				retval = false;
			}
			finally
			{
				if (tmpProz > 100 || tmpProz <= 0)
					retval = false;
			}

			if (pensVon.Value > pensBis.Value && pensVon.Enabled == true)
				retval = false;

			if (retval)
			{
				if (!chkKuendigung.Checked)
					retval = PersonData.SetPersonGesamtPensum(prsID, tmpProz, pensVon.Text, pensBis.Text);
				else
                    retval = PersonData.PersonPensumKuendigung(prsID, pensBis.Text);
			}

			if (retval)
				return false;
			return true;
		}

		private void chkKuendigung_CheckedChanged(object sender, EventArgs e)
		{
			if (chkKuendigung.Checked)
			{
				txtProzent.Enabled = false;
				pensVon.Enabled = false;
			}
			else
			{
				txtProzent.Enabled = true;
				pensVon.Enabled = true;
			}
		}

        private void AlterAbteilungPensum()
        {
            int pensum = 0;
            try
            {
                pensum = int.Parse(txtProzent.Text);
            }
            catch (InvalidCastException e)
            {
                Gencode.Common.ActionLogger.Logger.LoggMessage(Gencode.Common.ActionLogger.MessageType.USER_ERROR, "Change pensum failed. Invalid number.");
                Gencode.Common.Errorhandling.Logger.ToXML("Change pensum failed", "ArbeitspensumDlg.AlterAbteilungPensum()", e.Message);
            }

			GUDialogBox dlg = new GUDialogBox(new PersonSetAbteilung(prsID, pensVon.Value, pensBis.Value, pensum), "Abteilungen");
            dlg.ShowDialog();
        }

		#region IDialogControl Members

		public bool OKClicked()
		{
			//if (SaveData())
			//{
			//    if (!chkKuendigung.Checked)
			//    {
			//        //Pensum ist pro Abteilung
			//        AlterAbteilungPensum();
			//    }
			//    return true;
			//}
			//else
			//    MessageBox.Show("Es ist ein Fehler aufgetreten!", "Fehler", MessageBoxButtons.OK);

			return true;
		}

		public bool CancleClicked()
		{
			return true;
		}

		#endregion

		private void btnHistory_Click(object sender, EventArgs e)
		{
			Personalplaner.Modules.Base.PrsArbeitspensumHistory controler = new Personalplaner.Modules.Base.PrsArbeitspensumHistory(prsID);
			Gencode.Gengui.History.HistoryDialog hist = new Gencode.Gengui.History.HistoryDialog(controler);
			hist.ShowDialog();
		}

		private void pensVon_ValueChanged(object sender, EventArgs e)
		{
			if (_initialized)
				if (pensVon.Value.Day != 1)
				{
					pensVon.Value = new DateTime(pensVon.Value.Year, pensVon.Value.Month, 1);
				}
		}

		private void pensBis_ValueChanged(object sender, EventArgs e)
		{
			if(_initialized)
				if (pensBis.Value.Day != DateTime.DaysInMonth(pensBis.Value.Year, pensBis.Value.Month))
				{
					pensBis.Value = new DateTime(pensBis.Value.Year, pensBis.Value.Month, DateTime.DaysInMonth(pensBis.Value.Year, pensBis.Value.Month));
				}
		}

		private void btnAdd_Click(object sender, EventArgs e)
		{
            int pens = 0;
            int.TryParse(txtProzent.Text, out pens);
            if (pens <= 0)
            {
                MessageBox.Show(this, "Bitte geben Sie ein Pensum ein.", "Kein Pensum", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                return;
            }


			if (MessageBox.Show(this, "Pensum wird per Stichdatum gendert.\nSind Sie sicher dass das Pensum per Stichdatum-Von gendert werden soll?",
				"Pensum ndern",
				MessageBoxButtons.OKCancel,
				 MessageBoxIcon.Warning) == DialogResult.OK)
			{
				if (SaveData())
				{
					if (!chkKuendigung.Checked)
					{
						//Pensum ist pro Abteilung
						AlterAbteilungPensum();
					}

                    SetData();
                    InitializePensumGrid();
				}
				else
					MessageBox.Show("Beim speichern der Daten ist ein Fehler aufgetreten!", "Fehler", MessageBoxButtons.OK);
			}
		}
	}
}