using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Gencode.PrsCommon.App.AppData;
using Gencode.PrsCommon.App;
using Gencode.PrsCommon.Data.Plan;
using Gencode.Common.SQL;
using Personalplaner.Modules.Dienstplan;
//using System.Data.Odbc;

namespace Personalplaner
{
    [ToolboxItem(false)]
	public partial class CreatePlan : UserControl, Gencode.Gengui.Forms.IDialogControl
    {
        Personalplaner.Modules.Dienstplan.DiensplanSelector contPlan;
        DienstplanContainer planMain;
        MainForm mainForm;

		DataView m_months;
		DataView m_schema;

        public CreatePlan()
        {
            InitializeComponent();
        }

        public CreatePlan(Personalplaner.Modules.Dienstplan.DiensplanSelector contPlan, DienstplanContainer planMain, MainForm mainForm)
        {
            InitializeComponent();

            this.contPlan = contPlan;
            this.planMain = planMain;
            this.mainForm = mainForm;

            //SetSchemaCombo();
            SetMonatCombo();
            setZeiten();

			SetSchemaCombo();
        }
        
        private void newPlan_Load(object sender, EventArgs e)
        {
        }

        private string dateChange(string date)
        {
			return Convert.ToString(DateUtil.ToInt(date));
        }

        private void SetMonatCombo()
        {
			m_months = Gencode.PrsCommon.Data.Common.CommonData.GetActiveMonths(Verwaltung.Mandant);
			comboMonat.DataSource = m_months;
			comboMonat.DisplayMember = "Name";
        }

        private void SetSchemaCombo()
        {
			m_schema = PlanData.GetSchemas(Verwaltung.Mandant, true);
			comboSchema.DataSource = m_schema;
			comboSchema.DisplayMember = "name";

			if (m_schema.Count == 0)
			{
				if (MessageBox.Show(this,
					"Es ist kein aktives Schema vorhanden.\nOhne ein Schema kann kein Plan erstellt werden.\n\nMchten Sie ein neues Schema erstellen oder altes wieder aktivieren?",
					"Kein Schema vorhanden", 
					MessageBoxButtons.OKCancel, 
					MessageBoxIcon.Question) == DialogResult.OK)
				{
					Gencode.Gengui.Forms.GUDialogBox dlg = new Gencode.Gengui.Forms.GUDialogBox(new CreateSchemaDlg(), "Schema");
					if (dlg.ShowDialog(this) == DialogResult.OK)
						SetSchemaCombo();
				}
			}

			if (comboSchema.Items.Count > 1)
				comboSchema.SelectedIndex = comboSchema.Items.Count - 1;
        }

        private void setZeiten()
        {
			this.dateBegin.Text = DateUtil.FirstDateNextMonth();
			this.dateEnd.Text = DateUtil.LastDateNextMonth();
        }

        private void comboMonat_SelectedIndexChanged(object sender, EventArgs e)
        {
			dateBegin.Text = DateUtil.ConvertSqlToDate(m_months[comboMonat.SelectedIndex].Row["von"].ToString());
			dateEnd.Text = DateUtil.ConvertSqlToDate(m_months[comboMonat.SelectedIndex].Row["bis"].ToString());
        }

        private void comboSchema_SelectedIndexChanged(object sender, EventArgs e)
        {
        }

		#region IDialogControl Members

		public bool OKClicked()
		{
			int planID;
			if (txtPlanName.Text == "")
			{
				MessageBox.Show("Bitte geben Sie einen Planname ein", "Kein Name vorhanden", MessageBoxButtons.OK);
				txtPlanName.Focus();
				return false;
			}

			if (comboSchema.SelectedIndex < 0)
			{
				MessageBox.Show("Bitte whlen Sie ein Schema aus", "Kein Schema vorhanden", MessageBoxButtons.OK);
				comboSchema.Focus();
				return false;
			}

			int atID = int.Parse(m_months[comboMonat.SelectedIndex].Row["ATID"].ToString());

			//check if months have days set
			if (Gencode.PrsCommon.Data.Common.CommonData.GetDaysInMonth(atID, Verwaltung.Mandant) <= 0)
			{
				if (MessageBox.Show(this, "Diesem Monat wurde noch keine Anzahl an Arbeitstagen zugewiesen.\n" +
					"Es muss eine Anzahl an Arbeitstagen angegeben werden damit die Berechnung der Arbeitsstunden korrekt funktioniert.\n" +
					"Mchten Sie die Anzahl an Arbeitstagen jetzt angeben?",
					"Arbeitstage im Monat",
					MessageBoxButtons.OKCancel,
					MessageBoxIcon.Exclamation,
					MessageBoxDefaultButton.Button1) == DialogResult.OK)
				{
					Gencode.Gengui.Forms.GUDialogBox dlg = new Gencode.Gengui.Forms.GUDialogBox(new CreateMonthDlg(atID), "Monate bearbeiten");
					dlg.ShowDialog(this);
				}
			}

			if (!PlanData.CheckIfPlanExists(atID,
								int.Parse(m_schema[comboSchema.SelectedIndex].Row["schemaid"].ToString()), 
								Verwaltung.YearID))
			{
				bool continuous = true;//Fortlaufendes schema verwenden

				if (checkLaufendeSchemas.Checked)
					continuous = false;//speziefisches Schema verwenden

				planID = PlanData.CreateNewPlan(txtPlanName.Text, int.Parse(m_months[comboMonat.SelectedIndex].Row["ATID"].ToString()),
					int.Parse(m_schema[comboSchema.SelectedIndex].Row["schemaid"].ToString()), continuous);
			}
			else
			{
				MessageBox.Show("In dem angegebenen Zeitraum existiert bereis ein Plan mit diesem Schema,\noder einer der Abteilungen im Schema wird bereits in dem angegebenen Zeitraum eingesetzt.", "Plan existiert bereits", MessageBoxButtons.OK);
				return false;
			}

			Gencode.PrsCommon.Data.Dienst.DienstData.InitDienstSerie(planID);

			this.contPlan.PlanSel.SetComboPlan();
			//// * TODO:
			//// * Change
			mainForm.SetControlVisibility(1);
			this.contPlan.PlanSel.SetComboPlanIndex(planID);

			return true;
		}

		public bool CancleClicked()
		{
			//do nothing
			return true;
		}

		#endregion
	}
}