using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Gencode.PrsCommon.Data.Plan;
using Gencode.Gengui.Forms;


namespace Personalplaner
{
    [ToolboxItem(false)]
	public partial class CreateSchemaDlg : UserControl, Gencode.Gengui.Forms.ISaveNewDialog
    {
		/// <summary>
		/// is true if something in the schema changed. should save when ok is pressed
		/// </summary>
		bool changed = false;

        public CreateSchemaDlg()
        {
            InitializeComponent();

            GridSchema.ReloadData(Verwaltung.Mandant, checkShowAll.Checked);
            
            btnDiensteZuweisen.Enabled = false;
        }

        private void RefreshLayoutGrid(int schemaid)
        {
            GridLayout.ReloadData(schemaid);
        }

        private void addAbteilung_Click(object sender, EventArgs e)
        {
			AddAbteilungToSchemaDlg addAbt = new AddAbteilungToSchemaDlg();
			if (addAbt.ShowDialog() == DialogResult.OK)
			//GUDialog dlg = new GUDialog(new AddAbteilungToSchemaDlg(), "Abteilung hinzufgen");
			//if(dlg.ShowDialog() == DialogResult.OK)
			{
				((DataView)GridLayout.DataSource).Dispose();
				foreach (DataRowView row in addAbt.Abteilungen)
				{
					GridLayout.AddAbteilungToSchema(int.Parse(row["abteilungid"].ToString()));
				}
			    btnAddAbt.Enabled = false;

				changed = true;
			}
        }

        private void btnAddPrs_Click(object sender, EventArgs e)
        {
			if (GridLayout.Rows.Count > 0)
			{
				AddPrsToSchemaDlg dlg = new AddPrsToSchemaDlg(GridSchema.SchemaID);
				//OKCancleDialog dlg = new OKCancleDialog(new AddPrsToSchemaDlg(GridSchema.SchemaID), "Person hinzufgen");
				if (dlg.ShowDialog() == DialogResult.OK)
				{
					int abteilungID = dlg.AbteilungID;
					int prsID = dlg.PrsID;

					int oldAbtId = GridLayout.GetAbteilungID(0);
					for (int i = 0; i < GridLayout.Rows.Count; i++)
					{
						if (((abteilungID == oldAbtId) && (abteilungID != GridLayout.GetAbteilungID(i))) ||
							((abteilungID == oldAbtId) && (i == GridLayout.Rows.Count - 1)))
						{
							Gencode.Common.ActionLogger.Logger.LoggMessage(Gencode.Common.ActionLogger.MessageType.USER_EVENT, "Add person "
								+ prsID + " to abteilung " + abteilungID + " to schema " + GridSchema.SchemaID + " in CreateSchemaDlg");

							GridLayout.InsertPersonAt(i, prsID, abteilungID);
							return;
						}
						oldAbtId = GridLayout.GetAbteilungID(i);
					}

					changed = true;
				}
			}
        }

        
        private void btnDiensteZuweisen_Click(object sender, EventArgs e)
        {
			if (GridLayout.Rows.Count > 0 && int.Parse(((DataView)GridLayout.DataSource)[GridLayout.SelectedRows[0].Index].Row["schemaid"].ToString()) >= 0)
			{
				Gencode.Gengui.Forms.GUDialogBox dlg = new Gencode.Gengui.Forms.GUDialogBox(new AddDienstToSchema(GridSchema.SchemaID), "Dienst zuweisen");
				dlg.ShowDialog();
			}
        }

        private void btnRemove_Click(object sender, EventArgs e)
        {
			if (GridLayout.Rows.Count > 0)
			{
				if (GridLayout.SelectedRows[0].Index >= 0)
				{
					Gencode.Common.ActionLogger.Logger.LoggMessage(Gencode.Common.ActionLogger.MessageType.USER_EVENT, "Remove person "
						+ GridLayout["prsid", GridLayout.SelectedRows[0].Index].ToString()
						+ " from schema " + GridSchema.SchemaID + " in CreateSchemaDlg");

					GridLayout.RemoveRowAt(GridLayout.SelectedRows[0].Index);
					changed = true;
				}
			}
        }

        private void checkShowAll_CheckedChanged(object sender, EventArgs e)
        {
            GridSchema.ReloadData(Verwaltung.Mandant, checkShowAll.Checked);
        }

        private void btnDeactivate_Click(object sender, EventArgs e)
        {
			if (GridSchema.RowCount > 0)
				GridSchema.ChangeSchemaStatus(GridSchema.SelectedRows[0].Index);
        }

        private void GridSchema_CellClick(object sender, DataGridViewCellEventArgs e)
        {
			txtName.Enabled = false;
			btnAddAbt.Enabled = false;
            btnDiensteZuweisen.Enabled = true;
			btnAddPrs.Enabled = true;
			btnRemove.Enabled = true;
			btnDeactivate.Enabled = true;
			checkNewSchema.Checked = false;
            GridLayout.ReloadData(GridSchema.SchemaID);
			txtName.Text = ((DataView)GridSchema.DataSource)[GridSchema.SelectedCells[0].RowIndex].Row["name"].ToString();
        }

		#region IDeleteNewSaveDialog Members

		public void NewClicked()
		{
			btnAddAbt.Enabled = true;
			txtName.Enabled = true;
            btnDiensteZuweisen.Enabled = false;
			btnAddPrs.Enabled = false;
			btnRemove.Enabled = false;
			btnDeactivate.Enabled = false;
			GridLayout.ReloadData(-1);
		}

		public void SaveClicked()
		{
			int schemaindex = 0;
			if(GridSchema.Rows.Count > 0) 
				schemaindex = GridSchema.SelectedRows[0].Index;

			DataView schema = GridLayout.DataSource as DataView;
			if (schema.Count > 0)
			{
				int schemaid = int.Parse(schema[0].Row["schemaid"].ToString());
				if (schemaid == 0)
				{
					if (txtName.Text != string.Empty)
					{
						schemaid = PlanData.CreateNewSchema(txtName.Text, Verwaltung.Mandant);
						schemaindex = GridSchema.Rows.Count;
					}
					else
					{
						MessageBox.Show("Es ist kein Schemaname vorhanden.", "Kein Name", MessageBoxButtons.OK);
                        //schema.Dispose();
						return;
					}
				}
				else
                    if (!checkNewSchema.Checked)
					PlanData.DeleteRowsFromSchema(schemaid);

					if (checkNewSchema.Checked)
					{
						if (txtName.Text != string.Empty)
							schemaid = PlanData.CreateNewSchema(txtName.Text, Verwaltung.Mandant);
						else
						{
							MessageBox.Show("Es ist kein Schemaname vorhanden.", "Kein Name", MessageBoxButtons.OK);
							schema.Dispose();
							return;
						}
					}

				for (int i = 1; i <= schema.Count; i++)
				{
					PlanData.SaveSchemaRow(schemaid, i, int.Parse(schema[i - 1].Row["prsid"].ToString()),
						int.Parse(schema[i - 1].Row["abteilungid"].ToString()),
						int.Parse(schema[i - 1].Row["rgb"].ToString()));
				}
			}
			schema.Dispose();


			btnAddPrs.Enabled = true;
			btnRemove.Enabled = true;
			btnDeactivate.Enabled = true;
            btnDiensteZuweisen.Enabled = true;

			GridSchema.ReloadData(Verwaltung.Mandant, checkShowAll.Checked);
			GridSchema.Rows[schemaindex].Selected = true;
			GridLayout.ReloadData(int.Parse(((DataView)GridSchema.DataSource)[GridSchema.SelectedCells[0].RowIndex].Row["schemaid"].ToString()));

			changed = false;
		}

		public bool OKClicked()
		{
			if (changed)
			{
				DialogResult dlgres = MessageBox.Show("Es wurden Daten verndert.\nMchten Sie diese nderungen speichern?", "nderungen speichern?", MessageBoxButtons.YesNoCancel);
				if (dlgres == DialogResult.Yes)
				{
					SaveClicked();
				}
				else if (dlgres == DialogResult.Cancel)
					return false; //do nothing
			}

			//close dialog
			return true;
		}

		public bool CancleClicked()
		{
			//do nothing
			//just close
			return true;
		}

		#endregion

		private void checkNewSchema_CheckedChanged(object sender, EventArgs e)
		{
			if (checkNewSchema.Checked)
				txtName.Enabled = true;
			else
				txtName.Enabled = false;
		}

		private void button1_Click(object sender, EventArgs e)
		{
			if (GridLayout.Rows.Count > 0 && GridLayout.SelectedRows[0].Index >= 0)
			{
				ColorDialog color = new ColorDialog();
				if (color.ShowDialog() == DialogResult.OK)
				{
					GridLayout.AddColorToRowAt(GridLayout.SelectedRows[0].Index, color.Color);
					changed = true;
				}
			}
		}

		private void GridLayout_DataBindingComplete(object sender, DataGridViewBindingCompleteEventArgs e)
		{

		}
	}
}