using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Gencode.Common.SQL;
using Gencode.PrsCommon.Data.Common;
using Gencode.PrsCommon.App;

namespace Personalplaner
{
    public partial class CreateYearDlg : Form
    {
        int year = 0;

        public CreateYearDlg()
        {
            InitializeComponent();

            //setZeiten();
            SetComboJear();
        }

        private void SetZeitenJahr()
        {
            //Sets the Date to next Month
            this.dateBegin.Text = "01.01." + comboJear.Text;
            this.dateEnd.Text = "31.12." + comboJear.Text;
        }

        public int Year
        {
            get
            {
                return year;
            }
        }

        private void btnOK_Click(object sender, EventArgs e)
        {
            bool check = true;
            if (btnCreate.Enabled == true)
            {
                if (MessageBox.Show("Soll das Planungsjahr erstellt werden?", "", MessageBoxButtons.YesNo) == DialogResult.Yes)
                {
                    check = createYear();
                }
            }

            if (check)
            {
                DialogResult = DialogResult.OK;
                this.Close();
				Application.Restart();
            }
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            this.Close();
        }

		private bool createYear()
		{
			string von = Convert.ToString(DateUtil.ToInt(dateBegin.Text));
			string bis = Convert.ToString(DateUtil.ToInt(dateEnd.Text));

			if (comboJear.Text == "") { MessageBox.Show("Bitte geben Sie eine Jahrzahl ein", "Kein Jahr", MessageBoxButtons.OK); return false; }
			int jahr = Int32.Parse(comboJear.Text);

			if (MessageBox.Show(this, "Das neue Jahr wird das Aktive Jahr und das alte Jahr wird abgeschlossen.\n"+
				"Sind sie sicher dass die das alte Jahr abschliessen mchten?", 
				"Jahr erstellen?", 
				MessageBoxButtons.YesNo, 
				MessageBoxIcon.Exclamation) == DialogResult.Yes)
			{
				this.Enabled = false;

				int oldyear = 0;
				DataView dv = SimpleQuerry.ExecuteForDataView("select JahrID from PlanungsJahr where Status = 1 and mandid = '" + Verwaltung.Mandant + "'");
				if (dv.Count > 0)
				{
					oldyear = int.Parse(dv[0].Row[0].ToString());
				}

				CommonData.CreateJear(jahr, von, bis, Verwaltung.Mandant);
				//Querry stp = new Querry("set_planungsjahr");

				//stp.AddParameter("@jahr", jahr);
				//stp.AddParameter("@von", von);
				//stp.AddParameter("@bis", bis);

				//stp.Execute();

				year = jahr;

				dv = SimpleQuerry.ExecuteForDataView("select wert,textwert from Options where typ = 'Monat' order by wert");
				int month = 0;
				string name = "";
				string mVon = "";
				string mBis = "";

				for (int i = 0; i < dv.Count; i++)
				{
					try
					{
						month = int.Parse(dv[i].Row[0].ToString());
						name = dv[i].Row[1].ToString();
						mVon = setDateBegin(month);
						mBis = setDateEnd(month);
					}
					catch (Exception exp)
					{
						Gencode.Common.Util.ErrorHandler.LoggError("Year Error", "CreateYearDlg.createYear()", exp.Message);
					}
					finally
					{
						Gencode.PrsCommon.Data.Plan.PlanData.SaveMonth(year, month, name, mVon, mBis, Verwaltung.Mandant);
					}
				}

				if (oldyear > 0)
				{
					// Personen in personzeiten auf neues Jahr nachfhren
					dv = SimpleQuerry.ExecuteForDataView("select p.PrsId from Person p inner join Arbeitspensum ap on ap.PrsID = p.PrsId and ap.Bis > getdate() where p.PersonTyp = 1");
					for (int i = 0; i < dv.Count; i++)
					{
						Querry stp = new Querry("set_personzeitennew");

						stp.AddParameter("@prsid", int.Parse(dv[i].Row[0].ToString()));
						stp.AddParameter("@oldjahrid", oldyear);
						stp.Execute();
					}
				}
				//else
				//{
					//Error.ThrowErrorToLog("Kein aktives Planungsjahr vorhanden", "CreateYearDlg.createYear()");
					//Error.TrowErrorMessage("Es ist kein aktives Planungsjahr vorhanden.\nBitte kontaktieren sie Ihren Administrator", "Kein Planungsjahr");
				//}

				this.Enabled = true;
			}
			return true;
		}

        private string setDateBegin(int month)
        {
            string mo;
            if (month < 10)
            {
                mo = "0" + month;
            }
            else
            {
                mo = "" + month;
            }
            return year + "" + mo  +"01";
        }

        private string setDateEnd(int month)
        {
            string mo;
            string da;
            int day = DateTime.DaysInMonth(year, month);

            if (month < 10)
            {
                mo = "0" + month;
            }
            else
            {
                mo = "" + month;
            }

            if (day < 10)
            {
               da  = "0" + day;
            }
            else
            {
                da = "" + day;
            }
            return year + mo + da;
        }

        private void btnCreate_Click(object sender, EventArgs e)
        {
            bool check = true;
            check = createYear();

            if (check)
            {
                btnMonth.Enabled = true;
                btnFeiertage.Enabled = true;

                comboJear.Enabled = false;
                btnCreate.Enabled = false;
                btnOldYear.Enabled = false;
                
				Verwaltung.SetYear();
            }
        }

        private void btnCopy_Click(object sender, EventArgs e)
        {
            MessageBox.Show("Muss noch erstellt werden");
        }

        private void SetComboJear()
        {
            // Check if ther are any old jears
            int maxJahr = Gencode.PrsCommon.Data.Plan.PlanData.GetMaxYear();

			if(maxJahr == 0)
                maxJahr = DateTime.Today.Year - 1;

            maxJahr++;
            for (int i = 0; i < 10; i++)
            {
                comboJear.Items.Add("" + maxJahr);
                maxJahr++;
            }
        }

        private void comboJear_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (comboJear.SelectedIndex >= 0)
            {
                SetZeitenJahr();
            }
        }

        private void btnOldYear_Click(object sender, EventArgs e)
        {
            OldYearDlg oldYear = new OldYearDlg();
            
            if (oldYear.ShowDialog() != DialogResult.Cancel)
            {
                comboJear.Text = "" + oldYear.getYear();
                year = Int32.Parse(comboJear.Text);

                btnMonth.Enabled = true;
                btnFeiertage.Enabled = true;

                comboJear.Enabled = false;
                btnCreate.Enabled = false;
                btnOldYear.Enabled = false;
            }
        }

        private void btnMonth_Click(object sender, EventArgs e)
        {
			Gencode.Gengui.Forms.GUDialogBox dlg = new Gencode.Gengui.Forms.GUDialogBox(new CreateMonthDlg(), "Monate bearbeiten");
			dlg.ShowDialog();
			//Form monthDlg = new CreateMonthDlg(/*year*/);
			//monthDlg.ShowDialog();
        }

		private void btnFeiertage_Click(object sender, EventArgs e)
		{
			Gencode.Gengui.Forms.GUDialogBox dlg = new Gencode.Gengui.Forms.GUDialogBox(new FeiertageEditDlg(), "Feiertage");
			dlg.ShowDialog();
		}
    }
}