using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Gencode.PrsCommon.Data.Dienst;

namespace Personalplaner.Gui.Mask
{
    [ToolboxItem(false)]
    public partial class OpenDienst : UserControl, Gencode.Gengui.Forms.IDeleteNewSaveDialog
    {
        public OpenDienst()
        {
            InitializeComponent();

			comboTyp.InitData(Verwaltung.Mandant);
			comboDienst.InitData(Verwaltung.Mandant);
        }

        private void comboDienst_SelectedIndexChanged(object sender, EventArgs e)
        {
			txtName.Text = comboDienst.SelectedDienstName;
			txtKurzText.Text = comboDienst.SelectedDienstKurztext;
			txtBeschr.Text = comboDienst.SelectedDienstLangtext;

            if (comboDienst.SelectedDienstARGB != 0)
                txtFarbe.BackColor = Color.FromArgb(comboDienst.SelectedDienstARGB);
            else
                txtFarbe.BackColor = Color.FromArgb(-1);

			comboTyp.SetIndexAt(comboDienst.SelectedDienstTyp);

			checkRechnen.Checked = comboDienst.SelectedDienstGerechnet;
        }

        private void updateDB()
        {
            int check = 0;
            if (checkRechnen.Checked == true)
                check = 1;

			int rgb = 0;
			if(txtFarbe.BackColor.ToArgb() != -1)
				rgb = txtFarbe.BackColor.ToArgb();

			DienstData.UpdateDienst(comboDienst.SelectedDienstID, txtName.Text, txtKurzText.Text, txtBeschr.Text, check, rgb, comboTyp.SelectedDienstTyp, 0);

            comboDienst.Text = "";
            txtName.Text = "";
            txtKurzText.Text = "";
            txtBeschr.Text = "";
        }

        private void saveInDB()
        {            
            int check = 0;
            if(checkRechnen.Checked == true)
                check = 1;

			int rgb = 0;
			if (txtFarbe.BackColor.ToArgb() != -1)
				rgb = txtFarbe.BackColor.ToArgb();

			DienstData.SaveDienst(txtName.Text, txtKurzText.Text, txtBeschr.Text, check, rgb, comboTyp.SelectedDienstTyp, 0, Verwaltung.Mandant);

            txtName.Text = "";
            txtKurzText.Text = "";
            txtBeschr.Text = "";
        }

        private void openDienst_Load(object sender, EventArgs e)
        {

        }

        private void btnColor_Click(object sender, EventArgs e)
        {
            ColorDialog color = new ColorDialog();
            if (color.ShowDialog() == DialogResult.OK)
            {
                txtFarbe.BackColor = Color.FromArgb(color.Color.ToArgb());
            }
        }

		#region INewOKCancelDialog Members

		public void NewClicked()
		{
			if (comboDienst.Items.Count > 0)
				comboDienst.SelectedIndex = 0;
			if (comboTyp.Items.Count > 0)
				comboTyp.SelectedIndex = 0;
		}

		#endregion

		#region IDialogControl Members

		public bool OKClicked()
		{
			return true;
		}

		public bool CancleClicked()
		{
			return true;
		}

		#endregion

		#region IDeleteNewSaveDialog Members

		public void DeleteClicked()
		{
			if (comboDienst.SelectedDienstID > 0)
			{
				if (MessageBox.Show("Sind Sie sicher dass sie den Dienst: " + comboDienst.Text + "\n wirklich deaktivieren wollen", "Dienst deaktivieren?", MessageBoxButtons.OKCancel) == DialogResult.OK)
				{
					DienstData.DeactivateDienst(comboDienst.SelectedDienstID);
					comboDienst.RefreshData();
				}
			}
		}

		#endregion

		#region ISaveDialog Members

		public void SaveClicked()
		{
			if (txtName.Text == "" && txtFarbe.BackColor.ToArgb() == -1)
			{
				MessageBox.Show("Es wurde kein Anzeigename oder Farbe definiert.\nBitte geben Sie entweder einen Namen oder eine Farbe an.", "Kein Anzeigename oder Farbe", MessageBoxButtons.OK);
				return;
			}

			if (txtKurzText.Text == "")
			{
				MessageBox.Show("Kein Kurztext vorhanden...", "", MessageBoxButtons.OK);
				return;
			}

			if (txtBeschr.Text == "")
			{
				MessageBox.Show("Keine Beschreibung vorhanden...", "", MessageBoxButtons.OK);
				return;
			}

			if (comboTyp.SelectedIndex < 0)
			{
				MessageBox.Show("Es ist kein Typ vorhanden", "", MessageBoxButtons.OK);
				return;
			}

			if (comboDienst.SelectedDienstID > 0)
				updateDB();
			else
				saveInDB();

			txtName.Text = "";
			txtKurzText.Text = "";
			txtBeschr.Text = "";
			checkRechnen.Checked = true;
			comboTyp.SelectedIndex = 0;

			//Combo neu abfllen
			comboDienst.RefreshData();
		}

		#endregion

		private void btnZeiten_Click(object sender, EventArgs e)
		{

			if (comboDienst.SelectedDienstID > 0)
			{
				Gencode.Gengui.Forms.GUDialogBox dlg = new Gencode.Gengui.Forms.GUDialogBox(new DienstZeitDlg(comboDienst.SelectedDienstID), "Dienstzeiten");
				dlg.ShowDialog(this);
			}
			else
			{
				if (MessageBox.Show(this,
					"Der Dienst wurde noch nicht gespeichert.\nMchten Sie speichern?", "Speichern",
					 MessageBoxButtons.OKCancel,
					 MessageBoxIcon.Question) == DialogResult.OK)
				{
					SaveClicked();
				}
			}
		}
	}
}