using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Gencode.PrsCommon.Data.Person;
using Gencode.PrsCommon.App;
using Gencode.Common.SQL;
using Gencode.PrsCommon.Data.Abteilung;

namespace Personalplaner
{
    [ToolboxItem(false)]
    public partial class PersonSetAbteilung : UserControl, Gencode.Gengui.Forms.IDialogControl
    {
		int m_maxProzent;
		int prsID;
		DataView _pensum;
		bool _initialized = false;

        public PersonSetAbteilung()
        {
            InitializeComponent();

			_initialized = true;
        }

        public PersonSetAbteilung(int PrsID)
        {
            InitializeComponent();
            prsID = PrsID;

			InitializeListData();
			InitializePensumGrid();

			_initialized = true;

			dateVon.Value = DateTime.Now;
			dateBis.Value = new DateTime(DateTime.Now.AddYears(20).Year, DateTime.Now.Month, DateTime.Now.Day);
        }

		public PersonSetAbteilung(int PrsID, DateTime datumvon, DateTime datumbis, int maxpensum)
		{
			InitializeComponent();
			prsID = PrsID;

			InitializeListData();
			InitializePensumGrid();

			_initialized = true;

			try
			{
				dateVon.Value = datumvon;
				dateBis.Value = datumbis;
			}
			catch { }

            txtPensGesammt.Text = maxpensum.ToString();
            m_maxProzent = maxpensum;
		}

		private void InitializePensumGrid()
		{
			_pensum = Gencode.PrsCommon.Data.Person.PersonData.GetPersonAbteilungHistory(prsID);
			pensumGrid.AutoGenerateColumns = false;
			pensumGrid.DataSource = _pensum;

			pensumGrid.Columns["abteilung"].DataPropertyName = "Name";
			pensumGrid.Columns["von"].DataPropertyName = "Von";
			pensumGrid.Columns["bis"].DataPropertyName = "Bis";
			pensumGrid.Columns["prozent"].DataPropertyName = "Prozent";
			pensumGrid.Columns["status"].DataPropertyName = "Statustext";
		}

		private void InitializeListData()
		{
			listExisting.Assigned = false;

			listAdded.InitData(prsID, Verwaltung.Mandant);
			listExisting.InitData(prsID, Verwaltung.Mandant);
			
			m_maxProzent = PersonData.GetArbeitspensum(prsID);
			m_maxProzent -= listAdded.GetTotalPensum;

			txtPensGesammt.Text = m_maxProzent.ToString();
		}

		private void RefreshData()
		{
			//listExisting.InitializeDataSource();
			//listAdded.InitializeDataSource();
			
			m_maxProzent = PersonData.GetArbeitspensum(prsID, dateVon.Text);
			m_maxProzent -= listAdded.GetTotalPensum;

			txtPensAbt.Text = "0";
			txtPensGesammt.Text = m_maxProzent.ToString();
		}

		private void btnAdd_Click_1(object sender, EventArgs e)
		{
			if (listExisting.SelectedAbteilungID > 0)
			{
				int pens = 0;
				try
				{
					pens = int.Parse(txtPensAbt.Text);
				}
				catch 
				{
					MessageBox.Show(this, "Es wurde kein Pensum-Prozentsatz ausgewhlt.\nBitte geben Sie ein Prozentsatz ein.", "Kein Prozentsatz", MessageBoxButtons.OK);
					txtPensAbt.Focus();
				}

				if (pens > 0 && pens <= m_maxProzent)
				{
					if (dateVon.Value < dateBis.Value)
					{
						listAdded.AddItem(listExisting.SelectedRow, pens);
						listExisting.RemoveItem(listExisting.SelectedRow);

						RefreshData();
					}
				}
				else
				{
					MessageBox.Show(this, "Ungltiger Pensum-Prozentsatz.\nBitte wlhlen Sie einen Prozentsatz zwischen 0 und " + m_maxProzent + " aus", "Ungltiger Prozentsatz", MessageBoxButtons.OK);
					txtPensAbt.Focus();
				}
			}
		}

		private void button2_Click(object sender, EventArgs e)
		{
			if (listAdded.SelectedAbteilungID > 0)
			{
				listExisting.AddItem(listAdded.SelectedRow, 0);
				listAdded.RemoveItem(listAdded.SelectedRow);

				RefreshData();
			}
		}

		#region IDialogControl Members

		public bool OKClicked()
		{
			return true;
		}

		public bool CancleClicked()
		{
			return true;
		}

		#endregion

		private void dateVon_ValueChanged(object sender, EventArgs e)
		{
			if (_initialized)
				if (dateVon.Value.Day != 1)
				{
					dateVon.Value = new DateTime(dateVon.Value.Year, dateVon.Value.Month, 1);
				}

			
			m_maxProzent = PersonData.GetArbeitspensum(prsID, dateVon.Value.ToShortDateString());
			m_maxProzent -= listAdded.GetTotalPensum;

			txtPensGesammt.Text = m_maxProzent.ToString();
			 /**/
		}

		private void dateBis_ValueChanged(object sender, EventArgs e)
		{
			if (_initialized)
				if (dateBis.Value.Day != DateTime.DaysInMonth(dateBis.Value.Year, dateBis.Value.Month))
				{
					dateBis.Value = new DateTime(dateBis.Value.Year, dateBis.Value.Month, DateTime.DaysInMonth(dateBis.Value.Year, dateBis.Value.Month));
				}
		}

		private void btnHistory_Click(object sender, EventArgs e)
		{
			Personalplaner.Modules.Base.PrsAbteilungHistory controler = new Personalplaner.Modules.Base.PrsAbteilungHistory(prsID);
			Gencode.Gengui.History.HistoryDialog hist = new Gencode.Gengui.History.HistoryDialog(controler);
			hist.ShowDialog();
		}

		private void button1_Click(object sender, EventArgs e)
		{
			if (MessageBox.Show(this, "Pensum wird per Stichdatum gendert.\nSind Sie sicher dass das Pensum per Stichdatum-Von gendert werden soll?",
				"Pensum ndern",
				MessageBoxButtons.OKCancel,
				 MessageBoxIcon.Warning) == DialogResult.OK)
			{
				PersonData.RemovePersonAbteilung(prsID, 0, dateVon.Text, dateBis.Text);

				foreach (DataRow row in listAdded.AbteilungDataTable.Rows)
				{
					int pens = 0;
					int abteilungid = 0;
					try
					{
						pens = int.Parse(row["prozent"].ToString());
					}
					catch { }
					try
					{
						abteilungid = int.Parse(row["abteilungid"].ToString());
					}
					catch { }

					if (abteilungid > 0)
						PersonData.SetAbteilung(prsID, abteilungid, dateVon.Text, dateBis.Text, pens);
				}


				InitializePensumGrid();
				InitializeListData();
				RefreshData();
			}
		}
	}
}