using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Gencode.PrsCommon.Data.Plan.Termin;
using Gencode.PrsCommon.App;

namespace Personalplaner
{
    public partial class TerminAddDlg : Form
    {
		DataView m_termine;
		DataView m_terminart;
		DataView m_personenNotInTermin;
		DataView m_personenInTermin;

		int planid;
        string planDateVon;

        public TerminAddDlg(int planid)
        {
            InitializeComponent();

			this.planid = planid;
			this.planDateVon = DateUtil.ConvertSqlToDate(Gencode.PrsCommon.Data.Plan.PlanData.GetPlanDate(planid)[0].Row[0].ToString());

			InitData();
        }

		private void InitData()
		{
			SetComboTerminArt();
			SetComboTermin();

			InitPersonenList();
			InitAddedPersonList();
		}

        private void btnTermin_Click(object sender, EventArgs e)
        {
            Form newTerm = new TerminNewDlg();
            newTerm.ShowDialog();

            SetComboTerminArt();
        }

        private void btnOK_Click(object sender, EventArgs e)
        {
            bool valid = true;
            if (comboTerminArt.SelectedIndex < 0) { MessageBox.Show("Sie haben keine Terminart ausgewhlt.", "Termin whlen", MessageBoxButtons.OK); valid = false; }
            if (listZugewiesenPrs.Items.Count <= 0) { MessageBox.Show("Sie haben dem Termin keine Personene zugeteilt.", "Person zuteilen", MessageBoxButtons.OK); valid = false; }
            if (txtName.Text == "") { MessageBox.Show("Sie haben keine Namen angegeben.", "Name angegeben", MessageBoxButtons.OK); valid = false; }
            if (txtDate.Value.Month != int.Parse(DateUtil.GetMonth(planDateVon)) || txtDate.Value.Year != int.Parse(DateUtil.GetYear(planDateVon)))
            {
                MessageBox.Show("Datum ausserhalb des gltigen Planes.\nBitte ein anderes Datum auswhlen.", "Falsches Datum", MessageBoxButtons.OK);
                return;
            }

            if (valid)
            {
                SaveData();
                this.Close();
            }
        }

        private void SaveData()
        {
            int planterminid = 1;

            if (comboTermine.SelectedIndex >= 0)
            {
				planterminid = int.Parse(m_termine[comboTermine.SelectedIndex].Row["PlanTerminID"].ToString());
				TerminData.DeleteTerminPlan(planterminid, planid);
            }
            else
            {
				planterminid = TerminData.GetMaxPlanTerminID(planid);
            }

			for (int i = 0; i < m_personenInTermin.Count; i++)
			{
				TerminData.AddPlanTermin(planterminid, 
					planid,
					int.Parse(m_personenInTermin[i].Row["PrsId"].ToString()),
					txtName.Text,
					txtDate.Text,
					int.Parse(m_personenInTermin[i].Row["AbteilungID"].ToString()),
					int.Parse(m_terminart[comboTerminArt.SelectedIndex].Row["TerminID"].ToString()));
			}

			InitData();
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void InitPersonenList()
        {
			int planterminid = 0;
			if (comboTermine.SelectedIndex >= 0)
				planterminid = int.Parse(m_termine[comboTermine.SelectedIndex].Row["PlanTerminID"].ToString());

			m_personenNotInTermin = TerminData.GetPersonenNotInTermin(planid,planterminid);
			listAllePersonen.DataSource = m_personenNotInTermin;
			listAllePersonen.DisplayMember = "Anzeigename";
        }

        private void InitAddedPersonList()
        {
			int planterminid = 0;
			if (comboTermine.SelectedIndex >= 0)
				planterminid = int.Parse(m_termine[comboTermine.SelectedIndex].Row["PlanTerminID"].ToString());

			m_personenInTermin = TerminData.GetPersonenInTermin(planterminid, planid);
			listZugewiesenPrs.DataSource = m_personenInTermin;
			listZugewiesenPrs.DisplayMember = "Anzeigename";
        }

        private void SetComboTerminArt()
        {
			m_terminart = TerminData.GetTerminArten(Verwaltung.Mandant);
			comboTerminArt.DataSource = m_terminart;
			comboTerminArt.DisplayMember = "Name";
        }

        private void SetComboTermin()
        {
			m_termine = TerminData.GetPlanTermine(planid);
			comboTermine.DataSource = m_termine;
			comboTermine.DisplayMember = "Name";
        }

        private void btnAdd_Click(object sender, EventArgs e)
        {
            if (listAllePersonen.SelectedIndex >= 0)
            {
				if (m_personenInTermin == null)
					CreatePersonenDataView(ref m_personenInTermin);

				DataRow row = m_personenInTermin.Table.NewRow();
				row["PrsId"] = m_personenNotInTermin[listAllePersonen.SelectedIndex].Row["PrsId"];
				row["Vorname"] = m_personenNotInTermin[listAllePersonen.SelectedIndex].Row["Vorname"];
				row["Nachname"] = m_personenNotInTermin[listAllePersonen.SelectedIndex].Row["Nachname"];
				row["AbteilungID"] = m_personenNotInTermin[listAllePersonen.SelectedIndex].Row["AbteilungID"];
				row["Anzeigename"] = m_personenNotInTermin[listAllePersonen.SelectedIndex].Row["Anzeigename"];

				m_personenInTermin.Table.Rows.Add(row);

				m_personenNotInTermin.Table.Rows.RemoveAt(listAllePersonen.SelectedIndex);
            }
        }

		private void CreatePersonenDataView(ref DataView dv)
		{
			if (dv == null)
				dv = new DataView();

			dv.Table = new DataTable("1");
			DataColumn dc = new DataColumn();
			dc.ColumnName = "PrsId";
			dv.Table.Columns.Add(dc);

			dc = new DataColumn();
			dc.ColumnName = "Vorname";
			dv.Table.Columns.Add(dc);

			dc = new DataColumn();
			dc.ColumnName = "Nachname";
			dv.Table.Columns.Add(dc);

			dc = new DataColumn();
			dc.ColumnName = "Anzeigename";
			dv.Table.Columns.Add(dc);
		}

        private void btnRem_Click(object sender, EventArgs e)
        {
            if (listZugewiesenPrs.SelectedIndex >= 0)
            {
				DataRow row = m_personenNotInTermin.Table.NewRow();
				row["PrsId"] = m_personenInTermin[listZugewiesenPrs.SelectedIndex].Row["PrsId"];
				row["Vorname"] = m_personenInTermin[listZugewiesenPrs.SelectedIndex].Row["Vorname"];
				row["Nachname"] = m_personenInTermin[listZugewiesenPrs.SelectedIndex].Row["Nachname"];
				row["AbteilungID"] = m_personenInTermin[listZugewiesenPrs.SelectedIndex].Row["AbteilungID"];
				row["Anzeigename"] = m_personenInTermin[listZugewiesenPrs.SelectedIndex].Row["Anzeigename"];

				m_personenNotInTermin.Table.Rows.Add(row);

				m_personenInTermin.Table.Rows.RemoveAt(listZugewiesenPrs.SelectedIndex);
            }
        }

        private void comboTermin_SelectedIndexChanged(object sender, EventArgs e)
        {
        }

		private void btnNew_Click(object sender, EventArgs e)
        {
			SetComboTerminArt();
			SetComboTermin();

            comboTermine.SelectedIndex = -1;
            comboTerminArt.SelectedIndex = -1;

			InitPersonenList();
			InitAddedPersonList();

            txtName.Text = "";
            txtName.Focus();
        }

        private void comboTermine_SelectedIndexChanged(object sender, EventArgs e)
        {
			if (comboTermine.SelectedIndex >= 0)
			{
				InitPersonenList();
				InitAddedPersonList();

				int trmId = int.Parse(m_termine[comboTermine.SelectedIndex].Row["TerminId"].ToString());
				int i = 0;
				for (; i < m_terminart.Count; i++)
				{
					if (int.Parse(m_terminart[i].Row["TerminID"].ToString()) == trmId)
					{
						comboTerminArt.SelectedIndex = i;
						break;
					}
				}

				txtDate.Text = m_termine[comboTermine.SelectedIndex].Row["Datum"].ToString();
				txtName.Text = m_termine[comboTermine.SelectedIndex].Row["Name"].ToString();
			}
        }
    }
}
