using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Printing;
using Gencode.PrsCommon.Data.Plan;
using Gencode.PrsCommon.App;
using Personalplaner.Gui.Mask;
using Gencode.Gengui.Forms;

namespace Personalplaner.Modules.Dienstplan
{
    [ToolboxItem(false)]
	public partial class Plan : Gencode.Gengui.GUControl
	{
		Personalplaner.Modules.Dienstplan.Print.PlanGridPrinterClass _printerClass;
        Personalplaner.Modules.Dienstplan.Print.GridPrint _gridPrintDocument;
		DienstplanContainer mainPlan;

		bool _initialized = false;


		public Plan(int planID, DienstplanContainer mainPlan)
		{
			InitializeComponent();

            PlanID = planID;
            this.mainPlan = mainPlan;

            InitializeBase();
        }

		private void CPlan_Load(object sender, EventArgs e)
        {
            FormatGrid();
            InitDienstGrid();

			lblPlName.Text = GridData.PlanName;
			lblSchema.Text = GridData.SchemaName;
			lblVon.Text = GridData[0].Von;
			lblBis.Text = GridData[0].Bis;

			SetSplitterDistance();

			_initialized = true;
        }

		

		#region Grid

		private Gencode.PrsCommon.Data.Dienst.DienstSchema m_dinstschema;
		private DataView DienstSchemaTable
		{
			get
			{
				if (m_dinstschema == null || m_dinstschema.Table == null)
					m_dinstschema = new Gencode.PrsCommon.Data.Dienst.DienstSchema(GridData.SchemaID,
						Verwaltung.Mandant, ReservedColumns, GridData.Data.Columns.Count - ReservedColumns);
				return m_dinstschema.Table;
			}
		}

		/// <summary>
		/// [dienstid,gerechnet,einsatzprotag,dienstname,kurztext,langtext,abteilungid,abteilungname]
		/// </summary>
		private DataView DienstSchemaRawData
		{
			get
			{
				if (m_dinstschema == null || m_dinstschema.RawData == null)
					m_dinstschema = new Gencode.PrsCommon.Data.Dienst.DienstSchema(GridData.SchemaID,
						Verwaltung.Mandant, ReservedColumns, GridData.Data.Columns.Count - ReservedColumns);
				return m_dinstschema.RawData;
			}
		}

		private void AlterDienstGridCell(int abteilungid, int column)
		{
			for (int row = 0; row < DienstSchemaRawData.Count; row++)
			{
				if (int.Parse(DienstSchemaRawData[row].Row["abteilungid"].ToString()) == abteilungid)
				{
					int dienstid = int.Parse(DienstSchemaRawData[row].Row["dienstid"].ToString());

					int anzahldienste = 0;
					for (int r = 0; r < gridMainPlan.Rows.Count - ReservedRows; r++)
					{
						if (GridData[r].AbteilungID == abteilungid &&
							GridData[r, column - ReservedColumns].DienstID == dienstid)
							anzahldienste++;
					}

					int sum = (int.Parse(DienstSchemaRawData[row].Row["einsatzprotag"].ToString()) * -1) + anzahldienste;

					DienstSchemaTable[row].Row[column] = sum.ToString();

					if (sum == 0)
						gridDienste[column, row].Style.BackColor = Color.FromArgb(Gencode.PrsCommon.App.PlanOption.DienstCountEnoughColor);
					else if (sum < 0)
						gridDienste[column, row].Style.BackColor = Color.FromArgb(Gencode.PrsCommon.App.PlanOption.DienstCountMinusColor);
					else if (sum > 0)
						gridDienste[column, row].Style.BackColor = Color.FromArgb(Gencode.PrsCommon.App.PlanOption.DienstCountPlusColor);
				}
			}
		}

		private void InitializeBase()
		{
			GridData = new Grid(PlanID);
			gridMainPlan.DataSource = GridData.Data;

			gridSaldo.DataSource = GridData.SaldoDataTable;

			gridDienste.DataSource = DienstSchemaTable;
		}



		private void FormatGrid()
		{
			gridMainPlan.Font = new Font(GridData.Style.Font, GridData.Style.FontSize);
			gridMainPlan.AllowUserToOrderColumns = false;

			//Parse all columns to set Size and Syle
			for (int c = 0; c < gridMainPlan.Columns.Count; c++)
			{
				if (c == 0)//Personname
				{
					gridMainPlan.Columns[c].DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft;
					gridMainPlan.Columns[c].HeaderCell.Style.Alignment = DataGridViewContentAlignment.MiddleLeft;
				}
				else if (c > 0 && c == ReservedColumns-1)//Prozent
				{
					gridMainPlan.Columns[c].DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft;
					gridMainPlan.Columns[c].HeaderCell.Style.Alignment = DataGridViewContentAlignment.MiddleLeft;
				}
				else
				{
					gridMainPlan.Columns[c].DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleCenter;
					gridMainPlan.Columns[c].HeaderCell.Style.Alignment = DataGridViewContentAlignment.MiddleCenter;
				}

				gridMainPlan.Columns[c].SortMode = DataGridViewColumnSortMode.NotSortable;
			}

            //Color satColor = Color.FromArgb(Gencode.PrsCommon.App.PlanOption.SaturdayColorRGB);
            //Color sunColor = Color.FromArgb(Gencode.PrsCommon.App.PlanOption.SundayColorRGB);
            Feiertage = PlanData.GetFeiertageInPlan(PlanID);

			//Parse all rows
			for (int r = 0; r < GridData.Rows.Length; r++)
			{
                SetRowColors(r);

                ////Row Colors for Name
                //if (GridData[r].ARGB != 0 && GridData[r].ARGB != -1)
                //    gridMainPlan[0, r + ReservedRows].Style.BackColor = Color.FromArgb(GridData[r].ARGB);
                //else
                //    //TODO: make parameterizeable in db
                //    gridMainPlan[0, r + ReservedRows].Style.BackColor = Color.FromName("Gainsboro");

                ////Row Colors for Percent
                //if (ShowPercent)
                //    if (GridData[r].ARGB != 0 && GridData[r].ARGB != -1)
                //        gridMainPlan[1, r + ReservedRows].Style.BackColor = Color.FromArgb(GridData[r].ARGB);
                //    else
                //        //TODO: make parameterizeable in db
                //        gridMainPlan[1, r + ReservedRows].Style.BackColor = Color.FromName("Gainsboro");

                ////Parse all cells in Row
                //for (int c = 0; c < GridData[r].Cells.Length; c++)
                //{
                //    //1. Weekend Colors
                //    if (GridData[r, c].Day == DayOfWeek.Saturday)
                //        gridMainPlan[c + ReservedColumns, r + ReservedRows].Style.BackColor = satColor;
                //    if (GridData[r, c].Day == DayOfWeek.Sunday)
                //        gridMainPlan[c + ReservedColumns, r + ReservedRows].Style.BackColor = sunColor;

                //    if (IsDayFeiertag(GridData[r, c].Datum))
                //        gridMainPlan[c + ReservedColumns, r + ReservedRows].Style.BackColor = sunColor;

                //    //2. Row Colors
                //    if (GridData[r].ARGB != 0 && GridData[r].ARGB != -1)
                //        gridMainPlan[c + ReservedColumns, r + ReservedRows].Style.BackColor = Color.FromArgb(GridData[r].ARGB);

                //    //3. Grid (dienst) colors
                //    if (GridData[r, c].ARGB != 0 && GridData[r, c].ARGB != -1)
                //        gridMainPlan[c + ReservedColumns, r + ReservedRows].Style.BackColor = Color.FromArgb(GridData[r, c].ARGB);
                //}
			}

			//Saldo Grid
			foreach (DataGridViewColumn column in gridSaldo.Columns)
			{
				column.DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft;
				column.HeaderCell.Style.Alignment = DataGridViewContentAlignment.MiddleCenter;
				column.SortMode = DataGridViewColumnSortMode.NotSortable;
			}

			for (int i = 0; i < gridSaldo.RowCount; i++)
			{
				if (GridData[i].ARGB != -1 && GridData[i].ARGB != 0)
				{
					for (int c = 0; c < gridSaldo.ColumnCount; c++)
					{
						gridSaldo[c, i].Style.BackColor = Color.FromArgb(GridData[i].ARGB);
						//gridSaldo.Rows[i].DefaultCellStyle.BackColor = Color.FromArgb(GridData[i].ARGB);
					}
				}
			}
		}

		private void InitDienstGrid()
		{
			if (DienstSchemaTable != null && DienstSchemaTable.Count > 0)
			{
				gridDienste.AllowUserToOrderColumns = false;

				//Size
				int index = 0;
				if (ReservedColumns > 1)
				{
					int maxwidth = gridMainPlan.Columns[0].Width + gridMainPlan.Columns[1].Width;
					gridDienste.Columns[index].Width = 70;
					gridDienste.Columns[index].SortMode = DataGridViewColumnSortMode.NotSortable;
					if (gridMainPlan.ColumnCount > 1 && gridMainPlan.RowCount > 1)
						gridDienste.Columns[index].DefaultCellStyle.BackColor = gridMainPlan[0, 0].Style.BackColor;

					index++;
					gridDienste.Columns[index].Width = maxwidth - gridDienste.Columns[index - 1].Width;
					gridDienste.Columns[index].SortMode = DataGridViewColumnSortMode.NotSortable;
					if (gridMainPlan.ColumnCount > 1 && gridMainPlan.RowCount > 1)
						gridDienste.Columns[index].DefaultCellStyle.BackColor = gridMainPlan[0, 0].Style.BackColor;

					index++;
				}
				else
				{
					gridDienste.Columns[index].Width = gridMainPlan.Columns[0].Width;
					gridDienste.Columns[index].SortMode = DataGridViewColumnSortMode.NotSortable;
					if (gridMainPlan.ColumnCount > 1 && gridMainPlan.RowCount > 1)
						gridDienste.Columns[index].DefaultCellStyle.BackColor = gridMainPlan[0, 0].Style.BackColor;

					index++;
				}

				for (; index < gridMainPlan.Columns.Count; index++)
				{
					gridDienste.Columns[index].Width = gridMainPlan.Columns[index].Width;
					gridDienste.Columns[index].SortMode = DataGridViewColumnSortMode.NotSortable;

					int abteilungid = 0;
					for (int r = 0; r < DienstSchemaRawData.Count; r++)
					{
						if (abteilungid != int.Parse(DienstSchemaRawData[r].Row["abteilungid"].ToString()))
						{
							AlterDienstGridCell(int.Parse(DienstSchemaRawData[r].Row["abteilungid"].ToString()), index);
							abteilungid = int.Parse(DienstSchemaRawData[r].Row["abteilungid"].ToString());
						}
					}
				}
				lblDienstInfo.Visible = false;
			}
			else
			{
				lblDienstInfo.Visible = true;
				gridDienste.Visible = false;
			}
		}
		
        private void SetRowColors(int row)
        {
            Color satColor = Color.FromArgb(Gencode.PrsCommon.App.PlanOption.SaturdayColorRGB);
            Color sunColor = Color.FromArgb(Gencode.PrsCommon.App.PlanOption.SundayColorRGB);

            if (GridData[row].ARGB != 0 && GridData[row].ARGB != -1)
                gridMainPlan[0, row + ReservedRows].Style.BackColor = Color.FromArgb(GridData[row].ARGB);
            else
                //TODO: make parameterizeable in db
                gridMainPlan[0, row + ReservedRows].Style.BackColor = Color.FromName("Gainsboro");

            //Row Colors for Percent
            if (ShowPercent)
                if (GridData[row].ARGB != 0 && GridData[row].ARGB != -1)
                    gridMainPlan[1, row + ReservedRows].Style.BackColor = Color.FromArgb(GridData[row].ARGB);
                else
                    //TODO: make parameterizeable in db
                    gridMainPlan[1, row + ReservedRows].Style.BackColor = Color.FromName("Gainsboro");

            //Parse all cells in Row
            for (int c = 0; c < GridData[row].Cells.Length; c++)
            {
                //1. Weekend Colors
                if (GridData[row, c].Day == DayOfWeek.Saturday)
                    gridMainPlan[c + ReservedColumns, row + ReservedRows].Style.BackColor = satColor;
                if (GridData[row, c].Day == DayOfWeek.Sunday)
                    gridMainPlan[c + ReservedColumns, row + ReservedRows].Style.BackColor = sunColor;

                if (IsDayFeiertag(GridData[row, c].Datum))
                    gridMainPlan[c + ReservedColumns, row + ReservedRows].Style.BackColor = sunColor;

                //2. Row Colors
                if (GridData[row].ARGB != 0 && GridData[row].ARGB != -1)
                    gridMainPlan[c + ReservedColumns, row + ReservedRows].Style.BackColor = Color.FromArgb(GridData[row].ARGB);

                //3. Grid (dienst) colors
                if (GridData[row, c].ARGB != 0 && GridData[row, c].ARGB != -1)
                    gridMainPlan[c + ReservedColumns, row + ReservedRows].Style.BackColor = Color.FromArgb(GridData[row, c].ARGB);
            }
        }

		#endregion

		#region Methods

		public void ResetWidth()
		{
			int gridwidth = 0;
			int panel1Width = 0;
			for (int i = 0; i < gridMainPlan.Columns.Count; i++)
			{
				gridwidth += gridMainPlan.Columns[i].Width;
			}

			panel1Width += (gridwidth + splitContainer2.Margin.Left);

			gridwidth = 0;
			for (int i = 0; i < gridSaldo.Columns.Count; i++)
			{
				gridwidth += gridSaldo.Columns[i].Width;
			}
			//foreach (DataGridViewColumn col in gridSaldo.Columns)
			//{
			//    gridwidth += col.Width;
			//}

			gridwidth += (splitContainer2.Margin.Right + splitContainer2.SplitterWidth + splitContainer5.Margin.Left + splitContainer5.Margin.Right);

			splitContainer5.SplitterDistance = gridwidth;

			displayPanel.Width = gridwidth + panel1Width;

			splitContainer2.SplitterDistance = panel1Width;
			splitContainer4.SplitterDistance = panel1Width;
		}

		private bool IsDayFeiertag(string date)
		{
			bool retval = false;
			if (Feiertage != null && Feiertage.Count > 0)
			{
				for (int i = 0; i < Feiertage.Count; i++)
				{
					if (DateUtil.ConvertSqlToDate(Feiertage[i].Row["Von"].ToString().Substring(0,10)) == date)
					{
						retval = true;
						i = Feiertage.Count;
					}
				}
			}
			return retval;
		}

		#endregion

		#region Events

		private void SetSplitterDistance()
		{
			//height
			int panelHeight = 0;
			int grid1Height = 0;

			grid1Height = ((gridMainPlan.RowCount * gridMainPlan.Rows[0].Height) + gridMainPlan.ColumnHeadersHeight);
			panelHeight += splitContainer1.Margin.Top + splitContainer1.Margin.Bottom;
			grid1Height += splitContainer3.Margin.Top;
			panelHeight += grid1Height;

			if (gridDienste.Visible)
				panelHeight += ((gridDienste.RowCount * gridDienste.Rows[0].Height) + gridDienste.ColumnHeadersHeight);
			else
				panelHeight += splitContainer3.Panel2MinSize;

			displayPanel.Height = panelHeight;
			splitContainer3.SplitterDistance = grid1Height;

			//width
			int gridwidth = 0;
			int panel1Width = 0;
			for (int i = 0; i < gridMainPlan.Columns.Count; i++)
			{
				gridwidth += gridMainPlan.Columns[i].Width;
			}

			panel1Width += (gridwidth + splitContainer2.Margin.Left);

			gridwidth = 0;
			for (int i = 0; i < gridSaldo.Columns.Count; i++)
			{
				gridwidth += gridSaldo.Columns[i].Width;
			}

			gridwidth += (splitContainer2.Margin.Right + splitContainer2.SplitterWidth + splitContainer5.Margin.Left + splitContainer5.Margin.Right);

			splitContainer5.SplitterDistance = gridwidth;

			displayPanel.Width = gridwidth + panel1Width;

			splitContainer2.SplitterDistance = panel1Width;
			splitContainer4.SplitterDistance = panel1Width;
		}

		private void PlanAktualisieren()
		{
			InitializeBase();
			FormatGrid();
			InitDienstGrid();
		}

		private void gridMainPlan_ColumnWidthChanged(object sender, DataGridViewColumnEventArgs e)
		{
			if (_initialized)
				ResetWidth();
		}

		private void gridSaldo_ColumnWidthChanged(object sender, DataGridViewColumnEventArgs e)
		{
			if (_initialized)
				ResetWidth();
		}

		private void gridMainPlan_CellClick(object sender, DataGridViewCellEventArgs e)
		{
			if (e.RowIndex >= ReservedRows
				&& e.ColumnIndex >= ReservedColumns
				&& GridData[e.RowIndex - ReservedRows].RowType == Row.ROWTYPE.Person)
			{
				if (PlanUtil.UseDienst)
				{
					//1. save dienst
					Gencode.PrsCommon.Data.Dienst.DienstData.SaveDienstInPlan(PlanID,
						GridData[e.RowIndex - ReservedRows].PrsID,
						GridData[e.RowIndex - ReservedRows].AbteilungID,
						GridData[e.RowIndex - ReservedRows, e.ColumnIndex - ReservedColumns].Datum,
						PlanUtil.SelectedDienstID);

					//2. set data to grid
					GridData[e.RowIndex - ReservedRows, e.ColumnIndex - ReservedColumns].DienstID = PlanUtil.SelectedDienstID;
					GridData[e.RowIndex - ReservedRows, e.ColumnIndex - ReservedColumns].DienstName = PlanUtil.SelectedDienst;

					if (PlanUtil.SelectedDienstARGB == -1 || PlanUtil.SelectedDienstARGB == 0)
					{
						//If color is white and day is saturday or sunday
						//then set corect color
						if (GridData[e.RowIndex - ReservedRows, e.ColumnIndex - ReservedColumns].Day == DayOfWeek.Saturday)
						{
							GridData[e.RowIndex - ReservedRows, e.ColumnIndex - ReservedColumns].ARGB =
								Gencode.PrsCommon.App.PlanOption.SaturdayColorRGB;
						}
						else if (GridData[e.RowIndex - ReservedRows, e.ColumnIndex - ReservedColumns].Day == DayOfWeek.Sunday)
						{
							GridData[e.RowIndex - ReservedRows, e.ColumnIndex - ReservedColumns].ARGB =
								Gencode.PrsCommon.App.PlanOption.SundayColorRGB;
						}
						else if (IsDayFeiertag(GridData[e.RowIndex - ReservedRows, e.ColumnIndex - ReservedColumns].Datum))
						{
							GridData[e.RowIndex - ReservedRows, e.ColumnIndex - ReservedColumns].ARGB =
										Gencode.PrsCommon.App.PlanOption.SundayColorRGB;
						}
						else
							GridData[e.RowIndex - ReservedRows, e.ColumnIndex - ReservedColumns].ARGB = PlanUtil.SelectedDienstARGB;
					}
					else
						GridData[e.RowIndex - ReservedRows, e.ColumnIndex - ReservedColumns].ARGB = PlanUtil.SelectedDienstARGB;

					gridMainPlan[e.ColumnIndex, e.RowIndex].Value = PlanUtil.SelectedDienst;

					//set to normal background color if color is white or none
					if (GridData[e.RowIndex - ReservedRows, e.ColumnIndex - ReservedColumns].ARGB == 0)
						gridMainPlan[e.ColumnIndex, e.RowIndex].Style.BackColor = gridMainPlan.BackgroundColor;
					else
						gridMainPlan[e.ColumnIndex, e.RowIndex].Style.BackColor =
							Color.FromArgb(GridData[e.RowIndex - ReservedRows, e.ColumnIndex - ReservedColumns].ARGB);

					//3. set data to saldo grid
					GridData.ReloadSaldoRow(e.RowIndex, ReservedRows);

					//4. set dienst count in grid
					AlterDienstGridCell(GridData[e.RowIndex - ReservedRows].AbteilungID, e.ColumnIndex);

					//reset width because that can change depending on the value
					gridDienste.Columns[e.ColumnIndex].Width = gridMainPlan.Columns[e.ColumnIndex].Width;
				}
				
                    //    break;
                    //case PlanValueType.Color:
				if (PlanUtil.UseColor)
				{
					//1. save color
					Gencode.PrsCommon.Data.Dienst.DienstData.SaveColorInPlan(PlanID,
						GridData[e.RowIndex - ReservedRows].PrsID,
						GridData[e.RowIndex - ReservedRows].AbteilungID,
						GridData[e.RowIndex - ReservedRows, e.ColumnIndex - ReservedColumns].Datum,
						PlanUtil.ARGB);

					//2. set data to grid
					if (PlanUtil.ARGB == -1 || PlanUtil.ARGB == 0)
					{
						//If color is white and day is saturday or sunday
						//then set corect color
						if (GridData[e.RowIndex - ReservedRows, e.ColumnIndex - ReservedColumns].Day == DayOfWeek.Saturday)
						{
							GridData[e.RowIndex - ReservedRows, e.ColumnIndex - ReservedColumns].ARGB =
								Gencode.PrsCommon.App.PlanOption.SaturdayColorRGB;
						}
						else if (GridData[e.RowIndex - ReservedRows, e.ColumnIndex - ReservedColumns].Day == DayOfWeek.Sunday)
						{
							GridData[e.RowIndex - ReservedRows, e.ColumnIndex - ReservedColumns].ARGB =
								Gencode.PrsCommon.App.PlanOption.SundayColorRGB;
						}
						else if (IsDayFeiertag(GridData[e.RowIndex - ReservedRows, e.ColumnIndex - ReservedColumns].Datum))
						{
							GridData[e.RowIndex - ReservedRows, e.ColumnIndex - ReservedColumns].ARGB =
										Gencode.PrsCommon.App.PlanOption.SundayColorRGB;
						}
						else
							GridData[e.RowIndex - ReservedRows, e.ColumnIndex - ReservedColumns].ARGB = PlanUtil.ARGB;
					}
					else
						GridData[e.RowIndex - ReservedRows, e.ColumnIndex - ReservedColumns].ARGB = PlanUtil.ARGB;

					//set to normal background color if color is white or none
					if (GridData[e.RowIndex - ReservedRows, e.ColumnIndex - ReservedColumns].ARGB == 0)
						gridMainPlan[e.ColumnIndex, e.RowIndex].Style.BackColor = gridMainPlan.BackgroundColor;
					else
						gridMainPlan[e.ColumnIndex, e.RowIndex].Style.BackColor =
							Color.FromArgb(GridData[e.RowIndex - ReservedRows, e.ColumnIndex - ReservedColumns].ARGB);
				}
			}

			gridMainPlan[0, 0].Selected = true;
			//gridMainPlan.Invalidate();
			//gridMainPlan.InvalidateCell(e.ColumnIndex, e.RowIndex);
		}

        private void splitContainer2_SizeChanged(object sender, EventArgs e)
        {
        }

        private void splitContainer2_SplitterMoved(object sender, SplitterEventArgs e)
        {
            if (_initialized)
                splitContainer4.SplitterDistance = splitContainer2.SplitterDistance;
        }

        private void gridSaldo_CellClick(object sender, DataGridViewCellEventArgs e)
        {
            gridSaldo[e.ColumnIndex, e.RowIndex].Selected = false;
            gridSaldo[0, 0].Selected = true;
        }

        private void gridDienste_CellClick(object sender, DataGridViewCellEventArgs e)
        {
            gridDienste[e.ColumnIndex, e.RowIndex].Selected = false;
            gridDienste[0, 0].Selected = true;
        }

		#endregion

		#region Printer

		private void printDocument1_PrintPage(object sender, PrintPageEventArgs e)
		{
			// Add Title to printout
            String textToPrint = PlanData.GetPlanName(PlanID);
			Font printFont = new Font("Microsoft Sans Serif", 12);
			e.Graphics.DrawString(textToPrint, printFont, Brushes.Black, 100, 20);

			if (_printerClass.Print(e.Graphics))
			{
                Personalplaner.App.Util.AddProgress(10);
				e.HasMorePages = true;
			}
            else
                Personalplaner.App.Util.SetProgress(0);

			//TODO: Terminname sollte mit der Farbe gedruckt werden
			//      in Optionen ein und ausschaltbar
		}

        /// <summary>
        /// Prepares the printdocuments
        /// Sets properties like margins, papersize etc.
        /// </summary>
		private void PreparePrintDocuments()
		{
			//default print document
			printDocument.DefaultPageSettings.Landscape = true;
			printDocument.DefaultPageSettings.Margins.Left = 40;
			printDocument.DefaultPageSettings.Margins.Top = 60;
			//printDocument1.DefaultPageSettings.Margins.Bottom = 40;
			//printDocument1.DefaultPageSettings.PaperSize = new PaperSize("PaperA4", 826, 1169);
			printDocument.DocumentName = PlanData.GetPlanName(PlanID);

            if (printDocument.PrinterSettings.SupportsColor)
                printDocument.DefaultPageSettings.Color = true;

			//grid print document
			_gridPrintDocument = new Personalplaner.Modules.Dienstplan.Print.GridPrint();
			_gridPrintDocument.MainGrid = gridMainPlan;
			_gridPrintDocument.TimeGrid = gridSaldo;
			_gridPrintDocument.PageSize = printDocument.DefaultPageSettings.PaperSize.Width;
			_gridPrintDocument.GridData = GridData;
		}
		

		#endregion

		#region Properties

        private int ReservedRows
        {
            get
            {
                return 0;
            }
        }

        private int ReservedColumns
        {
            get
            {
                if (ShowPercent)
                    return 2;

                return 1;
            }
        }

        int m_showPercent = -1;
        private bool ShowPercent
        {
            get
            {
                if (m_showPercent < 0)
                    m_showPercent = Gencode.PrsCommon.App.PlanOption.ShowPersonPercentInPlan;

                if (m_showPercent == 1)
                    return true;

                return false;
            }
        }

        Grid m_gridData;
        private Grid GridData
        {
            get
            {
                if (m_gridData == null)
                    throw new NotImplementedException();
                return m_gridData;
            }
            set
            {
                m_gridData = value;
            }
        }

		private DataView Feiertage
		{
			get;
			set;
		}

		public int PlanID
        {
            get;
            private set;
        }

		public string PlanName
		{
			get
			{
				if (GridData != null && !string.IsNullOrEmpty(GridData.PlanName))
					return GridData.PlanName;
				return string.Empty;
			}
		}

		#endregion

		#region Menu

		private void gridMainPlan_CellToolTipTextNeeded(object sender, DataGridViewCellToolTipTextNeededEventArgs e)
		{
			if (e.RowIndex >= 0)
			{
				string text = gridMainPlan[0, e.RowIndex].Value.ToString();
				if (e.ColumnIndex > 1 && gridMainPlan[e.ColumnIndex, e.RowIndex].Value.ToString() != string.Empty)
					text += "\n" + gridMainPlan[e.ColumnIndex, e.RowIndex].Value.ToString();
				e.ToolTipText = text;
			}
		}

		private void aktualisierenToolStripMenuItem_Click(object sender, EventArgs e)
		{
            //Refresh
			PlanAktualisieren();
		}

		private void schliessenToolStripMenuItem_Click(object sender, EventArgs e)
		{
			mainPlan.CloseTab(PlanID);
		}

		private void abschliessenToolStripMenuItem_Click(object sender, EventArgs e)
		{
			if (MessageBox.Show("Sind Sie sicher dass Sie den Plan abschliessen mchten?", "Plan abschliessen?", MessageBoxButtons.YesNo) == DialogResult.Yes)
			{
				Gencode.Common.ActionLogger.Logger.LoggMessage(Gencode.Common.ActionLogger.MessageType.USER_EVENT, "Finish plan " + PlanID + " clicked");
                PlanData.UpdatePlanStatus(PlanID, PlanStatus.Closed);

				//Tab lschen
				this.mainPlan.planAbschliessen(PlanID);
			}
		}

		private void planLschenToolStripMenuItem_Click(object sender, EventArgs e)
		{
			if (MessageBox.Show(this,
				"Sind Sie sicher dass Sie den Plan lschen wollen?\nAlle Plandaten werden unwiederruflich gelscht!", 
				"Lschen", 
				MessageBoxButtons.OKCancel, 
				MessageBoxIcon.Warning) == DialogResult.OK)
			{
				Gencode.Common.ActionLogger.Logger.LoggMessage(Gencode.Common.ActionLogger.MessageType.USER_EVENT, "Delete plan " + PlanID + " clicked");
                PlanData.DeletePlan(PlanID);

				// Tab lschen
				this.mainPlan.planAbschliessen(PlanID);
			}
		}

		private void planLayoutAnpassenToolStripMenuItem_Click(object sender, EventArgs e)
		{
			GUDialogBox dlg = new GUDialogBox(new PlanLayout(PlanID), "Plan einrichten");
			if (dlg.ShowDialog() == DialogResult.OK)
			{
				//Refresh
				InitializeBase();
				FormatGrid();
			}
		}

        private void personInPlanEinfgenToolStripMenuItem_Click(object sender, EventArgs e)
        {
            GUDialogBox dlg = new GUDialogBox(new AddPersonToPlan(PlanID), "Person hinzufgen");
			Gencode.Common.ActionLogger.Logger.LoggMessage(Gencode.Common.ActionLogger.MessageType.USER_EVENT, "Add person plan " + PlanID + " clicked");
            if (dlg.ShowDialog() == DialogResult.OK)
            {
                //Refresh
                InitializeBase();
                FormatGrid();
            }
        }

		private void leerzeileEinfgenToolStripMenuItem_Click(object sender, EventArgs e)
		{
			GUDialogBox dlg = new GUDialogBox(new AddLineToPlan(PlanID), "Leerzeile hinzufgen");
			Gencode.Common.ActionLogger.Logger.LoggMessage(Gencode.Common.ActionLogger.MessageType.USER_EVENT, "Add empty row to plan " + PlanID + " clicked");
			if (dlg.ShowDialog() == DialogResult.OK)
			{
				//Refresh
				InitializeBase();
				FormatGrid();
			}
		}

		private void termineBearbeitenToolStripMenuItem_Click(object sender, EventArgs e)
		{
			// * TODO:
			// * Termine sollten nicht plangebunden sein
			//Form term = new TerminDlg(5, lblVon.Text);
			//term.ShowDialog();

			Gencode.Common.ActionLogger.Logger.LoggMessage(Gencode.Common.ActionLogger.MessageType.USER_EVENT, "Add termin to plan " + PlanID + " clicked");
			Form term = new TerminAddDlg(PlanID);
			term.ShowDialog();
		}

		private void saldoAnpassenToolStripMenuItem1_Click(object sender, EventArgs e)
		{
			Gencode.Common.ActionLogger.Logger.LoggMessage(Gencode.Common.ActionLogger.MessageType.USER_EVENT, "Edit saldo in plan " + PlanID + " clicked");
			GUDialogBox dlg = new GUDialogBox(new SaldoAnpassenDlg(PlanID), "Saldo anpassen");
			dlg.ShowDialog();
		}

		private void druckenToolStripMenuItem2_Click(object sender, EventArgs e)
		{
			GUDialogBox selection = new GUDialogBox(new Personalplaner.Modules.Dienstplan.Print.AbteilungPrintSelect(PlanID), "Druckauswahl - " + PlanData.GetPlanName(PlanID));
			if (selection.ShowDialog() == DialogResult.OK)
			{
				Personalplaner.Modules.Dienstplan.Print.AbteilungPrintRetVal retVal = selection.ReturnValue as Personalplaner.Modules.Dienstplan.Print.AbteilungPrintRetVal;

				try
				{
					if (retVal.Copies > 0)
					{
                        Personalplaner.App.Util.SetProgress(5);

						PreparePrintDocuments();

                        Personalplaner.App.Util.SetProgress(25);

						_gridPrintDocument.AbteilungID = retVal.AbteilungID;
                        _gridPrintDocument.Preview = retVal.Preview;
						_gridPrintDocument.CreatePrinteableGrids();

                        Personalplaner.App.Util.SetProgress(45);
						
						_printerClass = new Personalplaner.Modules.Dienstplan.Print.PlanGridPrinterClass(printDocument, _gridPrintDocument.Grids);

                        Personalplaner.App.Util.SetProgress(65);

						printDocument.PrinterSettings.Copies = (short)retVal.Copies;

						if (!retVal.UseDefaultPrinter)
						{
							printDocument.Print();
							//printPreviewDialog1.Document = printDocument1;
							//printPreviewDialog1.ShowDialog();
						}
						else
						{
							planPrintDialog.Document = printDocument;
							if (planPrintDialog.ShowDialog() == DialogResult.OK)
								printDocument.Print();
						}
					}
				}
				catch (Exception ex)
                {
                    Gencode.Common.Util.ErrorHandler.LoggError("Error while printig", "Plan.saldoAnpassenToolStripMenuItem1_Click", ex.Message);
                    Gencode.Common.ActionLogger.Logger.LoggMessage(Gencode.Common.ActionLogger.MessageType.ERROR, "Error while printing. See Errorlog for more info");
                }
			}
		}

		private void vorschauToolStripMenuItem1_Click(object sender, EventArgs e)
		{
            Personalplaner.App.Util.SetProgress(5);
			PreparePrintDocuments();

            Personalplaner.App.Util.SetProgress(25);

			_gridPrintDocument.CreatePrinteableGrids();
			
            Personalplaner.App.Util.SetProgress(45);

			_printerClass = new Personalplaner.Modules.Dienstplan.Print.PlanGridPrinterClass(printDocument, _gridPrintDocument.Grids);

            Personalplaner.App.Util.SetProgress(65);

			printPreviewDialog.Document = printDocument;
			printPreviewDialog.ShowDialog();
		}

		private void diensteGenerierenToolStripMenuItem1_Click(object sender, EventArgs e)
		{
			int schemaid = PlanData.GetPlanSchemaID(PlanID);
			if (PlanData.GetDienstCountInSchema(schemaid) > 0)
			{
				if (MessageBox.Show(this, "Es werden alle Dienste die noch nicht Eingeteilt wurden automatisch mit einem Zufallsgenerator eingeteilt.\n" +
					"\n"+
					"ACHTUNG!\n" +
					"Freitage und Dienste die fix eingeteilt werden mssen, sollten vor der automatischen Einteilung erfasst werden.",
					"Automatische Diensteinteilung", MessageBoxButtons.OKCancel, MessageBoxIcon.Warning) == DialogResult.OK)
				{
					Personalplaner.Modules.Dienstplan.App.PlanAutomation.ScedulePlan(PlanID);
					PlanAktualisieren();
				}
			}
			else
				MessageBox.Show("Dem Schema sind kein Dienste zugewiesen.\n\nDienste knnen im Men unter Stammdaten \\ Planschemas - Dienste zuweisen\ndem Schema zugewiesen werden.", "Keine Dienste", MessageBoxButtons.OK);
		}

        #endregion

        /// <summary>
        /// Drag and Drop implemented acording to the example of Frank Dzaebel
        /// http://dzaebel.net/DataGridViewDragDrop.htm
        /// </summary>
        #region Drag & Drop

        Rectangle _dragBoxFromMouseDown;
        int _rowIndexStartDragDrop;
        int _rowIndexEndDragDrop;

        private void gridMainPlan_MouseMove(object sender, MouseEventArgs e)
        {
            //rows can only be moved if cursor is on column 1
            //set cursor to hand when on column 1 to show it is moveable
            if (gridMainPlan.HitTest(e.X, e.Y).ColumnIndex == 0)
                this.Cursor = Cursors.Hand;
            else
                this.Cursor = Cursors.Default;

            if ((e.Button & MouseButtons.Left) == MouseButtons.Left)
            {
                // If the mouse moves outside the rectangle, start the drag.
                if (_dragBoxFromMouseDown != Rectangle.Empty &&
                    !_dragBoxFromMouseDown.Contains(e.X, e.Y))
                {
                    // Proceed with the drag and drop, passing in the list item. 
                    DragDropEffects dropEffect = gridMainPlan.DoDragDrop(
                      gridMainPlan.Rows[_rowIndexStartDragDrop], DragDropEffects.Move);
                }
            }
        }

        private void gridMainPlan_MouseDown(object sender, MouseEventArgs e)
        {
            if (gridMainPlan.HitTest(e.X, e.Y).ColumnIndex == 0 &&
                GridData[gridMainPlan.HitTest(e.X, e.Y).RowIndex].PrsID != 0 &&
                GridData[gridMainPlan.HitTest(e.X, e.Y).RowIndex].PrsID != -2)
		{
                // Get the index of the item the mouse is below.
                _rowIndexStartDragDrop = gridMainPlan.HitTest(e.X, e.Y).RowIndex;
		}
            else
                _rowIndexStartDragDrop = -1;

            if (_rowIndexStartDragDrop != -1)
        {
                // Remember the point where the mouse down occurred. 
                // The DragSize indicates the size that the mouse can move 
                // before a drag event should be started. 
                Size dragSize = SystemInformation.DragSize;

                // Create a rectangle using the DragSize, with the mouse position being
                // at the center of the rectangle.
                _dragBoxFromMouseDown = new Rectangle(
                  new Point(e.X - (dragSize.Width / 2), e.Y - (dragSize.Height / 2)), dragSize);
            }
            else
                // Reset the rectangle if the mouse is not over an item in the ListBox.
                _dragBoxFromMouseDown = Rectangle.Empty;

        }

        private void gridMainPlan_DragDrop(object sender, DragEventArgs e)
        {
            // The mouse locations are relative to the screen, so they must be 
            // converted to client coordinates.
            Point clientPoint = gridMainPlan.PointToClient(new Point(e.X, e.Y));

            // Get the row index of the item the mouse is below. 
            _rowIndexEndDragDrop =
              gridMainPlan.HitTest(clientPoint.X, clientPoint.Y).RowIndex;

            // If the drag operation was a move then remove and insert the row.
            if (e.Effect == DragDropEffects.Move)
            {
                if (GridData[_rowIndexStartDragDrop].AbteilungID == GridData[_rowIndexEndDragDrop].AbteilungID &&
                    GridData[_rowIndexEndDragDrop].PrsID != 0 &&
                    GridData[_rowIndexEndDragDrop].PrsID != -2)
		{

                    GridData.RepositionRow(_rowIndexStartDragDrop, _rowIndexEndDragDrop);
                    SetRowColors(_rowIndexEndDragDrop);
                }
            }
		}

        private void gridMainPlan_DragOver(object sender, DragEventArgs e)
		{
            e.Effect = DragDropEffects.Move;
		}

        #endregion
    }
}
