using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace Personalplaner.Modules.Personoverview
{
    [ToolboxItem(false)]
    public partial class FerienView : UserControl, IPersonDetail
    {
        List<IPersonRow> row = null;
        DataTable dt = null;
        int prsID;

        public FerienView(int prsid)
        {
            InitializeComponent();
			this.prsID = prsid;

            SetData(Verwaltung.YearID);
        }

		public void SetData(int yearid)
        {
			dt = new DataTable("1");

			DFerienGenerator ferienGenerator = new DFerienGenerator(prsID, yearid);
            row = ferienGenerator.Row;

            ferienGrid = GPersonGridGenerator.CreateGrid(ferienGrid, dt, GPersonGridGenerator.GRIDTYPE.Ferien, row);
            addSaldo();

			for (int i = 0; i < ferienGrid.Columns.Count; i++)
			{
				if (i < 1)
					ferienGrid.Columns[i].DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleLeft;
				else
					ferienGrid.Columns[i].DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleCenter;
			}

			setGridSize();
        }

		private void setGridSize()
        {
			if (ferienGrid.Rows.Count > 0)
			{
				//Height
				ferienGrid.Height = ((ferienGrid.RowCount * ferienGrid.Rows[0].Height) + ferienGrid.ColumnHeadersHeight) + 3; ;

				int width = 0;
				int i = 0;
				while (i < ferienGrid.ColumnCount)
				{
					width += ferienGrid.Columns[i].Width;
					i++;
				}

				ferienGrid.Width = width + 3;
			}
        }
        
        private void addSaldo()
        {
            dt.Columns.Add("Saldo");

            ferienGrid.Columns[ferienGrid.ColumnCount - 1].AutoSizeMode = DataGridViewAutoSizeColumnMode.AllCells;
            ferienGrid.Columns[ferienGrid.ColumnCount - 1].SortMode = DataGridViewColumnSortMode.NotSortable;

            PFerienRow ferien = null;
            for (int r = 0; r < row.Count; r++)
            {
                if (row[r] is PFerienRow)
                {
                    ferien = (PFerienRow)row[r];
                    ferienGrid[ferienGrid.ColumnCount - 1, r].Value = ferien.Saldo;
                }
            }
        }

        public void SetCellColor()
        {
            GGridColor.SetCellColor(ferienGrid, row);
        }

        private void CPersonDetailFerien_SizeChanged(object sender, EventArgs e)
        {
            setGridSize();
        }

        private void CPersonDetailFerien_Load(object sender, EventArgs e)
        {
            SetCellColor();
			setGridSize();
        }

        public DataGridView Grid
        {
            get { return ferienGrid; }
        }

		public string Title
		{
			get { return "Ferienbersicht"; }
		}

		#region IPersonDetail Members

		/// <summary>
		/// Resets the grid with the given YearID
		/// </summary>
		/// <param name="year">YearID</param>
		public void ResetGrid(int year)
		{
			SetData(year);
			SetCellColor();
		}

		#endregion
	}
}
