using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Gencode.Gengui.Display;

namespace Personalplaner.Modules.Personoverview
{
	public enum DETAILANZEIGE { Default, Monatsbersicht, Jahresplan, Ferien }

    [ToolboxItem(false)]
	public partial class OverviewBaseView : DisplayableView
    {
		///
		/// Indicates if the current view is being utilized in the VS.NET IDE or not.
		///
		public new bool DesignMode
		{
			get
			{
				return (System.Diagnostics.Process.GetCurrentProcess().ProcessName == "devenv");
			}
		}

		ContentPerson PartnerContent
		{
			get
			{
				return PartnerView as ContentPerson;
			}
		}

        List<PersonOverview> personList = new List<PersonOverview>();

        public OverviewBaseView()
        {
            InitializeComponent();

			if (!this.DesignMode)
			{
				SetData();
			}
        }

        private static void SetData()
        {
            //Global Data
            GPerson.SetData();
        }

        public void CloseTab()
        {
            personList.RemoveAt(tabControl1.SelectedIndex);
            tabControl1.TabPages.RemoveAt(tabControl1.SelectedIndex);

            //reset header text
            if (tabControl1.TabCount == 0)
                SetHeaderText(string.Empty, string.Empty);
        }

        public void ChangePerson(int prsid, string person)
        {
            int listTabIndex = GetPersonIndex(prsid);
            if (listTabIndex >= 0)
            {
                tabControl1.SelectedIndex = listTabIndex;

                string anzeige = string.Empty;
                if (personList[listTabIndex].CurrentView != DETAILANZEIGE.Default)
                    anzeige = personList[listTabIndex].CurrentView.ToString();

                SetHeaderText(personList[listTabIndex].PersonName, anzeige);
            }
            else
            {
                addTabPage(prsid, person);
                SetHeaderText(person, string.Empty);
            }
        }

        private void addTabPage(int prsid, string person)
        {
            personList.Add(new PersonOverview(prsid, person, this));

            //Add tab page
            //TabPage page = new TabPage("");
            Gencode.Gengui.CustomControls.GUTabPage page = new Gencode.Gengui.CustomControls.GUTabPage("");
            tabControl1.TabPages.Add(page);
            tabControl1.SelectedIndex = tabControl1.TabPages.Count - 1;

            //Add control to Tab
            personList[personList.Count - 1].Dock = DockStyle.Fill;
            tabControl1.TabPages[tabControl1.TabPages.Count - 1].Controls.Add(personList[personList.Count - 1]);
            //Tab Name
            tabControl1.TabPages[tabControl1.TabPages.Count - 1].Text = person;// Data.Person.PersonData[comboPersonIndex].Vorname + " " + Data.Person.PersonData[comboPersonIndex].Nachname;
        }

        private int GetPersonIndex(int prsid)// comboPersonIndex)
        {
            int index = -1;
            for (int i = 0; i < personList.Count; i++)
            {
                if (personList[i].PrsID == prsid)//comboPersonIndex)
                    index = i;
            }
            return index;
        }

		public void SetView(DETAILANZEIGE view)
		{
			personList[tabControl1.SelectedIndex].SetView(view);

            string anzeige = string.Empty;
            if (personList[tabControl1.SelectedIndex].CurrentView != DETAILANZEIGE.Default)
                anzeige = personList[tabControl1.SelectedIndex].CurrentView.ToString();

            SetHeaderText(personList[tabControl1.SelectedIndex].PersonName, anzeige);
		}

        private void tabControl1_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (tabControl1.SelectedIndex >= 0)
            {
                PartnerContent.SetComboPersonIndex(personList[tabControl1.SelectedIndex].PrsID);
            }
            else
            {
                PartnerContent.SetComboPersonIndex(-1);
            }
        }

        public void SetHeaderText(string person, string title)
        {
            string text = "Personbersicht";

            if (!string.IsNullOrEmpty(person))
            {
                text += " - " + person;
                if (!string.IsNullOrEmpty(title))
                    text += ": " + title;
            }

            lblTitle.Text = text;
        }
    }
}
