using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Gencode.PrsUtil.Print;
using Gencode.PrsCommon.App;

namespace Personalplaner.Modules.Personoverview
{
    [ToolboxItem(false)]
    public partial class PersonOverview : UserControl
    {
        OverviewBaseView mainPerson;
        DPersonVerwaltung verwaltung = new DPersonVerwaltung();
        //list for all detail masks
        List<IPersonDetail> personDetail = new List<IPersonDetail>();
		PrinterClass PrintC;
		DataView m_yearDV;

        int prsid;
		string personName;
        
        public PersonOverview(int prsid, string personName, OverviewBaseView mainPerson)
        {
            InitializeComponent();
            this.mainPerson = mainPerson;
            this.prsid = prsid;
			this.personName = personName;

			setComboDetail();

            SetData();
        }

        public void SetData()
        {
			m_yearDV = AppData.ComboDataGenerator.FillComboOpenYears(ref comboYear);

			int index = 0;
			for (int i = 0; i < m_yearDV.Count; i++)
			{
				int yearID = int.Parse(m_yearDV[i].Row[1].ToString());
				if (yearID == Verwaltung.YearID)
				{
					index = i;
					i = m_yearDV.Count;
				}
			}
			comboYear.SelectedIndex = index;
            setDetailMasks();
        }

        private void setComboDetail()
        {
            if (comboDetail.Items.Count > 0)
                comboDetail.Items.Clear();

            int i = 1;
            while (i <= GPerson.DetailCount)
            {
                comboDetail.Items.Add(Option.StringValue("PersonDetail", i));
                i++;
            }
        }

        private void setDetailMasks()
        {
            DiversView pdd = new DiversView(prsid);
            splitMainContainer.Panel2.Controls.Add(pdd);
            personDetail.Add(pdd);

            JahresplanView pdj = new JahresplanView(prsid);
            splitMainContainer.Panel2.Controls.Add(pdj);
            personDetail.Add(pdj);

            FerienView pdf = new FerienView(prsid);
            splitMainContainer.Panel2.Controls.Add(pdf);
            personDetail.Add(pdf);

            for (int i = 0; i < personDetail.Count; i++)
            {
                personDetail[i].Dock = DockStyle.Fill;
                personDetail[i].Visible = false;
            }
        }

        #region Properties
        public int PrsID
        {
            get { return prsid; }
            set { prsid = value; }
        }

        public string PersonName
        {
            get
            {
                return personName;
            }
        }

        public DETAILANZEIGE CurrentView
        {
            get
            {
                return verwaltung.DetailAnzeige;
            }
        }

        #endregion

        private void comboDetail_SelectedIndexChanged(object sender, EventArgs e)
        {
            switch (comboDetail.SelectedIndex)
            {
                case 0:
                    verwaltung.DetailAnzeige = DETAILANZEIGE.Monatsbersicht;
					btnSelection.Visible = false;
                    break;
                case 1:
                    verwaltung.DetailAnzeige = DETAILANZEIGE.Jahresplan;
					btnSelection.Visible = true;
                    break;
                case 2:
                    verwaltung.DetailAnzeige = DETAILANZEIGE.Ferien;
					btnSelection.Visible = false;
                    break;

                default: //bei -1
                    verwaltung.DetailAnzeige = DETAILANZEIGE.Default;
					btnSelection.Visible = false;
                    break;
            }
            setVisibility();
        }

		public void SetView(DETAILANZEIGE view)
		{
			switch (view)
			{
				case DETAILANZEIGE.Monatsbersicht:
					comboDetail.SelectedIndex = 0;
					break;
				case DETAILANZEIGE.Jahresplan:
					comboDetail.SelectedIndex = 1;
					break;
				case DETAILANZEIGE.Ferien:
					comboDetail.SelectedIndex = 2;
					break;
				case DETAILANZEIGE.Default:
					comboDetail.SelectedIndex = -1;
					break;
			}
		}

        private void setVisibility()
        {
            for (int i = 0; i < personDetail.Count; i++)
            {
                personDetail[i].Visible = false;
                //CPersonDetailMonat c = new CPersonDetailMonat();
                //c.Dock
            }

            switch (verwaltung.DetailAnzeige)
            {
                case DETAILANZEIGE.Monatsbersicht:
                    personDetail[0].Visible = true;
					comboYear.Visible = false;
                    //CPersonDetailJahresplan jp = (CPersonDetailJahresplan)personDetail[0];
                    //jp.SetCellColor();
                    break;
                case DETAILANZEIGE.Jahresplan:
                    personDetail[1].Visible = true;
					comboYear.Visible = true;
                    break;
                case DETAILANZEIGE.Ferien:
                    personDetail[2].Visible = true;
					comboYear.Visible = true;
                    break;
                case DETAILANZEIGE.Default:
                    break;
            }
        }

		/// <summary>
		/// Get the grid that's currently shown
		/// </summary>
		/// <returns>Datagridview</returns>
        private DataGridView GetGrid()
        {
            DataGridView grid = null;

            switch (verwaltung.DetailAnzeige)
            {
                case DETAILANZEIGE.Monatsbersicht:
                    grid = personDetail[0].Grid;
                    break;
                case DETAILANZEIGE.Jahresplan:
                    grid = personDetail[1].Grid;
                    break;
                case DETAILANZEIGE.Ferien:
                    grid = personDetail[2].Grid;
                    break;
                case DETAILANZEIGE.Default:
                    grid = null;
                    break;
            }

            return grid;
        }

		/// <summary>
		/// Returns name of Person and page title currently shown
		/// </summary>
		/// <returns></returns>
		private string GetGridTitle()
		{
			string text = "";

			switch (verwaltung.DetailAnzeige)
			{
				case DETAILANZEIGE.Monatsbersicht:
					text = personDetail[0].Title;
					break;
				case DETAILANZEIGE.Jahresplan:
					text = personDetail[1].Title;
					break;
				case DETAILANZEIGE.Ferien:
					text = personDetail[2].Title;
					break;
				case DETAILANZEIGE.Default:
					text = "";
					break;
			}

			return text;
		}

        private void schliessenToolStripMenuItem_Click(object sender, EventArgs e)
        {
            mainPerson.CloseTab();
        }

        private void druckenToolStripMenuItem2_Click(object sender, EventArgs e)
        {
            printDocument1.DefaultPageSettings.Landscape = true;
            printDocument1.DefaultPageSettings.Margins.Left = 40;
            printDocument1.DefaultPageSettings.Margins.Top = 60;
            printPreviewDialog1.Document = printDocument1;

			//Gencode.PrsUtil.Print.GridPrint.GridPreview gridView = new Gencode.PrsUtil.Print.GridPrint.GridPreview();
			//gridView.Grid = GetGrid();

			//if (gridView.Grid != null && gridView.Grid.Columns.Count > 0)
			//{
			//    gridView.Grid.Font = new Font("Microsoft Sans Serif", 7);
			//    gridView.Grid.AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.AllCells;

			//    gridView.Render();
			//    //create printerclass object
			//    PrintC = new PrinterClass(printDocument1, gridView.Grid);

			//    printDialog1.Document = printDocument1;
			//    if (printDialog1.ShowDialog() == DialogResult.OK)
			//        printDocument1.Print();
			//}
			DataGridView pgv = GetGrid();

			if (pgv != null && pgv.Columns.Count > 0)
			{
				pgv.Font = new Font("Microsoft Sans Serif", 7);
				pgv.AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.AllCells;
				//create printerclass object
				PrintC = new PrinterClass(printDocument1, pgv);

				printDialog1.Document = printDocument1;
				if (printDialog1.ShowDialog() == DialogResult.OK)
					printDocument1.Print();
			}
        }

        private void vorschauToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            printDocument1.DefaultPageSettings.Landscape = true;
            printDocument1.DefaultPageSettings.Margins.Left = 40;
            printDocument1.DefaultPageSettings.Margins.Top = 60; //40;

			//Gencode.PrsUtil.Print.GridPrint.GridPreview gridView = new Gencode.PrsUtil.Print.GridPrint.GridPreview();
			//gridView.Grid = GetGrid();

			//if (gridView.Grid != null && gridView.Grid.Columns.Count > 0)
			//{
			//    gridView.Grid.Font = new Font("Microsoft Sans Serif", 7);
			//    gridView.Grid.AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.AllCells;

			//    gridView.Render();
			//    //create printerclass object
			//    PrintC = new PrinterClass(printDocument1, gridView.Grid);

			//    printPreviewDialog1.Document = printDocument1;
			//    printPreviewDialog1.ShowDialog();
			//}
			DataGridView pgv = GetGrid();

			if (pgv != null && pgv.Columns.Count > 0)
			{
				pgv.Font = new Font("Microsoft Sans Serif", 7);
				pgv.AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.AllCells;
				//create printerclass object
				PrintC = new PrinterClass(printDocument1, pgv/*gridMainPlan*/);

				printPreviewDialog1.ShowDialog();
			}
        }

		private void printDocument1_PrintPage(object sender, System.Drawing.Printing.PrintPageEventArgs e)
		{
			// Add Title to printout
			String printText = GetGridTitle();
			Font printFont = new Font("Microsoft Sans Serif", 14);
			e.Graphics.DrawString(printText, printFont, Brushes.Black, 450, 20);

			printText = personName;
			printFont = new Font("Microsoft Sans Serif", 10);
			e.Graphics.DrawString(printText, printFont, Brushes.Black, 100, 25);
			

			bool morepages = PrintC.Print(e.Graphics);
			if (morepages)
			{
				e.HasMorePages = true;
			}
		}

		private void comboYear_SelectedIndexChanged(object sender, EventArgs e)
		{
			foreach (IPersonDetail det in personDetail)
			{
				det.ResetGrid(int.Parse(m_yearDV[comboYear.SelectedIndex].Row[1].ToString()));
			}
		}

        private void btnSelection_Click(object sender, EventArgs e)
        {
            switch (verwaltung.DetailAnzeige)
            {
                case DETAILANZEIGE.Monatsbersicht:
                    break;
                case DETAILANZEIGE.Jahresplan:
                    Gencode.Gengui.Forms.GUDialogBox selection = new Gencode.Gengui.Forms.GUDialogBox(new Gui.JahresplanSelection(personDetail[1]), "Selektion");
                    selection.ShowDialog();
                    break;
                case DETAILANZEIGE.Ferien:
                    break;
                case DETAILANZEIGE.Default:
                    break;
            }
        }
    }
}
