using System;
using System.Collections.Generic;
using System.Windows.Forms;

namespace Personalplaner
{
    static class Program
    {
        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);

            // check if the connection information is allready stored and is valid
			var anmeldng = new Gencode.Gengui.Logon.Logon();
			anmeldng.ConnectionTestTable = "PrsVerwaltung";
			bool anmeldungOK = anmeldng.TestConnection();
			if (!anmeldungOK)
			{
				if (anmeldng.ShowDialog() == DialogResult.OK)
				{
					anmeldungOK = true;
				}
			}

			if (anmeldungOK)
			{
				int v = Gencode.PrsCommon.Data.Common.CommonData.CompareDatabaseVersion(new Version(Application.ProductVersion));
				// if the db version is smaller than the application version
                // the user has to start the sqlwizz to update the database
				if (v < 0)
				{
					switch (v)
					{
						case -1:
                            MessageBox.Show("Die Datenbankversion ist lter als die Version des WF Personalplaners." +
                                "\n\nBitte starten Sie den SQLWizz fr die aktualisierung der Datenbank oder benachrichtigen ihren Systemadministrator." +
                                "\n\nDatenbank Version: " + Gencode.PrsCommon.Data.Common.CommonData.DatabaseVersion +
                                "\nPersonalplaner Version: " + Application.ProductVersion
                                , "Consistency check"
                                , MessageBoxButtons.OK
                                , MessageBoxIcon.Exclamation);
							break;
						case -2:
							MessageBox.Show("Es existiert keine Datenbank.\n\nBitte starten Sie den SQLWizz fr die erstellung der Datenbank.", "Consistency check", MessageBoxButtons.OK);
							break;
					}
#if DEBUG // start application anyway if is in debugmode
#else
					return;
#endif
				}

				if(Gencode.PrsCommon.Data.Plan.PlanData.CheckIfOpenYearExists() == false)
				{
					// Wenn noch kein aktives Jahr vorhanden ist dann wird der wizard fr die basisdaten gestartet
					Gencode.PrsUtil.InitWizard.InitWizard wiz = new Gencode.PrsUtil.InitWizard.InitWizard();
					if (wiz.ShowDialog() == DialogResult.Cancel)
					{
						return;
					}
					else
					{
						var data = wiz.DataObjects.GetType(typeof(Gencode.PrsUtil.InitWizard.InitData)) as Gencode.PrsUtil.InitWizard.InitData;
						Properties.Settings.Default.User = data.User;
//#if DEBUG
						
//#else
						Properties.Settings.Default.Save();
//#endif
					}
				}

				/// Initialize
				if (!Verwaltung.Initialize())
				{
					Gencode.Common.Util.ErrorHandler.LoggError("initialization failed", "initializiation of default values", "could not initialize default values of Verwaltungs object.\nApplication was terminated.");
					Gencode.Common.ActionLogger.Logger.LoggMessage(Gencode.Common.ActionLogger.MessageType.SYSTEM_EVENT, "Application was terminated because default values in Verwaltungs object could not be initialized");
					Gencode.Common.Util.ErrorHandler.ShowError("Die Applikation wird beendet da Verwaltungsdaten nicht initialisiert werden konten.\n\nBitte schauen Sie in den Logdaten fr genauere Hinweise\nund kontaktieren Sie den Systemadministrator","Applikation wird beendet");
					return;
				}

                try
                {
                    /// Start main application
                    Application.Run(new MainForm());
                }
                catch (Exception e)
                {
                    MessageBox.Show("Es ist ein schwerer Fehler aufgetreten.\n\n" + e.Message, "Fehler", MessageBoxButtons.OK, MessageBoxIcon.Error);

                    Gencode.Common.Util.ErrorHandler.LoggError("common exception in the application", e);
                    Gencode.Common.ActionLogger.Logger.LoggMessage(Gencode.Common.ActionLogger.MessageType.ERROR, e.Message);
                }
			}
        }
    }
}