using System;
using System.Drawing;
using System.Collections;
using System.Data;
using System.ComponentModel;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Printing;

namespace Gencode.PrsUtil.Print
{
	/// <summary>
	/// Represents a Data Grid with advanced options
	/// </summary>
	public class Grid
	{
		
		private Font fontGridFont;
		private Font fontGridHeadersFont;
		
		public Color HeaderBackColor;
		public Color HeaderForeColor;
		public Color LineColor;
		public Color ForeColor;
		public Color BackColor;
		
		private int rows;
		private int columns;

		public Cell[,] Cells;
		
		public Header[] Headers;

		public Header this[int Column]
		{
			get {return Headers[columns];}
			
		}

		/// <summary>
		/// The Font of the text in the cells
		/// </summary>
		public Font Font
		{
			get {return fontGridFont;}
			set {fontGridFont = value;}
		}

		/// <summary>
		/// The Font of the text in the header cells
		/// </summary>
		public Font HeaderFont
		{
			get {return fontGridHeadersFont;}
			set {fontGridHeadersFont = value;}
		}

		public int Rows
		{
			get {return rows;}
		}

		public int Columns
		{
			get {return columns;}
		}

		/// <summary>
		/// Gets or Sets a Cell
		/// </summary>
		public Cell this[int RowNumber,int ColumnNumber]
		{
			get 
			{
				//check to see if the cell exists
				if (RowNumber>=0 && ColumnNumber>=0 && RowNumber<=Cells.GetUpperBound(0) && ColumnNumber<=Cells.GetUpperBound(1))
				{
					//return found cell
					return Cells[RowNumber,ColumnNumber];
				}
				else
				{
					//error - no cell found
					return null;
					//throw new NoCellException
				}
			}
			set
			{
				//Check the number of Cell to exist
				if (RowNumber>=0 && ColumnNumber>=0 && RowNumber<=Cells.GetUpperBound(0) && ColumnNumber<=Cells.GetUpperBound(1))
				{
					//set value
					Cells[RowNumber,ColumnNumber]=value;
				}
				else
				{
					//throw new NoCellException
				}
			}
		}

		/// <summary>
		/// Set a new value for a cell
		/// </summary>
		public Cell this[Cell cell]
		{
			set {Cells[cell.RowNumber,cell.ColumnNumber]=value;}
		}

		public Grid(DataGridView TheGrid)
		{
			try
			{
				//get the Data in the grid
				DataTable TableGrid = null;

                if (TheGrid.DataSource.GetType() == typeof(DataView))
                {
                    DataView ViewGrid = (DataView)TheGrid.DataSource;
                    TableGrid = ViewGrid.Table;
					
                }
                else
                {
					TableGrid = (DataTable)TheGrid.DataSource;
                }
				//set number of rows
				rows = TableGrid.Rows.Count;
			
				//set number of columns
				//first check if the grid has tablestyle and columnstyle
				
				//check for table styles
				if (TheGrid.Columns.Count == 0)//TheGrid.TableStyles.Count==0)
				{
					//create table style and column style
					CreateColumnStyles(TheGrid,TableGrid);
				}
                // TODO: CHECK IF IT IS NOT NEEDED
                //else
                //{
                //    //create column styles if there are none
                //    if (TheGrid.TableStyles[TableGrid.TableName].GridColumnStyles.Count==0)
                //        CreateColumnStyles(TheGrid,TableGrid);
                //}
			
				//set number of columns
				columns = TheGrid.Columns.Count;// .TableStyles[TableGrid.TableName].GridColumnStyles.Count;

				

				

				Cells = new Cell[Rows,Columns];

				//Copy Cells
				for (int i=0;i<Rows;i++)
				{
					for (int j=0;j<Columns;j++)
					{
                        //Cells[i, j] = new Cell(i, j, TheGrid.Font, TheGrid.TableStyles[TableGrid.TableName].GridColumnStyles[j].Alignment, TheGrid.GetCellBounds(i, j), TheGrid[i, j].ToString());
                        //Cells[i, j] = new Cell(i, j, TheGrid.Font, (HorizontalAlignment)TheGrid[j,i].Style.Alignment, TheGrid[j,i].ContentBounds /*.GetCellBounds(i, j)*/, TheGrid[i, j].ToString());

                        //Cells[i, j] = new Cell(i, j, TheGrid.Font, (HorizontalAlignment)TheGrid[j, i].Style.Alignment, TheGrid.GetCellDisplayRectangle(j, i, false), TheGrid[j, i].Value.ToString(), TheGrid[j, i].Style.ForeColor, TheGrid[j, i].Style.BackColor);
						
						Cells[i, j] = new Cell(i, j, TheGrid.Font, (DataGridViewContentAlignment)TheGrid[j, i].Style.Alignment, new RectangleF(TheGrid[j, i].GetContentBounds(i).Location, TheGrid[j, i].Size), TheGrid[j, i].Value.ToString(), TheGrid[j, i].Style.ForeColor, TheGrid[j, i].Style.BackColor);
					}
				}

				//init number of columns to headers
				Headers = new Header[Columns];
				SetHeaders(TheGrid,TableGrid);
				
				//define grid colors
				SetColors(TheGrid);
			}
			catch (Exception e)
			{
                //Console.WriteLine(e.Message);
				Gencode.Common.Util.ErrorHandler.LoggError("Grid Error", "Grid.Grid(DataGridView TheGrid)", e.Message);
			}
		}

		private void CreateColumnStyles(DataGridView TheGrid,DataTable TableGrid)
		{
			// Define new table style.
            //DataGridTableStyle tableStyle = new DataGridTableStyle();
            //DataGridViewTextBoxColumn taleCol = new DataGridViewTextBoxColumn();
			
			Graphics g = TheGrid.CreateGraphics();

			try
			{
            
				// Clear any existing table styles.
                TheGrid.Columns.Clear();// TableStyles.Clear();
  
				// Use mapping name that is defined in the data source.
                //tableStyle.MappingName = TableGrid.TableName;
  
				// Now create the column styles within the table style.
				DataGridViewTextBoxColumn columnStyle;
				
				
				for (int iCurrCol = 0; iCurrCol < TableGrid.Columns.Count; 
					iCurrCol++)
				{
					DataColumn dataColumn = TableGrid.Columns[iCurrCol];
    
					columnStyle = new DataGridViewTextBoxColumn();

					
					columnStyle.HeaderText = dataColumn.ColumnName;
					columnStyle.Name = dataColumn.ColumnName;


                    columnStyle.Width = TheGrid.Columns[iCurrCol].Width;// GetCellBounds(0, iCurrCol).Width;  
                    
                    // TODO: TEST IF NO HEIGHT S NEEDED
					//columnStyle.TextBox.Height = (int)g.MeasureString(columnStyle.HeaderText,TheGrid.ColumnHeadersDefaultCellStyle.Font/*HeaderFont*/).Height + 10;
                    
					
					// Add the new column style to the table style.
                    //tableStyle.GridColumnStyles.Add(columnStyle);

                    TheGrid.Columns.Add(columnStyle);

				}    

				
				// Add the new table style to the data grid.
			
                //TheGrid.TableStyles.Add(tableStyle);
			}
			catch(Exception e)
			{
				Gencode.Common.Util.ErrorHandler.LoggError("Grid Error", "Grid.CreateColumnStyles(DataGridView TheGrid,DataTable TableGrid)", e.Message);
			}
			finally
			{
				g.Dispose();
			}
			
		}
		private void SetHeaders(DataGridView TheGrid,DataTable TableGrid)
		{
			try
			{
				//Check if there are styles
                //if (TheGrid.TableStyles.Count>0)
                //{
                if (TheGrid.Columns.Count>0)//.TableStyles[TableGrid.TableName].GridColumnStyles.Count>0)	 
                {
                    for (int i = 0; i <= Headers.GetUpperBound(0); i++)
                    {
                        //Known bug - when there are no rows headers are not displayed properly
                        DataGridViewColumnHeaderCell headerCell = (DataGridViewColumnHeaderCell)TheGrid.Columns[i].HeaderCell;
                        DataGridViewCellStyle columnStyle = (DataGridViewCellStyle)TheGrid.Columns[i].HeaderCell.Style;//.DefaultCellStyle;// ColumnHeadersDefaultCellStyle;
                        //DataGridViewTextBoxColumn columnStyle = (DataGridViewTextBoxColumn)TheGrid.ColumnHeadersDefaultCellStyle; //.TableStyles[TableGrid.TableName].GridColumnStyles[i];
                        if (Cells.GetUpperBound(0) > 0)
                        {

							Headers[i] = new Header(i, TheGrid.ColumnHeadersDefaultCellStyle.Font, (DataGridViewContentAlignment)columnStyle.Alignment, new RectangleF(Cells[0, i].Location.X, headerCell.ContentBounds.Y, TheGrid.Columns[i].HeaderCell.Size.Width, TheGrid.Columns[i].HeaderCell.Size.Height), TheGrid.Columns[i].HeaderText);
                            //Headers[i] = new Header(i, TheGrid.ColumnHeadersDefaultCellStyle.Font, (HorizontalAlignment)columnStyle.Alignment /*(HorizontalAlignment)headerCell.Style.Alignment*/, new RectangleF(Cells[0, i].Location.X, headerCell.ContentBounds.Y, TheGrid.Columns[i].HeaderCell.Size.Width, TheGrid.Columns[i].HeaderCell.Size.Height), TheGrid.Columns[i].HeaderText);
                        }
                        else
                        {
                            //headerCell.Value
							Headers[i] = new Header(i, TheGrid.ColumnHeadersDefaultCellStyle.Font, (DataGridViewContentAlignment)headerCell.Style.Alignment, headerCell.ContentBounds, TheGrid.Columns[i].HeaderText);
                            //Headers[i] = new Header(i, TheGrid.ColumnHeadersDefaultCellStyle.Font/*HeaderFont*/, (HorizontalAlignment)headerCell.Style.Alignment, headerCell.ContentBounds /*columnStyle.TextBox.Bounds*/, TheGrid.Columns[i].HeaderText/*columnStyle.HeaderText*/);
                        }
                    }

                }
                //}
			}
			catch (Exception ex)
			{
                //Console.WriteLine(ex.Message);
				Gencode.Common.Util.ErrorHandler.LoggError("Grid Error", "Grid.SetHeaders(DataGridView TheGrid,DataTable TableGrid)", ex.Message);
			}
		}

		private void SetColors(DataGridView TheGrid)
		{
            Color col = Color.FromName("Black");
            HeaderBackColor = TheGrid.ColumnHeadersDefaultCellStyle.BackColor;// HeaderBackColor;
            HeaderForeColor = TheGrid.ColumnHeadersDefaultCellStyle.ForeColor;// HeaderForeColor;
            LineColor = col;// TheGridGridColor;// .GridLineColor;
			ForeColor = TheGrid.ForeColor;
			BackColor = TheGrid.BackColor;
		}

	}
}
